;
;*******************************************************
;*******************************************************
;**                                                   **
;**  Program prezentacji obslugi USART w przerwaniach **
;**                                                   **
;**         Andrzej Pawluczuk                         **
;**                                                   **
;**         Mikrokontroler: ATMEGA8515                **
;**                                                   **
;*******************************************************
;*******************************************************
;
	.nolist
;*******************************************************
.include "m8515def.inc"
;*******************************************************
	.list
	.listmac
;-----------------------------------------------------------------------------
.macro	ldz
	ldi	zl,low(@0)
	ldi	zh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	ldx
	ldi	xl,low(@0)
	ldi	xh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	pushz
	push	zl
	push	zh
.endm
;-----------------------------------------------------------------------------
.macro	popz
	pop	zh
	pop	zl
.endm
;-----------------------------------------------------------------------------
.macro	pushf
	push	acc
	in	acc,sreg
	push	acc
.endm
;-----------------------------------------------------------------------------
.macro	popf
	pop	acc
	out	sreg,acc
	pop	acc
.endm
;-----------------------------------------------------------------------------
.macro	starg1
	ldi	zl,low(@0)			; @0 := Argument1 ; (32 bit)
	ldi	zh,high(@0)			;
	st	z+,Argument1LL			;
	st	z+,Argument1LH			;
	st	z+,Argument1HL			;
	st	z,Argument1HH			;
.endm
;	funkcja		wywolanie procedury 'starg1' zachowuje w pamici RAM
;			zawartosc 32-bitowego rejestru Argument1 (operacje arytmetyczne 32-bitowe)
;	uzycie		starg1	<nazwa 4-bajtowej zmiennej w RAM>
;	przyklad	starg1	LeftArgument
;-----------------------------------------------------------------------------
.macro	starg2
	ldi	zl,low(@0)			; @0 := Argument2 ; (32 bit)
	ldi	zh,high(@0)			;
	st	z+,Argument2LL			;
	st	z+,Argument2LH			;
	st	z+,Argument2HL			;
	st	z,Argument2HH			;
.endm
;	funkcja		wywolanie procedury 'starg2' zachowuje w pamici RAM
;			zawartosc 32-bitowego rejestru Argument2 (operacje arytmetyczne 32-bitowe)
;	uzycie		starg2	<nazwa 4-bajtowej zmiennej w RAM>
;	przyklad	starg2	RightArgument
;-----------------------------------------------------------------------------
.macro	loarg1
	ldi	zl,low(@0)			; Argument1 := @0 ; (32 bit)
	ldi	zh,high(@0)			;
	ld	Argument1LL,z+			;
	ld	Argument1LH,z+			;
	ld	Argument1HL,z+			;
	ld	Argument1HH,z			;
.endm
;	funkcja		wywolanie procedury 'loarg1' laduje z pamici RAM
;			zawartosc 32-bitowego rejestru Argument1 (operacje arytmetyczne 32-bitowe)
;	uzycie		loarg1	<nazwa 4-bajtowej zmiennej w RAM>
;	przyklad	loarg1	LeftArgument
;-----------------------------------------------------------------------------
.macro	loarg2
	ldi	zl,low(@0)			; Argument2 := @0 ; (32 bit)
	ldi	zh,high(@0)			;
	ld	Argument2LL,z+			;
	ld	Argument2LH,z+			;
	ld	Argument2HL,z+			;
	ld	Argument2HH,z			;
.endm
;	funkcja		wywolanie procedury 'loarg2' laduje z pamici RAM
;			zawartosc 32-bitowego rejestru Argument2 (operacje arytmetyczne 32-bitowe)
;	uzycie		loarg2	<nazwa 4-bajtowej zmiennej w RAM>
;	przyklad	loarg2	LeftArgument
;-----------------------------------------------------------------------------
.macro	incvar
	ldi	zl,low(@0)			; @0 := @0 + 1 ;
	ldi	zh,high(@0)			;
	ld	acc2,z				;
	inc	acc2				;
	st	z,acc2				;
.endm
;	funkcja		zwiekwsza o 1 zawartosc wskazanej zmiennej
;	uzycie		incvar	<nazwa 1-bajtowej zmiennej w RAM>
;	przyklad	incvar	TextLineInx
;-----------------------------------------------------------------------------
.macro	copyarg2toarg1
	mov	Argument1LL,Argument2LL		; Argument1 := Argument2 ;
	mov	Argument1LH,Argument2LH		;
	mov	Argument1HL,Argument2HL		;
	mov	Argument1HH,Argument2HH		;
.endm
;	funkcja		kopiuje zawartosc rejestru Argument2 do rejestru Argument1
;	przyklad	copyarg2toarg1
;-----------------------------------------------------------------------------
.macro	copyrestoarg1
	mov	Argument1LL,ResultLL		; Argument := Result ;
	mov	Argument1LH,ResultLH		;
	mov	Argument1HL,ResultHL		;
	mov	Argument1HH,ResultHH		;
.endm
;	funkcja		kopiuje zawartosc rejestru wynikowego Reslt do rejestru Argument1
;	przyklad	copyrestoarg1
;-----------------------------------------------------------------------------
.macro	clrarg1
	ldi	acc,0				; Argument1 := 0 ;
	mov	Argument1LL,acc			;
	mov	Argument1LH,acc			;
	mov	Argument1HL,acc			;
	mov	Argument1HH,acc			;
.endm
;-----------------------------------------------------------------------------
.def DivLoopCounter	= r1
.def Argument1LL	= r2
.def Argument1LH	= r3
.def Argument1HL	= r4
.def Argument1HH	= r5
.def Argument2LL	= r6
.def Argument2LH	= r7
.def Argument2HL	= r8
.def Argument2HH	= r9
.def WorkRegLL		= r10
.def WorkRegLH		= r11
.def WorkRegHL		= r12
.def WorkRegHH		= r13
.def acc		= r16
.def acc2		= r17
.def param		= r18
.def ResultLL		= r22
.def ResultLH		= r23
.def ResultHL		= r24
.def ResultHH		= r25
;-----------------------------------------------------------------------------
.equ		Stand_TIMSK	=	0b00000010
;	Timer/counter Interrupt Mask	  ||| | |
;					  ||| | +--- TOIE0 : Timer/Counter 0 Overflow Interrupt Enable
;					  ||| +----- TICIE1 : Timer/Counter1 Input Capture Interrupt Enable
;					  ||+------- OCIE1B : Timer/counter1 Output Compare B Match Interrupt Enable
;					  |+-------- OCE1A : Timer/counter1 Output Compare A Match Interrupt Enable
;					  +--------- TOIE1 : Timer/Counter1 Overflow Interrupt Enable
;-----------------------------------------------------------------------------
.equ		Stand_TCCR0	=	0b00000011
;	Timer/Counter0 Control Register	       |||
;					       ||+-- CS00
;					       |+--- CS01
;					       +---- CS02
;		CS02	CS01	CS00
;		0	0	0	timer/counter0 is stopped
;		0	0	1	clk
;		0	1	0	clk/8
;		0	1	1	clk/64
;		1	0	0	clk/256
;		1	0	1	clk/1024
;		1	1	0	external pin T0; falling edge
;		1	1	1	external pin T0; rising edge
;-----------------------------------------------------------------------------
.equ		Stand_PADir	=	0b00000000
;					  ||||||||
;					  |||||||+-- input : 
;					  ||||||+--- input : 
;					  |||||+---- input : 
;					  ||||+----- input : 
;					  |||+------ input : 
;					  ||+------- input : 
;					  |+-------- input : 
;					  +--------- input : 
.equ		Stand_PAData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PBDir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output : 
;					  ||||||+--- output : 
;					  |||||+---- output : 
;					  ||||+----- output : 
;					  |||+------ output : 
;					  ||+------- output : 
;					  |+-------- output : RXD LED
;					  +--------- output : TXD LED
.equ		Stand_PBData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PCDir	=	0b00000000
;					  ||||||||
;					  |||||||+-- input : 
;					  ||||||+--- input : 
;					  |||||+---- input : 
;					  ||||+----- input : 
;					  |||+------ input : 
;					  ||+------- input : 
;					  |+-------- input : 
;					  +--------- input : 
.equ		Stand_PCData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PDDir	=	0b11111110
;					  ||||||||
;					  |||||||+-- input :	[RXD]
;					  ||||||+--- output :	[TXD]
;					  |||||+---- output : 
;					  ||||+----- output : 
;					  |||+------ output : 
;					  ||+------- output : 
;					  |+-------- output : 
;					  +--------- output : 
.equ		Stand_PDData	=	0b11111111
;-----------------------------------------------------------------------------
	.dseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  DATA  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
;						; CONST
.equ	Cr		=	0x0D		;  Cr = 0DH ;
.equ	Lf		=	0x0A		;  Lf = 0AH ;
.equ	Bs		=	0x08		;  Bs = 08H ;
;						;
.equ	StringSize	=	20		;  StringSize = 20 ;
.equ	TextLineSize	= 	32		;  TextLineSize = 32 ;
.equ	NumbConv	=	16		;  NumbConv = 16 ;
.equ	TXDBuffSize	=	16		;  TXDBuffSize = 16 ;
.equ	RXDBuffSize	=	8		;  RXDBuffSize = 8 ;
;						;
;						; VAR
TextLine:	.byte	TextLineSize		;  TextLine : ARRAY [ 0 .. TextLineSize ] OF CHAR ;
TextLineInx:	.byte	1			;  TextLineInx : BYTE ;
TextSize:	.byte	1			;  TextSize : BYTE ;
LeftArgument:	.byte	4			;  LeftArgument : LONGINT ;
RightArgument:	.byte	4			;  RightArgument : LONGINT ;
Wynik:		.byte	4			;  Wynik : LONGINT ;
Fraction:	.byte	4			;  Fraction : LONGWORD ;
NumbChars:	.byte	StringSize		;  NumbChars : ARRAY [ 0 .. StringSize - 1 ] OF CHAR ;
CharCounter:	.byte	1			;  CharCounter : BYTE ;
FormatOK:	.byte	1			;  FormatOK : BOOLEAN ;
Operator:	.byte	1			;  Operator : CHAR ;
TermCh:		.byte	1			;  TermCh : CHAR ;
;-----------------------------------------------------------------------------
	.cseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  CODE  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
	.org	0				;
	rjmp	ResetProcessor			;
;-----------------------------------------------------------------------------
	.org	INT0addr			;External Interrupt0 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT1addr			;External Interrupt1 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ICP1addr			;Input Capture1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Aaddr			;Output Compare1A Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Baddr			;Output Compare1B Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF1addr			;Overflow1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF0addr			;Overflow0 Interrupt Vector Address
	rjmp	TC0OvfInterrupt			;
;-----------------------------------------------------------------------------
	.org	SPIaddr				;SPI Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	URXCaddr			;UART Receive Complete Interrupt Vector Address
	rjmp	USARTRXDInterrupt		;
;-----------------------------------------------------------------------------
	.org	UDREaddr			;UART Data Register Empty Interrupt Vector Address
	rjmp	USARTDataRegEmptInterrupt	;
;-----------------------------------------------------------------------------
	.org	UTXCaddr			;UART Transmit Complete Interrupt Vector Address
	rjmp	USARTTXDInterrupt		;
;-----------------------------------------------------------------------------
	.org	ACIaddr				;Analog Comparator Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT2addr			;External Interrupt2 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC0addr				;Output Compare0 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ERDYaddr			;EEPROM Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	SPMaddr				;SPM complete Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
; definicje na potrzeby biblioteki obslugi USART
.equ	USART_Speed	= 47
;	USART BAUD Rate Generator fosc=7.3728 MHz , 9600 bps
.equ	USART_LED_Trace	= 1
.equ	RXDLED		= 7
.equ	TXDLED		= 6
.equ	USARTTracePort	= PORTB
.equ	LEDOnState	= 0
.include "usartlib.inc"
;-----------------------------------------------------------------------------
.include "arithmetic.inc"
;-----------------------------------------------------------------------------
.include "util.inc"
;-----------------------------------------------------------------------------
HardwareInit :					;PROCEDURE HardwareInit ;
;*************					;BEGIN
	ldi	acc,Stand_TIMSK			; [TIMSK] := Stand_TIMSK ;
	out	TIMSK,acc			;
	ldi	acc,Stand_TCCR0			; [TCCR0] := Stand_TCCR0 ;
	out	TCCR0,acc			;
	ret					;END (* HardwareInit *) ;
;-----------------------------------------------------------------------------
EnvirInit :					;PROCEDURE EnvirInit ;
;**********					;BEGIN
	ldi	acc,Stand_PADir			; [DDRA] := Stand_PADir ;
	out	DDRA,Acc			;
	ldi	acc,Stand_PAData		; [PORTA] := Stand_PAData ;
	out	PORTA,acc			;
	ldi	acc,Stand_PBDir			; [DDRB] := Stand_PBDir ;
	out	DDRB,Acc			;
	ldi	acc,Stand_PBData		; [PORTB] := Stand_PBData ;
	out	PORTB,acc			;
	ldi	acc,Stand_PCDir			; [DDRC] := Stand_PCDir ;
	out	DDRC,Acc			;
	ldi	acc,Stand_PCData		; [PORTC] := Stand_PCData ;
	out	PORTC,acc			;
	ldi	acc,Stand_PDDir			; [DDRD] := Stand_PDDir ;
	out	DDRD,Acc			;
	ldi	acc,Stand_PDData		; [PORTD] := Stand_PDData ;
	out	PORTD,acc			;
	ldz	USART_Speed			; USARTHardwareInit ( USART_Speed ) ;
	rcall	USARTHardwareInit		;
	ret					;END (* EnvirInit *) ;
;-----------------------------------------------------------------------------
SoftwareInit :					;PROCEDURE SoftwareInit ;
;*************					;BEGIN (* SoftwareInit *)
	rcall	USARTSoftwareInit		; USARTSoftwareInit ( ) ;
	ret					;END SoftwareInit ;
;-----------------------------------------------------------------------------
TC0OvfInterrupt :				;PROCEDURE TC0OvfInterrupt ;
;****************				;BEGIN
	pushf					;
.if USART_LED_Trace == 1
	rcall	USARTTimeService		; USARTTimeService ( ) ;
.endif
	popf					;
	reti					;END (* TC0OvfInterrupt *) ;
;-----------------------------------------------------------------------------
IntToStringLONG	:				;PROCEDURE IntToStringLONG ( IntV[z]          : POINTER TO INTEGER ;
;****************				;		         VAR String[x]        : ARRAY OF CHAR ;
;						;                            ConvWidth[param] : BYTE ;
;						;                            EmptyCh[acc]     : CHAR ) ;
;						;VAR
;						; Inx[acc2] : BYTE ;
;						; ModValue[WorkRegLL] : BYTE ;
;						; Minus[stack] : BOOLEAN ;
;						;BEGIN (* IntToStringLONG *)
	push	zl				;
	push	zh				;
	mov	zl,xl				;
	mov	zh,xh				;
	ldi	acc2,0				; FOR Inx := 0 TO ConvWidth DO
SCToSLC_0 :					;  BEGIN
	st	z+,acc				;   String [ Inx ] := EmptyCh ;
	inc	acc2				;  END (* FOR *) ;
	cp	acc2,param			;
	brne	SCToSLC_0			;
	mov	zl,xl				; String [ ConvWidth ] := CHR ( 0 ) ;
	mov	zh,xh				;
	add	zl,param			;
	ldi	acc,0				;
	adc	zh,acc				;
	st	z,acc				;
	dec	param				; Inx := ConvWidth - 1 ;
	mov	acc2,param			;
	pop	zh				; Argument1 := IntV ^ ;
	pop	zl				;
	ld	Argument1LL,z+			;
	ld	Argument1LH,z+			;
	ld	Argument1HL,z+			;
	ld	Argument1HH,z			;
	ldi	param,0				; Minus := FALSE ;
	mov	acc,Argument1HH			; IF Argument1 < 0 THEN
	andi	acc,0x80			;
	breq	SCToSLC_3			;  BEGIN
	rcall	NegArg1				;   Argument1 := - Argument1 ;
	ldi	param,1				;   Minus := TRUE ;
SCToSLC_3 :					;  END (* IF *) ;
	push	param				;
SCToSLC_1 :					; LOOP
	push	acc2				;
	rcall	DivModArg1By10			;  Argument2 := Argument1 DIV 10 ;
						;  ModV := Argument1 MOD 10 ;
	mov	acc,WorkRegLL			;  String [ Inx ] := CHR ( ModV + ORD ( '0' ) ) ;
	ldi	acc2,'0'			;
	add	acc,acc2			;
	pop	acc2				;
	mov	zl,xl				;
	mov	zh,xh				;
	add	zl,acc2				;
	ldi	param,0				;
	adc	zh,param			;
	st	z,acc				;
	copyarg2toarg1				;  Argument1 := Argument2 ;
	cpi	acc2,0				;  IF ( Inx = 0 ) OR ( Argument1 = 0 ) THEN
	breq	SCToSLC_2			;
	mov	acc,Argument1LL			;
	or	acc,Argument1LH			;
	breq	SCToSLC_2			;   BEGIN
						;    EXIT ;
						;   END (* IF *) ;
	dec	acc2				;  Inx := Inx - 1 ;
	rjmp	SCToSLC_1			; END (* LOOP *) ;
SCToSLC_2 :					;
	pop	acc				; IF Minus THEN
	cpi	acc,0				;
	breq	SCToSLC_4			;  BEGIN
	cpi	acc2,1				;   IF Inx >= 1 THEN
	brcs	SCToSLC_5			;    BEGIN
	dec	acc2				;     Inx := Inx - 1 ;
	mov	zl,xl				;     String [ Inx ] := '-' ;
	mov	zh,xh				;
	add	zl,acc2				;
	ldi	acc,0				;
	adc	zh,acc				;
	ldi	acc,'-'				;
	st	z,acc				;
SCToSLC_5 :					;    END (* IF *) ;
SCToSLC_4 :					;  END (* IF *) ;
	ret					;END IntToStringLONG ;
;-----------------------------------------------------------------------------
SendNewLine :					;PROCEDURE SendNewLine ( ) ;
;************					;BEGIN
	ldi	param,Cr			; SendSerial ( Cr ) ;
	rcall	SendSerial			;
	ldi	param,Lf			; SendSerial ( Lf ) ;
	rcall	SendSerial			;
	ret					;END SendNewLine ;
;-----------------------------------------------------------------------------
NextChar :					;FUNCTION NextChar ( ) : CHAR [acc] ;
;*********					;VAR
;						; Ch [stack] : CHAR ;
;						;BEGIN
NeCh_0	:					; REPEAT
	rcall	SerialDataPresent		; UNTIL SerialDataPresent ( ) ;
	brcc	NeCh_0				;
	rcall	GetSerial			; Ch := GetSerial ( ) ;
	push	acc				;
	cpi	acc,' '				; IF Ch >= ' ' THEN
	brcs	NeCh_1				;  BEGIN
	mov	param,acc			;   SendSerial ( Ch ) ;
	rcall	SendSerial			;
NeCh_1	:					;  END (* IF *) ;
	pop	acc				; NextChar := Ch ;
	ret					;END (* NextChar *) ;
;-----------------------------------------------------------------------------
DisplayInputLine :				;PROCEDURE DisplayInputLine ( ) ;
;*****************				;VAR
;						; Loop [stack] : BYTE ;
;						; ChPtr [Z] : ^ CHAR ;
;						;BEGIN
	lds	acc,TextLineInx			; IF TextLineInx <> 0 THEN
	cpi	acc,0				;
	breq	DiInpL_3			;  BEGIN
	ldz	TextLine			;   ChPtr := ADR ( TextLine [ 0 ] ) ;
	ldi	acc,1				;   FOR Loop := 1 TO TextLineInx DO
DiInpL_2 :					;    BEGIN
	push	acc				;
	ld	param,z+			;     SendSerial ( ChPtr ^ ) ;
						;     ChPtr := ChPtr + 1 ;
	pushz					;
	rcall	SendSerial			;
	popz					;
	pop	acc				;
	lds	acc2,TextLineInx		;
	cp	acc,acc2			;
	breq	DiInpL_4			;
	inc	acc				;
	rjmp	DiInpL_2			;
DiInpL_4 :					;    END (* FOR *) ;
DiInpL_3 :					;  END (* IF *) ;
	ret					;END (* DisplayInputLine *) ;
;-----------------------------------------------------------------------------
RefreashText :					;PROCEDURE RefreashText ( ) ;
;*************					;VAR
;						; Loop [stack] : BYTE ;
;						;BEGIN
	ldi	param,Cr			; SendSerial ( CR ) ;
	rcall	SendSerial			;
	ldi	acc,0				; FOR Loop := 0 TO TextLineSize-1 DO
TefTx_1	:					;  BEGIN
	push	acc				;
	ldi	param,' '			;   SendSerial ( ' ' ) ;
	rcall	SendSerial			;
	pop	acc				;
	cpi	acc,TextLineSize-1		;
	breq	RefTx_0				;
	inc	acc				;
	rjmp	TefTx_1				;
RefTx_0	:					;  END (* FOR *) ;
	ldi	param,Cr			; SendSerial ( CR ) ;
	rcall	SendSerial			;
	rcall	DisplayInputLine		; DisplayInputLine ( );
	ret					;END (* RefreashText ) ;
;-----------------------------------------------------------------------------
AddChToTextLine	:				;PROCEDURE AddChToTextLine ( Ch [param] : CHAR ) ;
;****************				;BEGIN
	lds	acc,TextLineInx			; IF TextLineInx < TextLineSize THEN
	cpi	acc,TextLineSize		;
	brcc	AChTTxL_0			;  BEGIN
	ldz	TextLine			;   TextLine [ TextLineInx ] := Ch ;
	add	zl,acc				;
	ldi	acc,0				;
	adc	zh,acc				;
	st	z,param				;
	incvar	TextLineInx			;   TextLineInx := TextLineInx + 1 ;
AChTTxL_0 :					;  END (* IF *) ;
	ret					;END (* AddChToTextLine *) ;
;-----------------------------------------------------------------------------
GetTextLine :					;PROCEDURE GetTextLine ( ) ;
;************					;VAR
;						; Ch [acc] : CHAR ;
;						;BEGIN
	ldi	acc,0				; TextLineInx := 0 ;
	sts	TextLineInx,acc			;
GeTL_0	:					; LOOP
	rcall	NextChar			;  Ch := NextChar ( ) ;
	cpi	acc,Cr				;  IF Ch = Cr THEN
	brne	GeTL_1				;   BEGIN
	rcall	SendNewLine			;    SendNewLine ( ) ;
	ldz	InpTextPrompt<<1		;    SendSerialFlashTxt ( InpTextPrompt ) ;
	rcall	SendSerialFlashTxt		;
	rcall	DisplayInputLine		;    DisplayInputLine ( );
	rcall	SendNewLine			;    SendNewLine ( ) ;
	rcall	SendNewLine			;    SendNewLine ( ) ;
	lds	acc,TextLineInx			;    TextSize := TextLineInx ;
	sts	TextSize,acc			;
	ret					;    RETURN ;
GeTL_1	:					;   END (* IF *) ;
	cpi	acc,Bs				;  IF Ch = Bs THEN
	brne	GeTL_2				;   BEGIN
	ldz	TextLineInx			;    IF TextLineInx <> 0 THEN
	ld	acc,z				;
	cpi	acc,0				;
	breq	GeTL_4				;     BEGIN
	dec	acc				;      TextLineInx := TextLineInx - 1 ;
	st	z,acc				;
	rcall	RefreashText			;      RefreashText ( ) ;
GeTL_4	:					;     END (* IF *) ;
	rjmp	GeTL_3				;   END (* IF ... *)
GeTL_2	:					;  ELSE
						;   BEGIN
	mov	param,acc			;    AddChToTextLine ( Ch ) ;
	rcall	AddChToTextLine			;
GeTL_3	:					;   END (* IF ... ELSE *) ;
	rjmp	GeTL_0				;
						;END GetTextLine ;
;-----------------------------------------------------------------------------
GChar	:					;FUNCTION GChar ( ) : CHAR [acc] ;
;********					;VAR
;						; Ch [acc] : CHAR ;
;						;BEGIN
	lds	acc,TextLineInx			; IF TextLineInx = TextSize THEN
	lds	acc2,TextSize			;
	cp	acc,acc2			;
	brne	GCh_0				;  BEGIN
	ldi	acc,0				;   GCh := CHR ( 0 ) ;
	ret					;   RETURN ;
GCh_0	:					;  END (* IF *) ;
	ldz	TextLine			; Ch := TextLine [ TextLineInx ] ;
	add	zl,acc				;
	ldi	acc,0				;
	adc	zh,acc				;
	ld	acc,z				;
	incvar	TextLineInx			; TextLineInx := TextLineInx + 1 ;
						; GChar := Ch ;
	ret					;END (* GChar *) ;
;-----------------------------------------------------------------------------
StringToBin :					;FUNCTION StringToBin ( VAR OK [Z] : BOOLEAN ) : LONGINT[Arg1] ;
;************					;VAR
;						; Minus [acc2] : BOOLEAN ;
;						; ChPtr [X] : ^ CHAR ;
;						; Ch [acc] : CHAR ;
;						;BEGIN
	lds	acc,CharCounter			; IF CharCounter >= 11 THEN
	cpi	acc,11				;
	brcs	StrTB_8				;  BEGIN
	ldi	acc,0				;   OK := FALSE ;
	st	z,acc				;
	clrarg1					;   Argument1 := 0 ;
	ret					;   RETURN ;
StrTB_8	:					;  END (* IF *) ;
	ldi	acc,1				; OK := TRUE ;
	st	z,acc				;
	ldi	acc2,0				; Minus := FALSE ;
	ldi	xl,low(NumbChars)		; ChPtr := ADR ( NumbChars [ 0 ] ) ;
	ldi	xh,high(NumbChars)		;
	ld	acc,x				; IF ChPtr ^ = '-' THEN
	cpi	acc,'-'				;
	brne	StrTB_0				;  BEGIN
	ldi	acc2,1				;   Minus := TRUE ;
	adiw	xl,1				;   ChPtr := ChPtr + 1 ;
	rjmp	StrTB_1				;  END (* IF ... *)
StrTB_0	:					; ELSE
						;  BEGIN
	cpi	acc,'+'				;   IF NumbChars [ 0 ] = '+' THEN
	brne	StrTB_2				;    BEGIN
	adiw	xl,1				;     ChPtr := ChPtr + 1 ;
StrTB_2	:					;    END (* IF *) ;
StrTB_1	:					;  END (* IF ... ELSE *) ;
	clrarg1					; Argument1 := 0 ;
	ld	acc,x+				; Ch := ChPtr ^ ;
						; ChPtr := ChPtr + 1 ;
StrTB_3	:					; WHILE Ch <> CHR ( 0 ) DO
	cpi	acc,0				;
	breq	StrTB_4				;  BEGIN
	rcall	MultAgr1And10			;   Argument1 := Argument1 * 10 + ( ORD ( Ch ) - '0' ) ;
	subi	acc,'0'				;
	add	Argument1LL,acc			;
	ldi	acc,0				;
	adc	Argument1LH,acc			;
	adc	Argument1HL,acc			;
	adc	Argument1HH,acc			;
	ld	acc,x+				;   Ch := ChPtr ^ ;
						;   ChPtr := ChPtr + 1 ;
	rjmp	StrTB_3				;
StrTB_4	:					;  END (* WHILE *) ;
	mov	acc,Argument1HH			; IF Argument1HH.63 = 1 THEN
	andi	acc,0x80			;
	breq	StrTB_7				;  BEGIN
	ldi	acc,0				;   OK := FALSE ;
	st	z,acc				;
	clr	xl				;   StringToBin := 0 ;
	clr	xh				;
	ret					;   RETURN ;
StrTB_7	:					;  END (* IF *) ;
	cpi	acc2,0				; IF Minus THEN
	breq	StrTB_6				;  BEGIN
	rcall	NegArg1				;   Argument1 := - Argument1 ;
StrTB_6	:					;  END (* IF *) ;
	ret					;END (* StringToBin *) ;
;-----------------------------------------------------------------------------
AddChToString :					;PROCEDURE AddChToString ( Ch [param] : CHAR ) ;
;**************					;BEGIN
	lds	acc,CharCounter			; IF CharCounter < StringSize THEN
	cpi	acc,StringSize			;
	brcc	AddCh_0				;  BEGIN
	ldz	NumbChars			;   NumbChars [ CharCounter ] := Ch ;
	add	zl,acc				;
	ldi	acc,0				;
	adc	zh,acc				;
	st	z,param				;
	incvar	CharCounter			;   CharCounter := CharCounter + 1 ;
AddCh_0	:					;  END (* IF *) ;
	ret					;END (* AddChToString *) ;
;-----------------------------------------------------------------------------
Digit	:					;PROCEDURE Digit ( Ch [acc] : CHAR ) : BOOLEAN ;
;********					;BEGIN
	cpi	acc,'0'				; IF ( Ch < '0' ) OR ( Ch > '9' ) THEN
	brcs	Dig_0				;
	cpi	acc,'9'+1			;
	brcs	Dig_1				;
Dig_0	:					;  BEGIN
	clc					;   Digit := FALSE ;
	ret					;   RETURN ;
Dig_1	:					;  END (* IF *) ;
	sec					; Digit := TRUE ;
	ret					;END (* Digit *) ;
;-----------------------------------------------------------------------------
GetNumber :					;FUNCTION GetNumber ( ) : BOOLEAN ;
;**********					;VAR
;						; Ch [acc] : CHAR ;
;						;BEGIN
	clr	acc				; CharCounter := 0 ;
	sts	CharCounter,acc			;
	rcall	GChar				; Ch := GChar ( ) ;
GeNu_5	:					; WHILE Ch = ' ' DO
	cpi	acc,' '				;
	brne	GeNu_6				;  BEGIN
	rcall	GChar				;   Ch := GChar ( ) ;
	rjmp	GeNu_5				;
GeNu_6	:					;  END (* WHILE *) ;
	cpi	acc,'+'				; IF Ch = '+' THEN
	brne	GeNu_7				;  BEGIN
	mov	param,acc			;   AddChToString ( Ch ) ;
	rcall	AddChToString			;
	rcall	GChar				;   Ch := GChar ( ) ;
	rjmp	GeNu_8				;  END (* IF ... *)
GeNu_7	:					; ELSE
						;  BEGIN
	cpi	acc,'-'				;   IF Ch = '-' THEN
	brne	GeNu_9				;    BEGIN
	mov	param,acc			;     AddChToString ( Ch ) ;
	rcall	AddChToString			;
	rcall	GChar				;     Ch := GChar ( ) ;
GeNu_9	:					;    END (* IF *) ;
GeNu_8	:					;  END (* IF ... ELSE *) ;
	rcall	Digit				; IF NOT Digit ( Ch ) THEN
	brcs	GeNu_2				;  BEGIN
	clc					;   GetNumber := FALSE ;
	ret					;   RETURN ;
GeNu_2	:					;  END (* IF *) ;
	mov	param,acc			; AddChToString ( Ch ) ;
	push	acc				;
	rcall	AddChToString			;
	pop	acc				;
GeNu_0	:					; WHILE Ch = '0' DO
	cpi	acc,'0'				;
	brne	GeNu_1				;  BEGIN
	rcall	GChar				;   Ch := GChar ( ) ;
	rjmp	GeNu_0				;
GeNu_1	:					;  END (* WHILE *) ;
	rcall	GChar				; Ch := GChar ( ) ;
GeNu_3	:					; WHILE Digit ( Ch ) DO
	rcall	Digit				;
	brcc	GeNu_4				;  BEGIN
	mov	param,acc			;   AddChToString ( Ch ) ;
	rcall	AddChToString			;
	rcall	GChar				;   Ch := GChar ( ) ;
	rjmp	GeNu_3				;
GeNu_4	:					;  END (* WHILE *) ;
	sts	TermCh,acc			; TermCh := Ch ;
	ldi	param,0				; AddChToString ( CHR ( 0 ) ) ;
	rcall	AddChToString			;
	sec					; GetNumber := TRUE ;
	ret					;END (* GetNumber *) ;
;-----------------------------------------------------------------------------
GetData	:					;FUNCTION GetData ( ) : BOOLEAN ;
;********					;BEGIN
	rcall	GetTextLine			; GetTextLine ( ) ;
	ldi	acc,0				; TextLineInx := 0 ;
	sts	TextLineInx,acc			;
	rcall	GetNumber			; IF NOT GetNumber ( ) THEN
	brcs	GetDa_1				;  BEGIN
	clc					;   GetData := FALSE ;
	ret					;   RETURN ;
GetDa_1	:					;  END (* IF *) ;
	ldz	FormatOK			; LeftArgument := StringToBin ( FormatOK ) ;
	rcall	StringToBin			;
	starg1	LeftArgument			;
	lds	acc,FormatOK			; IF NOT FormatOK THEN
	cpi	acc,0				;
	brne	GetDa_0				;  BEGIN
	clc					;   GetData := FALSE ;
	ret					;   RETURN ;
GetDa_0	:					;  END (* IF *) ;
GetDa_2	:					; WHILE TermCh = ' ' DO
	lds	acc,TermCh			;
	cpi	acc,' '				;
	brne	GetDa_6				;  BEGIN
	rcall	GChar				;   TermCh := GChar ( ) ;
	sts	TermCh,acc			;
	rjmp	GetDa_2				;
GetDa_6	:					;  END (* WHILE *) ;
	lds	acc,TermCh			; Operator := TermCh ;
	sts	Operator,acc			;
	cpi	acc,'+'				; IF NOT ( ( Operator = '+' ) OR
	breq	GetDa_5				;
	cpi	acc,'-'				;          ( Operator = '-' ) OR
	breq	GetDa_5				;
	cpi	acc,'*'				;          ( Operator = '*' ) OR
	breq	GetDa_5				;
	cpi	acc,'/'				;          ( Operator = '/' ) ) THEN
	breq	GetDa_5				;
						;  BEGIN
	clc					;   GetData := FALSE ;
	ret					;   RETURN ;
GetDa_5	:					;  END (* IF *) ;		
	rcall	GetNumber			; IF NOT GetNumber ( ) THEN
	brcs	GetDa_3				;  BEGIN
	clc					;   GetData := FALSE ;
	ret					;   RETURN ;
GetDa_3	:					;  END (* IF *) ;
	ldz	FormatOK			; RightArgument := StringToBin ( FormatOK ) ;
	rcall	StringToBin			;
	starg1	RightArgument			;
	lds	acc,FormatOK			; IF NOT FormatOK THEN
	cpi	acc,0				;
	brne	GetDa_4				;  BEGIN
	clc					;   GetData := FALSE ;
	ret					;   RETURN ;
GetDa_4	:					;  END (* IF *) ;
	sec					; GetData := TRUE ;
	ret					;END (* GetData *) ;
;-----------------------------------------------------------------------------
DisplayArguments:				;PROCEDURE DisplayArguments ( ) ;
;****************				;BEGIN
	ldi	param,' '			; SendSerial ( ' ' ) ;
	rcall	SendSerial			;
	ldz	LeftArgument			; IntToStringLONG ( LeftArgument , NumbChars , NumbConv , ' ' ) ;
	ldx	NumbChars			;
	ldi	param,NumbConv			;
	ldi	acc,' '				;
	rcall	IntToStringLONG			;
	ldz	NumbChars			; SendSerialRAMTxt ( NumbChars ) ;
	rcall	SendSerialRAMTxt		;
	rcall	SendNewLine			; SendNewLine ( ) ;
	lds	param,Operator			; SendSerial ( Operator ) ;
	rcall	SendSerial			;
	ldz	RightArgument			; IntToStringLONG ( RightArgument , NumbChars , NumbConv , ' ' ) ;
	ldx	NumbChars			;
	ldi	param,NumbConv			;
	ldi	acc,' '				;
	rcall	IntToStringLONG			;
	ldz	NumbChars			; SendSerialRAMTxt ( NumbChars ) ;
	rcall	SendSerialRAMTxt		;
	rcall	SendNewLine			; SendNewLine ( ) ;
	ldz	LineText<<1			; SendSerialFlashTxt ( LineText ) ;
	rcall	SendSerialFlashTxt		;
	rcall	SendNewLine			; SendNewLine ( ) ;
	ret					;END (* DisplayArguments *) ;
;-----------------------------------------------------------------------------
DisplayResult	:				;PROCEDURE DisplayResult ( ) ;
;****************				;BEGIN
	ldz	Wynik				; IntToStringLONG ( Wynik , NumbChars , NumbConv , ' ' ) ;
	ldx	NumbChars			;
	ldi	param,NumbConv			;
	ldi	acc,' '				;
	rcall	IntToStringLONG			;
	ldi	param,'='			; SendSerial ( '=' ) ;
	rcall	SendSerial			;
	ldz	NumbChars			; SendSerialRAMTxt ( NumbChars ) ;
	rcall	SendSerialRAMTxt		;
	rcall	SendNewLine			; SendNewLine ( ) ;
	ret					;END (* DisplayResult *) ;
;-----------------------------------------------------------------------------
AddOperation	:				;PROCEDURE AddOperation ( ) ;
;****************				;BEGIN
	rcall	DisplayArguments		; DisplayArguments ( ) ;
	loarg1	LeftArgument			; Argument1 := LeftArgument ;
	loarg2	RightArgument			; Argument2 := RightArgument ;
	add	Argument1LL,Argument2LL		; Argument1 := Argument1 + Argument2 ;
	adc	Argument1LH,Argument2LH		;
	adc	Argument1HL,Argument2HL		;
	adc	Argument1HH,Argument2HH		;
	starg1	Wynik				; Wynik := Argument1 ;
	rcall	DisplayResult			; DisplayResult ( ) ;
	ret					;END (* AddOperation *) ;
;-----------------------------------------------------------------------------
SubOperation	:				;PROCEDURE SubOperation ( ) ;
;****************				;BEGIN
	rcall	DisplayArguments		; DisplayArguments ( ) ;
	loarg1	LeftArgument			; Argument1 := LeftArgument ;
	loarg2	RightArgument			; Argument2 := RightArgument ;
	sub	Argument1LL,Argument2LL		; Argument1 := Argument1 - Argument2 ;
	sbc	Argument1LH,Argument2LH		;
	sbc	Argument1HL,Argument2HL		;
	sbc	Argument1HH,Argument2HH		;
	starg1	Wynik				; Wynik := Argument1 ;
	rcall	DisplayResult			; DisplayResult ( ) ;
	ret					;END SubOperation ;
;-----------------------------------------------------------------------------
MultOperation	:				;PROCEDURE MultOperation ( ) ;
;****************				;BEGIN
	rcall	DisplayArguments		; DisplayArguments ( ) ;
	loarg1	LeftArgument			; Argument1 := LeftArgument ;
;	mov	acc,Argument1LH			; IF Argument1 > 16bit THEN
;	andi	acc,0x80			;
;	or	acc,Argument1HL			;
;	or	acc,Argument1HH			;
;	breq	MultOp_0			;  BEGIN
;	ldz	MultArgRText<<1			;   SendSerialFlashTxt ( MultArgRText ) ;
;	rcall	SendSerialFlashTxt		;
;	ret					;   RETURN ;
;MultOp_0:					;  END (* IF *) ;
	loarg2	RightArgument			; Argument2 := RightArgument ;
;	mov	acc,Argument2LH			; IF Argument2 > 16bit THEN
;	andi	acc,0x80			;
;	or	acc,Argument2HL			;
;	or	acc,Argument2HH			;
;	breq	MultOp_1			;  BEGIN
;	ldz	MultArgRText<<1			;   SendSerialFlashTxt ( MultArgRText ) ;
;	rcall	SendSerialFlashTxt		;
;	ret					;   RETURN ;
;MultOp_1:					;  END (* IF *) ;
	rcall	MultArg1AndArg2U2		; MultArg1AndArg2U2 ( ) ;
	copyrestoarg1				;
	starg1	Wynik				; Wynik := Argument1 ;
	rcall	DisplayResult			; DisplayResult ( ) ;
	ret					;END (* MultOperation *) ;
;-----------------------------------------------------------------------------
FractionResult	:				;PROCEDURE FractionResult ( ) ;
;****************				;VAR
;						; Minus[acc2] : BOOLEAN ;
;						;BEGIN
	ldi	acc2,0				; Minus := FALSE ;
	loarg1	LeftArgument			; Argument1 := LeftArgument ;
	mov	acc,Argument1HH			; IF Argument1 < 0 THEN
	andi	acc,0x80			;
	breq	DivOpF_2			;  BEGIN
	rcall	NegArg1				;   Argument1 := - Argument1 ;
	ldi	acc2,1				;   Minus := TRUE ;
DivOpF_2:					;  END (* IF *) ;
	loarg2	RightArgument			; Argument2 := RightArgument ;
	mov	acc,Argument2HH			; IF Argument2 < 0 THEN
	andi	acc,0x80			;
	breq	DivOpF_3			;  BEGIN
	rcall	NegArg2				;   Argument2 := - Argument2 ;
	ldi	acc,1				;   Minus := NOT Minus ;
	eor	acc2,acc			;
DivOpF_3:					;  END (* IF *) ;
	push	acc2				; Result := Argument1 DIV Argument2 ;
	rcall	DivideArg1ByArg2Fract		; Argument1 := Argument1 MOD Argument2 ;
	copyrestoarg1				;
	brvc	DivOpF_5			; IF <nadmiar z dzielenia> THEN
						;  BEGIN
	ldz	VMessageText<<1			;   SendSerialFlashTxt ( VMessageText ) ;
	rcall	SendSerialFlashTxt		;
	pop	acc				;
	ret					;   RETURN ;
DivOpF_5:					;  END (* IF *) ;
	pop	acc				; IF Minus THEN
	cpi	acc,0				;
	breq	DivOpF_4			;  BEGIN
	rcall	NegArg1				;   Argument1 := - Argument1 ;
DivOpF_4:					;  END (* IF *) ;
	starg1	Wynik				; Wynik := Argument1 ;
	ldz	Fraction			; Fraction := WorkReg ;
	st	z+,WorkRegLL			;
	st	z,WorkRegLH			;
	ldz	Wynik				; IntToStringLONG ( Wynik , NumbChars , NumbConv , ' ' ) ;
	ldx	NumbChars			;
	ldi	param,NumbConv			;
	ldi	acc,' '				;
	rcall	IntToStringLONG			;
	ldi	param,'='			; SendSerial ( '=' ) ;
	rcall	SendSerial			;
	ldz	NumbChars			; SendSerialRAMTxt ( NumbChars ) ;
	rcall	SendSerialRAMTxt		;
	ldz	Fraction			; Argument1 := Fraction ;
	ld	Argument1LL,z+			;
	ld	Argument1LH,z			;
	ldi	acc,0				;
	mov	Argument1HL,acc			;
	mov	Argument1HH,acc			;
	rcall	MultAgr1And10			; Argument1 := Argument1 * 10000 ;
	rcall	MultAgr1And10			;
	rcall	MultAgr1And10			;
	rcall	MultAgr1And10			;
	loarg2	RightArgument			; Argument2 := RightArgument ;
	mov	acc,Argument2HH			; IF Argument2 < 0 THEN
	andi	acc,0x80			;
	breq	DivOpF_1			;  BEGIN
	rcall	NegArg2				;   Argument2 := - Argument2 ;
DivOpF_1:					;  END (* IF *) ;
	rcall	DivideArg1ByArg2Fract		; Argument1 := Argument1 MOD Argument2 ;
	copyrestoarg1				;
	starg1	Wynik				; Wynik := Argument1 ;
	ldi	param,'.'			; SendSerial ( '.' ) ;
	rcall	SendSerial			;
	ldz	Wynik				; IntToStringLONG ( Wynik , NumbChars , 4 , '0' ) ;
	ldx	NumbChars			;
	ldi	param,4				;
	ldi	acc,'0'				;
	rcall	IntToStringLONG			;
	ldz	NumbChars			; SendSerialRAMTxt ( NumbChars ) ;
	rcall	SendSerialRAMTxt		;
	ldz	TruncMode<<1			; SendSerialFlashTxt ( TruncMode ) ;
	rcall	SendSerialFlashTxt		;
	rcall	SendNewLine			; SendNewLine ( ) ;
	ret					;END (* FractionResult *) ;
;-----------------------------------------------------------------------------
RoundResult	:				;PROCEDURE RoundResult ( ) ;
;****************				;VAR
;						; Minus[acc2] : BOOLEAN ;
;						;BEGIN
	ldi	acc2,0				; Minus := FALSE ;
	loarg1	LeftArgument			; Argument1 := LeftArgument ;
	mov	acc,Argument1HH			; IF Argument1 < 0 THEN
	andi	acc,0x80			;
	breq	DivOpR_2			;  BEGIN
	rcall	NegArg1				;   Argument1 := - Argument1 ;
	ldi	acc2,1				;   Minus := TRUE ;
DivOpR_2:					;  END (* IF *) ;
	loarg2	RightArgument			; Argument2 := RightArgument ;
	mov	acc,Argument2HH			; IF Argument2 < 0 THEN
	andi	acc,0x80			;
	breq	DivOpR_3			;  BEGIN
	rcall	NegArg2				;   Argument2 := - Argument2 ;
	ldi	acc,1				;   Minus := NOT Minus ;
	eor	acc2,acc			;
DivOpR_3:					;  END (* IF *) ;
	push	acc2				; Result := Argument1 DIV Argument2 ;
	rcall	DivideArg1ByArg2Fract		; Argument1 := Argument1 MOD Argument2 ;
	copyrestoarg1				;
	brvc	DivOpR_9			; IF <nadmiar z dzielenia> THEN
						;  BEGIN
	ldz	VMessageText<<1			;   SendSerialFlashTxt ( VMessageText ) ;
	rcall	SendSerialFlashTxt		;
	pop	acc				;
	ret					;   RETURN ;
DivOpR_9:					;  END (* IF *) ;
	starg1	Wynik				; Wynik := Argument1 ;
	mov	Argument1LL,WorkRegLL		; Argument1 := WorkReg ;
	mov	Argument1LH,WorkRegLH		;
	ldi	acc,0				;
	mov	Argument1HL,acc			;
	mov	Argument1HH,acc			;
	rcall	MultAgr1And10			; Argument1 := Argument1 * 10000 ;
	rcall	MultAgr1And10			;
	rcall	MultAgr1And10			;
	rcall	MultAgr1And10			;
	loarg2	RightArgument			; Argument2 := RightArgument ;
	mov	acc,Argument2HH			; IF Argument2 < 0 THEN
	andi	acc,0x80			;
	breq	DivOpR_5			;  BEGIN
	rcall	NegArg2				;   Argument2 := - Argument2 ;
DivOpR_5:					;  END (* IF *) ;
	rcall	DivideArg1ByArg2Round		; Argument1 := Argument1 MOD Argument2 ;
	copyrestoarg1				;
	ldi	acc,low(10000)			; Argument2 := 10000 ;
	mov	Argument2LL,acc			;
	ldi	acc,high(10000)			;
	mov	Argument2LH,acc			; IF Argument1 = Argument2 THEN
	cp	Argument1LL,Argument2LL		;
	cpc	Argument1LH,Argument2LH		;
	brne	DivOpR_1			;  BEGIN
	loarg2	Wynik				;   Argument2 := Wynik ;
	ldi	acc,1				;   Argument2 := Argument2 + 1 ;
	add	Argument2LL,acc			;
	ldi	acc,0				;
	adc	Argument2LH,acc			;
	adc	Argument2HL,acc			;
	adc	Argument2HH,acc			;
	clrarg1					;   Argument1 := 0 ;
	starg2	Wynik				;   Wynik := Argument2 ;
DivOpR_1:					;  END (* IF *) ;
	starg1	Fraction			; Fraction := Argument1 ;
	pop	acc				; IF Minus THEN
	cpi	acc,0				;
	breq	DivOpR_4			;  BEGIN
	loarg1	Wynik				;   Argument1 := Wynik ;
	rcall	NegArg1				;   Argument1 := - Argument1 ;
	starg1	Wynik				;   Wynik := Argument1 ;
DivOpR_4:					;  END (* IF *) ;
	ldz	Wynik				; IntToStringLONG ( Wynik , NumbChars , NumbConv , ' ' ) ;
	ldx	NumbChars			;
	ldi	param,NumbConv			;
	ldi	acc,' '				;
	rcall	IntToStringLONG			;
	ldi	param,'='			; SendSerial ( '=' ) ;
	rcall	SendSerial			;
	ldz	NumbChars			; SendSerialRAMTxt ( NumbChars ) ;
	rcall	SendSerialRAMTxt		;
	ldi	param,'.'			; SendSerial ( '.' ) ;
	rcall	SendSerial			;
	ldz	Fraction			; IntToStringLONG ( Fraction , NumbChars , 4 , '0' ) ;
	ldx	NumbChars			;
	ldi	param,4				;
	ldi	acc,'0'				;
	rcall	IntToStringLONG			;
	ldz	NumbChars			; SendSerialRAMTxt ( NumbChars ) ;
	rcall	SendSerialRAMTxt		;
	ldz	RoundMode<<1			; SendSerialFlashTxt ( RoundMode ) ;
	rcall	SendSerialFlashTxt		;
	rcall	SendNewLine			; SendNewLine ( ) ;
	ret					;END (* RoundResult *) ;
;-----------------------------------------------------------------------------
DivideOperation	:				;PROCEDURE DivideOperation ( ) ;
;****************				;BEGIN
	rcall	DisplayArguments		; DisplayArguments ( ) ;
	loarg1	RightArgument			; Argument1 := RightArgument ;
	mov	acc,Argument1HL			; IF Argument1 > 16bit THEN
	and	acc,Argument1HH			;
	mov	acc2,Argument1LH		;
	ori	acc2,0x7F			;
	and	acc,acc2			;
	cpi	acc,0xFF			;
	breq	DivOp_1				;
	mov	acc,Argument1HL			;
	or	acc,Argument1HH			;
	mov	acc2,Argument1LH		;
	andi	acc2,0x80			;
	or	acc,acc2			;
	breq	DivOp_1				;  BEGIN
	ldz	MultArgRText<<1			;   SendSerialFlashTxt ( MultArgRText ) ;
	rcall	SendSerialFlashTxt		;
	ret					;   RETURN ;
DivOp_1	:					;  END (* IF *) ;
	rcall	FractionResult			; FractionResult ( ) ;
	rcall	RoundResult			; RoundResult ( ) ;
	ret					;END (* DivideOperation *) ;
;-----------------------------------------------------------------------------
Calculate	:				;PROCEDURE Calculate ( ) ;
;****************				;BEGIN
	rcall	SendNewLine			; SendNewLine ( ) ;
	lds	acc,Operator			; CASE Operator OF
	cpi	acc,'+'				; | '+' :
	brne	Calc_1				;  BEGIN
	rcall	AddOperation			;   AddOperation ( ) ;
	ret					;   RETURN ;
						;  END (* CASE WARIANT *) ;
Calc_1	:					;
	cpi	acc,'-'				; | '-' :
	brne	Calc_2				;  BEGIN
	rcall	SubOperation			;   SubOperation ( ) ;
	ret					;   RETURN ;
						;  END (* CASE WARIANT *) ;
Calc_2	:					;
	cpi	acc,'*'				; | '*' :
	brne	Calc_3				;  BEGIN
	rcall	MultOperation			;   MultOperation ( ) ;
						;  END (* CASE WARIANT *) ;
	ret					;   RETURN ;
Calc_3	:					;
	cpi	acc,'/'				; | '/' :
	brne	Calc_4				;  BEGIN
	rcall	DivideOperation			;   DivideOperation ( ) ;
						;  END (* CASE WARIANT *) ;
	ret					;   RETURN ;
Calc_4	:					; ELSE
						; END (* CASE *) ;
	ret					;END (* Calculate *) ;
;-----------------------------------------------------------------------------
ResetProcessor	:				;
	ldi	acc,HIGH(RAMEND)		;
	out	SPH,acc				;
	ldi	acc,LOW(RAMEND)			;
	out	SPL,acc				;
						;BEGIN (* Main *)
	rcall	ClearRAM			; ClearRAM ( ) ;
	rcall	ClearRegs			; ClearRegs ( ) ;
	rcall	HardwareInit			; HardwareInit ( ) ;
	rcall	EnvirInit			; EnvirInit ( ) ;
	rcall	SoftwareInit			; SoftwareInit ( ) ;
	sei					; EI ( ) ;
	ldz	HelloText<<1			; SendSerialFlashTxt ( HelloText ) ;
	rcall	SendSerialFlashTxt		;
Main_0	:					; LOOP
	ldz	PromptText<<1			;  SendSerialFlashTxt ( PromptText ) ;
	rcall	SendSerialFlashTxt		;
	rcall	GetData				;  IF GetData ( ) THEN
	brcc	Main_1				;   BEGIN
	rcall	Calculate			;    Calculate ( ) ;
	rjmp	Main_2				;   END (* IF ... *)
Main_1	:					;  ELSE
						;   BEGIN
	ldz	ErrorText<<1			;    SendSerialFlashTxt ( ErrorText ) ;
	rcall	SendSerialFlashTxt		;
Main_2	:					;   END (* IF ... ELSE *) ;
	rjmp	Main_0				; END (* LOOP *) ;
						;END (* Main *).
;-----------------------------------------------------------------------------
HelloText:	.db	Cr,Lf,"Prosty czterodzialaniowy kalkulator",Cr,Lf,Cr,Lf,0
PromptText:	.db	Cr,Lf,Cr,Lf,"Podaj dzialanie [np. 12*34<Enter>]:",Cr,Lf,0
ErrorText:	.db	Cr,Lf,"Blad w zapisie.",Cr,Lf,0
InpTextPrompt:	.db	"Wprowadzono :",0
NumberTxt:	.db	Cr,Lf,"liczba : ",0
TruncMode:	.db	"   (wynik z obcieciem ulamka)",0
RoundMode:	.db	"   (wynik z zaokragleniem ulamka)",0
LineText:	.db	"-----------------",0
VMessageText:	.db	Cr,Lf,"Nadmiar arytmetyczny.",Cr,Lf,0
MultArgRText:	.db	"Argumenty dla operacji mnozenia musza byc 16-bitowe:",Cr,Lf
		.db	"zawierac sie w przedziale -32768 .. +32767",Cr,Lf,0,0
.exit
;   65519 / 32760
