;
;*******************************************************
;*******************************************************
;**                                                   **
;**  Program prezentacji algorytmow konwersji         **
;**                                                   **
;**         Andrzej Pawluczuk                         **
;**                                                   **
;**         Mikrokontroler: ATMEGA8515                **
;**                                                   **
;*******************************************************
;*******************************************************
;
	.nolist
;*******************************************************
.include "m8515def.inc"
;*******************************************************
	.list
	.listmac
;-----------------------------------------------------------------------------
.macro	ldz
	ldi	zl,low(@0)
	ldi	zh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	ldx
	ldi	xl,low(@0)
	ldi	xh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	pushz
	push	zl
	push	zh
.endm
;-----------------------------------------------------------------------------
.macro	popz
	pop	zh
	pop	zl
.endm
;-----------------------------------------------------------------------------
.macro	pushf
	push	acc
	in	acc,sreg
	push	acc
.endm
;-----------------------------------------------------------------------------
.macro	popf
	pop	acc
	out	sreg,acc
	pop	acc
.endm
;-----------------------------------------------------------------------------
.def DivLoopCounter	= r1
.def Argument1LL	= r2
.def Argument1LH	= r3
.def Argument1HL	= r4
.def Argument1HH	= r5
.def Argument2LL	= r6
.def Argument2LH	= r7
.def Argument2HL	= r8
.def Argument2HH	= r9
.def WorkRegLL		= r10
.def WorkRegLH		= r11
.def WorkRegHL		= r12
.def WorkRegHH		= r13
.def acc		= r16
.def acc2		= r17
.def param		= r18
.def ResultLL		= r22
.def ResultLH		= r23
.def ResultHL		= r24
.def ResultHH		= r25
;-----------------------------------------------------------------------------
.equ		Stand_TIMSK	=	0b00000010
;	Timer/counter Interrupt Mask	  ||| | |
;					  ||| | +--- TOIE0 : Timer/Counter 0 Overflow Interrupt Enable
;					  ||| +----- TICIE1 : Timer/Counter1 Input Capture Interrupt Enable
;					  ||+------- OCIE1B : Timer/counter1 Output Compare B Match Interrupt Enable
;					  |+-------- OCE1A : Timer/counter1 Output Compare A Match Interrupt Enable
;					  +--------- TOIE1 : Timer/Counter1 Overflow Interrupt Enable
;-----------------------------------------------------------------------------
.equ		Stand_TCCR0	=	0b00000011
;	Timer/Counter0 Control Register	       |||
;					       ||+-- CS00
;					       |+--- CS01
;					       +---- CS02
;		CS02	CS01	CS00
;		0	0	0	timer/counter0 is stopped
;		0	0	1	clk
;		0	1	0	clk/8
;		0	1	1	clk/64
;		1	0	0	clk/256
;		1	0	1	clk/1024
;		1	1	0	external pin T0; falling edge
;		1	1	1	external pin T0; rising edge
;-----------------------------------------------------------------------------
.equ		Stand_PADir	=	0b00000000
;					  ||||||||
;					  |||||||+-- input : 
;					  ||||||+--- input : 
;					  |||||+---- input : 
;					  ||||+----- input : 
;					  |||+------ input : 
;					  ||+------- input : 
;					  |+-------- input : 
;					  +--------- input : 
.equ		Stand_PAData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PBDir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output : 
;					  ||||||+--- output : 
;					  |||||+---- output : 
;					  ||||+----- output : 
;					  |||+------ output : 
;					  ||+------- output : 
;					  |+-------- output : RXD LED
;					  +--------- output : TXD LED
.equ		Stand_PBData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PCDir	=	0b00000000
;					  ||||||||
;					  |||||||+-- input : 
;					  ||||||+--- input : 
;					  |||||+---- input : 
;					  ||||+----- input : 
;					  |||+------ input : 
;					  ||+------- input : 
;					  |+-------- input : 
;					  +--------- input : 
.equ		Stand_PCData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PDDir	=	0b11111110
;					  ||||||||
;					  |||||||+-- input :	[RXD]
;					  ||||||+--- output :	[TXD]
;					  |||||+---- output : 
;					  ||||+----- output : 
;					  |||+------ output : 
;					  ||+------- output : 
;					  |+-------- output : 
;					  +--------- output : 
.equ		Stand_PDData	=	0b11111111
;-----------------------------------------------------------------------------
	.dseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  DATA  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
;						; CONST
.equ	Cr		=	0x0D		;  Cr = 0DH ;
.equ	Lf		=	0x0A		;  Lf = 0AH ;
;						;
.equ	TextLineSize	= 	32		;  TextLineSize = 32 ;
.equ	NumbConv	=	32		;  NumbConv = 32 ;
.equ	TXDBuffSize	=	16		;  TXDBuffSize = 16 ;
.equ	RXDBuffSize	=	8		;  RXDBuffSize = 8 ;
;						;
;						; VAR
TextLine:	.byte	TextLineSize		;  TextLine : ARRAY [ 0 .. TextLineSize ] OF CHAR ;
TextLineInx:	.byte	1			;  TextLineInx : BYTE ;
Number:		.byte	NumbConv		;  Number : ARRAY [ 0 .. NumbConv - 1 ] OF CHAR ;
BinaryNumb:	.byte	2			;  BinaryNumb : INTEGER ;
;-----------------------------------------------------------------------------
	.cseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  CODE  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
	.org	0				;
	rjmp	ResetProcessor			;
;-----------------------------------------------------------------------------
	.org	INT0addr			;External Interrupt0 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT1addr			;External Interrupt1 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ICP1addr			;Input Capture1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Aaddr			;Output Compare1A Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Baddr			;Output Compare1B Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF1addr			;Overflow1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF0addr			;Overflow0 Interrupt Vector Address
	rjmp	TC0OvfInterrupt			;
;-----------------------------------------------------------------------------
	.org	SPIaddr				;SPI Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	URXCaddr			;UART Receive Complete Interrupt Vector Address
	rjmp	USARTRXDInterrupt		;
;-----------------------------------------------------------------------------
	.org	UDREaddr			;UART Data Register Empty Interrupt Vector Address
	rjmp	USARTDataRegEmptInterrupt	;
;-----------------------------------------------------------------------------
	.org	UTXCaddr			;UART Transmit Complete Interrupt Vector Address
	rjmp	USARTTXDInterrupt		;
;-----------------------------------------------------------------------------
	.org	ACIaddr				;Analog Comparator Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT2addr			;External Interrupt2 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC0addr				;Output Compare0 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ERDYaddr			;EEPROM Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	SPMaddr				;SPM complete Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
; definicje na potrzeby biblioteki obslugi UART
.equ	USART_Speed	= 47
;	USART BAUD Rate Generator fosc=7.3728 MHz , 9600 bps
.equ	USART_LED_Trace	= 1
.equ	RXDLED		= 7
.equ	TXDLED		= 6
.equ	USARTTracePort	= PORTD
.equ	LEDOnState	= 0
.include "usartlib.inc"
;-----------------------------------------------------------------------------
.include "arithmetic.inc"
;-----------------------------------------------------------------------------
.include "util.inc"
;-----------------------------------------------------------------------------
HardwareInit :					;PROCEDURE HardwareInit ;
;*************					;BEGIN
	ldi	acc,Stand_TIMSK			; [TIMSK] := Stand_TIMSK ;
	out	TIMSK,acc			;
	ldi	acc,Stand_TCCR0			; [TCCR0] := Stand_TCCR0 ;
	out	TCCR0,acc			;
	ret					;END (* HardwareInit *) ;
;-----------------------------------------------------------------------------
EnvirInit :					;PROCEDURE EnvirInit ;
;**********					;BEGIN
	ldi	acc,Stand_PADir			; [DDRA] := Stand_PADir ;
	out	DDRA,Acc			;
	ldi	acc,Stand_PAData		; [PORTA] := Stand_PAData ;
	out	PORTA,acc			;
	ldi	acc,Stand_PBDir			; [DDRB] := Stand_PBDir ;
	out	DDRB,Acc			;
	ldi	acc,Stand_PBData		; [PORTB] := Stand_PBData ;
	out	PORTB,acc			;
	ldi	acc,Stand_PCDir			; [DDRC] := Stand_PCDir ;
	out	DDRC,Acc			;
	ldi	acc,Stand_PCData		; [PORTC] := Stand_PCData ;
	out	PORTC,acc			;
	ldi	acc,Stand_PDDir			; [DDRD] := Stand_PDDir ;
	out	DDRD,Acc			;
	ldi	acc,Stand_PDData		; [PORTD] := Stand_PDData ;
	out	PORTD,acc			;
	ldz	USART_Speed			; USARTHardwareInit ( USART_Speed ) ;
	rcall	USARTHardwareInit		;
	ret					;END (* EnvirInit *) ;
;-----------------------------------------------------------------------------
SoftwareInit :					;PROCEDURE SoftwareInit ;
;*************					;BEGIN (* SoftwareInit *)
	rcall	USARTSoftwareInit		; USARTSoftwareInit ( ) ;
	ret					;END SoftwareInit ;
;-----------------------------------------------------------------------------
TC0OvfInterrupt :				;PROCEDURE TC0OvfInterrupt ;
;****************				;BEGIN (* TC0OvfInterrupt *)
	pushf					;
.if USART_LED_Trace == 1
	rcall	USARTTimeService		; USARTTimeService ( ) ;
.endif
	popf					;
	reti					;END (* TC0OvfInterrupt *) ;
;-----------------------------------------------------------------------------
BinToStrBin	:				;PROCEDURE BinToStrBin ( VAR Number[Z] : ARRAY OF CHAR ;
;****************				;       		     Value[param] : BYTE ) ;
						; VAR
						;  Loop [acc2] : BYTE ;
						;BEGIN
	ldi	acc2,0				; FOR Loop := 0 TO 7 DO
BiToSB_1:					; BEGIN
	rol	param				;  Number[Loop] := Value.Loop+'0' ;
	ldi	acc,0				;
	adc	acc,acc				;
	subi	acc,-'0'			;
	st	z+,acc				;
	cpi	acc2,7				; END (* FOR *) ;
	breq	BiToSB_0			;
	inc	acc2				;
	rjmp	BiToSB_1			;
BiToSB_0:					;
	ldi	acc,0				; Number[8] := 0 ;
	st	z,acc				;
	ret					;END (* BinToStrBin *) ;
;-----------------------------------------------------------------------------
HexConvTable :	.db				"0123456789ABCDEF"
ConvToHex	:				;PROCEDURE ConvToHex (Val[param]:BYTE):WORD[x] ;
;****************				;BEGIN (* ConvToHex *)
	pushz					;
	push	param				;
	ldz	HexConvTable<<1			; xh:=HexConvTable[Val DIV 16];
	swap	param				;
	andi	param,0x0F			;
	clr	acc				;
	add	zl,param			;
	adc	zh,acc				;
	lpm					;
	mov	xh,r0				;
	pop	param				; xl:=HexConvTable[Val MOD 16];
	ldz	HexConvTable<<1			;
	andi	param,0x0F			;
	clr	acc				;
	add	zl,param			;
	adc	zh,acc				;
	lpm					;
	mov	xl,r0				;
	popz					;
	ret					;END (* ConvToHex *) ;
;-----------------------------------------------------------------------------
WordToHexString:				;PROCEDURE WordToHexString ( CardV[z] : POINTER TO WORD ;
;******************				;	                 VAR String[x] : ARRAY OF CHAR ;
;						;			     ConvWidth[param] : BYTE ;
;						;			     EmptyCh[acc] : CHAR ) ;
;						; VAR
;						;  Inx[acc2] : BYTE ;
;						;  Mod16[acc] : BYTE ;
;						;BEGIN (* WordToHexString *)
	push	zl				;
	push	zh				;
	mov	zl,xl				;
	mov	zh,xh				;
	ldi	acc2,0				; FOR Inx := 0 TO ConvWidth DO
UCToHeSt_0 :					;
	st	z+,acc				;  String [ Inx ] := EmptyCh ;
	inc	acc2				; END (* FOR *) ;
	cp	acc2,param			;
	brne	UCToHeSt_0			;
	mov	zl,xl				; String[ConvWidth] := CHR ( 0 ) ;
	mov	zh,xh				;
	add	zl,param			;
	ldi	acc,0				;
	adc	zh,acc				;
	st	z,acc				;
	dec	param				; Inx := ConvWidth - 1 ;
	mov	acc2,param			;
	pop	zh				; Argument1 := CardV ^ ;
	pop	zl				;
	ld	acc,z+				;
	mov	Argument1LL,acc			;
	ld	acc,z				;
	mov	Argument1LH,acc			;
UCToHeSt_1 :					; LOOP
	mov	acc,Argument1LL			;  Mod16 := Argument1 MOD 16 ;
	andi	acc,0x0F			;
	ldz	HexConvTable<<1			;  String[Inx]:=HexConvTable[Mod16];
	add	zl,acc				;
	ldi	acc,0				;
	adc	zh,acc				;
	lpm					;
	mov	zl,xl				;
	mov	zh,xh				;
	add	zl,acc2				;
	ldi	acc,0				;
	adc	zh,acc				;
	st	z,r0				;
	lsr	Argument1LH			;  Argument1 := Argument1 >> 4 ;
	ror	Argument1LL			;
	lsr	Argument1LH			;
	ror	Argument1LL			;
	lsr	Argument1LH			;
	ror	Argument1LL			;
	lsr	Argument1LH			;
	ror	Argument1LL			;
	cpi	acc2,0				;  IF (Inx=0) OR (Argument1=0) THEN
	breq	UCToHeSt_2			;
	mov	acc,Argument1LL			;
	or	acc,Argument1LH			;
	breq	UCToHeSt_2			;   EXIT ;
						;  END (* IF *) ;
	dec	acc2				;  Inx := Inx - 1 ;
	rjmp	UCToHeSt_1			; END (* LOOP *) ;
UCToHeSt_2 :					;
	ret					;END(* WordToHexString *) ;
;-----------------------------------------------------------------------------
WordToString	:				;PROCEDURE WordToString ( CardV[z] : POINTER TO WORD ;
;******************				;	    	      VAR String[x] : ARRAY OF CHAR ;
;						;			  ConvWidth[param] : BYTE ;
;						;			  EmptyCh[acc] : CHAR ) ;
;						; VAR
;						;  Inx[acc2] : BYTE ;
;						;  ModV[WorkRegLL] : BYTE ;
;						;BEGIN (* WordToString *)
	push	zl				;
	push	zh				;
	mov	zl,xl				;
	mov	zh,xh				;
	ldi	acc2,0				; FOR Inx := 0 TO ConvWidth DO
UCToSt_0 :					;
	st	z+,acc				;  String [ Inx ] := EmptyCh ;
	inc	acc2				; END (* FOR *) ;
	cp	acc2,param			;
	brne	UCToSt_0			;
	mov	zl,xl				; String [ ConvWidth ] := CHR(0) ;
	mov	zh,xh				;
	add	zl,param			;
	ldi	acc,0				;
	adc	zh,acc				;
	st	z,acc				;
	dec	param				; Inx := ConvWidth - 1 ;
	mov	acc2,param			;
	pop	zh				; Argument1 := CardV ^ ;
	pop	zl				;
	ld	acc,z+				;
	mov	Argument1LL,acc			;
	ld	acc,z				;
	mov	Argument1LH,acc			;
	ldi	acc,0				;
	mov	Argument1HL,acc			;
	mov	Argument1HH,acc			;
UCToSt_1 :					; LOOP
	push	acc2				;
	rcall	DivModArg1By10			;  Argument2 := Argument1 DIV 10 ;
						;  ModV := Argument1 MOD 10 ;
	mov	acc,WorkRegLL			;  String[Inx]:=CHR(ModV+ORD('0'));
	ldi	acc2,'0'			;
	add	acc,acc2			;
	pop	acc2				;
	mov	zl,xl				;
	mov	zh,xh				;
	add	zl,acc2				;
	ldi	param,0				;
	adc	zh,param			;
	st	z,acc				;
	mov	Argument1LL,Argument2LL		;  Argument1 := Argument2 ;
	mov	Argument1LH,Argument2LH		;
	cpi	acc2,0				;  IF (Inx=0) OR (Argument1=0) THEN
	breq	UCToSt_2			;
	mov	acc,Argument1LL			;
	or	acc,Argument1LH			;
	breq	UCToSt_2			;   EXIT ;
						;  END (* IF *) ;
	dec	acc2				;  Inx := Inx - 1 ;
	rjmp	UCToSt_1			; END (* LOOP *) ;
UCToSt_2 :					;
	ret					;END (* WordToString *) ;
;-----------------------------------------------------------------------------
IntToString	:				;PROCEDURE IntToString ( IntV[z]          : POINTER TO INTEGER ;
;****************				;		     VAR String[x]        : ARRAY OF CHAR ;
;						;                        ConvWidth[param] : BYTE ;
;						;                        EmptyCh[acc]     : CHAR ) ;
;						;VAR
;						; Inx[acc2] : BYTE ;
;						; ModValue[WorkRegLL] : BYTE ;
;						; Minus[stack] : BOOLEAN ;
;						;BEGIN (* IntToString *)
	push	zl				;
	push	zh				;
	mov	zl,xl				;
	mov	zh,xh				;
	ldi	acc2,0				; FOR Inx := 0 TO ConvWidth DO
SCToS_0 :					;  BEGIN
	st	z+,acc				;   String [ Inx ] := EmptyCh ;
	inc	acc2				;  END (* FOR *) ;
	cp	acc2,param			;
	brne	SCToS_0				;
	mov	zl,xl				; String [ ConvWidth ] := CHR ( 0 ) ;
	mov	zh,xh				;
	add	zl,param			;
	ldi	acc,0				;
	adc	zh,acc				;
	st	z,acc				;
	dec	param				; Inx := ConvWidth - 1 ;
	mov	acc2,param			;
	pop	zh				; Argument1 := IntV ^ ;
	pop	zl				;
	ld	Argument1LL,z+			;
	ld	Argument1LH,z+			;
	ldi	acc,0				;
	mov	Argument1HL,acc			;
	mov	Argument1HH,acc			;
	ldi	param,0				; Minus := FALSE ;
	mov	acc,Argument1LH			; IF Argument1 < 0 THEN
	andi	acc,0x80			;
	breq	SCToS_3				;  BEGIN
	rcall	NegArg1				;   Argument1 := - Argument1 ;
	ldi	param,1				;   Minus := TRUE ;
SCToS_3 :					;  END (* IF *) ;
	ldi	acc,0				;
	mov	Argument1HL,acc			;
	mov	Argument1HH,acc			;
	push	param				;
SCToS_1 :					; LOOP
	push	acc2				;
	rcall	DivModArg1By10			;  Argument2 := Argument1 DIV 10 ;
						;  ModV := Argument1 MOD 10 ;
	mov	acc,WorkRegLL			;  String [ Inx ] := CHR ( ModV + ORD ( '0' ) ) ;
	ldi	acc2,'0'			;
	add	acc,acc2			;
	pop	acc2				;
	mov	zl,xl				;
	mov	zh,xh				;
	add	zl,acc2				;
	ldi	param,0				;
	adc	zh,param			;
	st	z,acc				;
	mov	Argument1LL,Argument2LL		;  Argument1 := Argument2 ;
	mov	Argument1LH,Argument2LH		;
	cpi	acc2,0				;  IF ( Inx = 0 ) OR ( Argument1 = 0 ) THEN
	breq	SCToS_2				;
	mov	acc,Argument1LL			;
	or	acc,Argument1LH			;
	breq	SCToS_2				;   BEGIN
						;    EXIT ;
						;   END (* IF *) ;
	dec	acc2				;  Inx := Inx - 1 ;
	rjmp	SCToS_1				; END (* LOOP *) ;
SCToS_2 :					;
	pop	acc				; IF Minus THEN
	cpi	acc,0				;
	breq	SCToS_4				;  BEGIN
	cpi	acc2,1				;   IF Inx >= 1 THEN
	brcs	SCToS_5				;    BEGIN
	dec	acc2				;     Inx := Inx - 1 ;
	mov	zl,xl				;     String [ Inx ] := '-' ;
	mov	zh,xh				;
	add	zl,acc2				;
	ldi	acc,0				;
	adc	zh,acc				;
	ldi	acc,'-'				;
	st	z,acc				;
SCToS_5 :					;    END (* IF *) ;
SCToS_4 :					;  END (* IF *) ;
	ret					;END IntToString ;
;-----------------------------------------------------------------------------
NextChar :					;FUNCTION NextChar ( ) : CHAR [acc] ;
;*********					;VAR
;						; Ch[acc]:CHAR;
;						;BEGIN (* NextChar *)
NeCh_1	:					; REPEAT
NeCh_0	:					;  REPEAT
	rcall	SerialDataPresent		;  UNTIL SerialDataPresent ( ) ;
	brcc	NeCh_0				;
	rcall	GetSerial			;  Ch := GetSerial ( ) ;
	cpi	acc,' '				; UNTIL Ch <> ' ' ;
	breq	NeCh_1				;
	ret					; NextChar := Ch ;
						;END (* NextChar *) ;
;-----------------------------------------------------------------------------
SendNewLine :					;PROCEDURE SendNewLine ( ) ;
;************					;BEGIN
	ldi	param,Cr			; SendSerial ( Cr ) ;
	rcall	SendSerial			;
	ldi	param,Lf			; SendSerial ( Lf ) ;
	rcall	SendSerial			;
	ret					;END SendNewLine ;
;-----------------------------------------------------------------------------
AddCh	:					;PROCEDURE AddCh ( Ch[param] : CHAR ) ;
;********					;BEGIN (* AddCh *)
	lds	acc2,TextLineInx		; IF TextLineInx < TextLineSize THEN
	cpi	acc2,TextLineSize		;
	brcc	Adc_0				;  BEGIN
	ldz	TextLine			;   TextLine [ TextLineInx ] := Ch ;
	add	zl,acc2				;
	ldi	acc2,0				;
	adc	zh,acc2				;
	st	z,param				;
	ldz	TextLineInx			;   TextLineInx := TextLineInx + 1 ;
	ld	acc,z				;
	inc	acc				;
	st	z,acc				;
Adc_0	:					;  END (* IF *) ;
	ret					;END (* AddCh *) ;
;-----------------------------------------------------------------------------
ReadTextLine :					;PROCEDURE ReadTextLine ( ) ;
;************					;VAR
;						; Ch [acc] : CHAR ;
;						;BEGIN
	ldi	acc,0				; TextLineInx := 0 ;
	sts	TextLineInx,acc			;
ReTeL_0	:					; LOOP
	rcall	NextChar			;  Ch := NextChar ( ) ;
	cpi	acc,' '				;  IF Ch >= ' ' THEN
	brcs	ReTeL_5				;   BEGIN
	push	acc				;
	mov	param,acc			;    SendSerial ( Ch ) ;
	rcall	SendSerial			;
	pop	acc				;
ReTeL_5	:					;   END (* IF *) ;
	cpi	acc,Cr				;  IF Ch = Cr THEN
	brne	ReTeL_1				;   BEGIN
	ldi	param,0				;    AddCh ( 0 ) ;
	rcall	AddCh				;
	rcall	SendNewLine			;    SendNewLine ( ) ;
	ret					;    RETURN ;
ReTeL_1	:					;   END (* IF *) ;
	mov	param,acc			;  AddCh ( Ch ) ;
	rcall	AddCh				;
	rjmp	ReTeL_0				; END (* LOOP *) ;
						;END ReadTextLine ;
;-----------------------------------------------------------------------------
Digit	:					;PROCEDURE Digit ( Ch [acc] : CHAR ) : BOOLEAN ;
;********					;BEGIN (* Digit *)
	cpi	acc,'0'				; IF ( Ch < '0' ) OR ( Ch > '9' ) THEN
	brcs	Dig_0				;
	cpi	acc,'9'+1			;
	brcs	Dig_1				;
Dig_0	:					;  BEGIN
	clc					;   Digit := FALSE ;
	ret					;   RETURN ;
Dig_1	:					;  END (* IF *) ;
	sec					; Digit := TRUE ;
	ret					;END (* Digit *) ;
;-----------------------------------------------------------------------------
StringToBin :					;FUNCTION StringToBin ( ) : BOOLEAN[C] ;
;************					;VAR
;						; Minus [acc2] : BOOLEAN ;
;						; ChPtr [X] : ^ CHAR ;
;						; Ch [acc] : CHAR ;
;						;BEGIN
	ldi	acc2,0				; Minus := FALSE ;
	ldx	TextLine			; ChPtr := ADR ( TextLine [ 0 ] ) ;
	ld	acc,x				; IF ChPtr ^ = '-' THEN
	cpi	acc,'-'				;
	brne	StrTB_0				;  BEGIN
	ldi	acc2,1				;   Minus := TRUE ;
	adiw	xl,1				;   ChPtr := ChPtr + 1 ;
	rjmp	StrTB_1				;  END (* IF ... *)
StrTB_0	:					; ELSE
						;  BEGIN
	cpi	acc,'+'				;   IF ChPtr ^ = '+' THEN
	brne	StrTB_2				;    BEGIN
	adiw	xl,1				;     ChPtr := ChPtr + 1 ;
StrTB_2	:					;    END (* IF *) ;
StrTB_1	:					;  END (* IF ... ELSE *) ;
	ldi	acc,0				; Argument1 := 0 ;
	mov	Argument1LL,acc			;
	mov	Argument1LH,acc			;
	mov	Argument1HL,acc			;
	mov	Argument1HH,acc			;
	ld	acc,x+				; Ch := ChPtr ^ ;
						; ChPtr := ChPtr + 1 ;
StrTB_3	:					; WHILE Ch <> CHR ( 0 ) DO
	cpi	acc,0				;
	breq	StrTB_4				;  BEGIN
	rcall	Digit				;   IF NOT Digit ( Ch ) THEN
	brcs	StrTB_5				;    BEGIN
	ret					;     StringToBin := FALSE ;
						;     RETURN ;
StrTB_5	:					;    END (* IF *) ;
	cpi	acc2,0				;   IF Minus THEN
	breq	StrTB_6				;    BEGIN
	rcall	MultAgr1And10			;     Argument1 := Argument1 * 10 - ( ORD ( Ch ) - '0' ) ;
	subi	acc,'0'				;
	sub	Argument1LL,acc			;
	ldi	acc,0				;
	sbc	Argument1LH,acc			;
	rjmp	StrTB_7				;    END (* IF ... *)
StrTB_6	:					;   ELSE
						;    BEGIN
	rcall	MultAgr1And10			;     Argument1 := Argument1 * 10 + ( ORD ( Ch ) - '0' ) ;
	subi	acc,'0'				;
	add	Argument1LL,acc			;
	ldi	acc,0				;
	adc	Argument1LH,acc			;
StrTB_7	:					;    END (* IF ... ELSE *) ;
	ld	acc,x+				;   Ch := ChPtr ^ ;
						;   ChPtr := ChPtr + 1 ;
	rjmp	StrTB_3				;
StrTB_4	:					;  END (* WHILE *) ;
	sec					; StringToBin := TRUE ;
	ret					;END (* StringToBin *) ;
;-----------------------------------------------------------------------------
Conwersion	:				;PROCEDURE Conwersion ( ) ;
;****************				;BEGIN (* Conwersion *)
	ldz	DecU2Txt<<1			; SendSerialFlashTxt ( DecU2Txt ) ;
	rcall	SendSerialFlashTxt		;
	ldz	BinaryNumb			; IntToString ( BinaryNumb , Number , 6 , ' ' ) ;
	ldx	Number				;
	ldi	param,6				;
	ldi	acc,' '				;
	rcall	IntToString			;
	ldz	Number				; SendSerialRAMTxt ( Number ) ;
	rcall	SendSerialRAMTxt		;
	ldz	DecNBTxt<<1			; SendSerialFlashTxt ( DecNBTxt ) ;
	rcall	SendSerialFlashTxt		;
	ldz	BinaryNumb			; WordToString ( BinaryNumb , Number , 6 , ' ' ) ;
	ldx	Number				;
	ldi	param,6				;
	ldi	acc,' '				;
	rcall	WordToString			;
	ldz	Number				; SendSerialRAMTxt ( Number ) ;
	rcall	SendSerialRAMTxt		;
	ldz	BinTxt<<1			; SendSerialFlashTxt ( BinTxt ) ;
	rcall	SendSerialFlashTxt		;
	ldz	Number				; BinToStrBin ( Number , Hi ( BinaryNumb ) ) ;
	lds	param,BinaryNumb+1		;
	rcall	BinToStrBin			;
	ldz	Number+8			; BinToStrBin ( Number+8 , Lo ( BinaryNumb ) ) ;
	lds	param,BinaryNumb		;
	rcall	BinToStrBin			;
	ldz	Number				; SendSerialRAMTxt ( Number ) ;
	rcall	SendSerialRAMTxt		;
	ldz	HexTxt<<1			; SendSerialFlashTxt ( HexTxt ) ;
	rcall	SendSerialFlashTxt		;
	ldz	BinaryNumb			; WordToHexString ( BinaryNumb , Number , 4 , '0' ) ;
	ldx	Number				;
	ldi	param,4				;
	ldi	acc,'0'				;
	rcall	WordToHexString			;
	ldz	Number				; SendSerialRAMTxt ( Number ) ;
	rcall	SendSerialRAMTxt		;
	ret					;END (* Conwersion *) ;
;-----------------------------------------------------------------------------
ResetProcessor	:				;
	ldi	acc,HIGH(RAMEND)		;
	out	SPH,acc				;
	ldi	acc,LOW(RAMEND)			;
	out	SPL,acc				;
						;BEGIN (* Main *)
	rcall	ClearRAM			; ClearRAM ( ) ;
	rcall	ClearRegs			; ClearRegs ( ) ;
	rcall	HardwareInit			; HardwareInit ( ) ;
	rcall	EnvirInit			; EnvirInit ( ) ;
	rcall	SoftwareInit			; SoftwareInit ( ) ;
	sei					; EI ( ) ;
	ldz	HelloText<<1			; SendSerialFlashTxt ( HelloText ) ;
	rcall	SendSerialFlashTxt		;
Main_0	:					; LOOP
	ldz	PromptText<<1			;  SendSerialFlashTxt ( PromptText ) ;
	rcall	SendSerialFlashTxt		;
	rcall	ReadTextLine			;  ReadTextLine ( ) ;
	rcall	StringToBin			;  IF StringToBin ( ) THEN
	brcc	Main_1				;   BEGIN
	ldz	BinaryNumb			;    BinaryNumb := Arg1 ;
	st	z+,Argument1LL			;
	st	z,Argument1LH			;
	rcall	Conwersion			;    Conwersion ( ) ;
	rjmp	Main_2				;   END (* IF ... *)
Main_1	:					;   ELSE
						;   BEGIN
	ldz	ErrorText<<1			;    SendSerialFlashTxt ( ErrorText ) ;
	rcall	SendSerialFlashTxt		;
Main_2	:					;   END (* IF *) ;
	rjmp	Main_0				; END (* LOOP *) ;
						;END (* Main *).
;-----------------------------------------------------------------------------
HelloText:	.db	Cr,Lf,"Program prezentacji konwersji liczb",Cr,Lf,Cr,Lf,0
PromptText:	.db	Cr,Lf,"Wprowadz liczbe (moze byc ze znakiem):",Cr,Lf,0,0
ErrorText:	.db	Cr,Lf,"Blad zapisu liczby.",Cr,Lf,0
DecNBTxt:	.db	Cr,Lf,"Postac dziesietna (NB) : ",0
DecU2Txt:	.db	Cr,Lf,"Postac dziesietna (U2) : ",0
BinTxt:		.db	Cr,Lf,"Postac binarna         : ",0
HexTxt:		.db	Cr,Lf,"Postac szesnastkowa    : ",0
.exit
