;
;*******************************************************
;*******************************************************
;**                                                   **
;**         Modul obslugi wyswietlacza LCD            **
;**         zgodnego ze standardem HD44780            **
;**                                                   **
;**           Napisal: Andrzej Pawluczuk              **
;**                                                   **
;**             Mikrokontroler: AVR                   **
;**                                                   **
;**               Bialystok, 2005                     **
;**                                                   **
;*******************************************************
;*******************************************************
;
.message	"***>>> Dolaczenie bibioteki obslugi LCD"
;-----------------------------------------------------------------------------
;
;	Modul obslugi wyswietlaczy LCD uzywa nastepujacych parametrow:
;
;	lcd_E_port	- okresla port, do ktorego przylaczony jest
;			  sygnal E modulu LCD
;	lcd_E_pin	- okresla pin w porcie okreslonym przez lcd_E_port
;			  do ktorego przylaczony jest sygnal E modulu LCD
;	lcd_RS_port	- okresla port, do ktorego przylaczony jest
;			  sygnal RS modulu LCD
;	lcd_RS_pin	- okresla pin w porcie okreslonym przez lcd_RS_port
;			  do ktorego przylaczony jest sygnal RS modulu LCD
;	lcd_RW_port	- okresla port, do ktorego przylaczony jest
;			  sygnal RW modulu LCD
;	lcd_RW_pin	- okresla pin w porcie okreslonym przez lcd_RW_port
;			  do ktorego przylaczony jest sygnal RW modulu LCD
;	lcd_Data_port	- okresla port, do ktorego przylaczona jest
;			  szyna danych modulu LCD; w przypadku interfejsu
;			  8-bitowego szyna danych modulu LCD jest szyna danych
;			  portu (odpowiadajace sobie bity maja te same numery);
;			  w przypadku interfejsu 4-bitowego istnieje moliwosc
;			  dowolnego okreslenia (w obrebie jednego portu)
;			  przyporzadkowania bitow szyny danych modulu LCD i
;			  portu danych,
;	lcd_Interf4b	- okresla, czy uyty modul LCD jest z 4-bitowa
;			  szyna danych (w przeciwnym wypadku intefejs jest
;			  8-bitowy),
;	lcdD0		- okresla polozenie w porcie najmlodszego bitu szyny danych
;			  dla intefejsu 4-bitowego
;	lcdD1		- okresla polozenie w porcie kolejnego bitu szyny danych
;			  dla intefejsu 4-bitowego
;	lcdD2		- okresla polozenie w porcie kolejnego bitu szyny danych
;			  dla intefejsu 4-bitowego
;	lcdD3		- okresla polozenie w porcie najstarszego bitu szyny danych
;			  dla intefejsu 4-bitowego
;	V1x16_as_2x8	- okresla przypadek modulu LCD majacego dwa wiersze po 8 znakow
;			  wyswietlanych w formie jednego wiersza z 16-oma znakami
;	lcd_as1line	- okresla modul zawierasjacy jeden wiersz (w przeciwnym wypadku
;			  zawierajacy dwa wiersze); UWAGA parametr "V1x16_as_2x8" ma wiekszy
;			  priorytet w stosunku do parametru "lcd_as1line"
;-----------------------------------------------------------------------------
.ifndef lcd_E_port
	.error	"*****> Blad: brak definicji stalej lcd_E_port"
	.exit
.endif
.ifndef lcd_RS_port
	.error	"*****> Blad: brak definicji stalej lcd_RS_port"
	.exit
.endif
.ifndef lcd_RW_port
    	.error	"*****> Blad: brak definicji stalej lcd_RW_port"
	.exit
.endif
.ifndef lcd_E_pin
	.error	"*****> Blad: brak definicji stalej lcd_E_pin"
	.exit
.endif
.ifndef lcd_RS_pin
	.error	"*****> Blad: brak definicji stalej lcd_RS_pin"
	.exit
.endif
.ifndef lcd_RW_pin
	.error	"*****> Blad: brak definicji stalej lcd_RW_pin"
	.exit
.endif
.ifdef lcd_Interf4b
	.ifndef	lcdD0
		.error	"*****> Blad: interfejs 4 bitowy - brak okreslenia szyny danych"
		.exit
	.endif
	.ifndef	lcdD1
		.error	"*****> Blad: interfejs 4 bitowy - brak okreslenia szyny danych"
		.exit
	.endif
	.ifndef	lcdD2
		.error	"*****> Blad: interfejs 4 bitowy - brak okreslenia szyny danych"
		.exit
	.endif
	.ifndef	lcdD3
		.error	"*****> Blad: interfejs 4 bitowy - brak okreslenia szyny danych"
		.exit
	.endif
.endif
.ifdef	V1x16_as_2x8
	.message	"***LCD --> Modul LCD: 1 wiersz x 16 jako 2 wiersze x 8"
.endif
;
;-----------------------------------------------------------------------------
.ifdef lcd_Interf4b
	.message	"***LCD --> interfejs 4-bitowy"
.else
	.message	"***LCD --> interfejs 8-bitowy"
.endif
;-----------------------------------------------------------------------------
.equ	_lcd_DataDir_port	= lcd_Data_port - 1
.equ	_lcd_DataPin_port	= lcd_Data_port - 2
.equ	_lcd_E_Dir_port		= lcd_E_port - 1
.equ	_lcd_RS_Dir_port	= lcd_RS_port - 1
.equ	_lcd_RW_Dir_port	= lcd_RW_port - 1
;-----------------------------------------------------------------------------
.ifdef	V1x16_as_2x8
	.dseg
;						;VAR
_CharPos :	.byte	1			; _CharPos : BYTE ;
.endif
;-----------------------------------------------------------------------------
	.cseg
;-----------------------------------------------------------------------------
InitLCDEnvir :					;PROCEDURE InitLCDEnvir ( ) ;
;*************					;BEGIN (* InitLCDEnvir *)
	sbi	_lcd_E_Dir_port,lcd_E_pin	; [lcd_E_Dir_port . lcd_E_pin] := [output] ;
	sbi	_lcd_RS_Dir_port,lcd_RS_pin	; [lcd_RS_Dir_port . lcd_RS_pin] := [output] ;
	sbi	_lcd_RW_Dir_port,lcd_RW_pin	; [lcd_RW_Dir_port . lcd_RW_pin] := [output] ;
	cbi	lcd_E_port,lcd_E_pin		; [E=0] ;
	rcall	_SetOutputDataBus		; _SetOutputDataBus ( ) ;
.ifdef	V1x16_as_2x8
	clr	acc				; _CharPos := 0 ;
	sts	_CharPos,acc			;
.endif
	ret					;END (* InitLCDEnvir *) ;
;-----------------------------------------------------------------------------
_SetOutputDataBus :				;PROCEDURE _SetOutputDataBus ( ) ;
;******************				;BEGIN (* _SetOutputDataBus *)
.ifdef lcd_Interf4b
	sbi	_lcd_DataDir_port,lcdD0		; [lcd_DataDir_port . lcdD0 ] := [output] ;
	sbi	_lcd_DataDir_port,lcdD1		; [lcd_DataDir_port . lcdD1 ] := [output] ;
	sbi	_lcd_DataDir_port,lcdD2		; [lcd_DataDir_port . lcdD2 ] := [output] ;
	sbi	_lcd_DataDir_port,lcdD3		; [lcd_DataDir_port . lcdD3 ] := [output] ;
.else
	ldi	acc,0xFF			; [lcd_DataDir_port] := 0xFF ;
	out	_lcd_DataDir_port,acc		;
.endif
	ret					;END (* _SetOutputDataBus *) ;
;-----------------------------------------------------------------------------
_SetInputDataBus :				;PROCEDURE _SetInputDataBus ( ) ;
;******************				;BEGIN (* _SetInputDataBus *)
.ifdef lcd_Interf4b
	cbi	_lcd_DataDir_port,lcdD0		; [lcd_DataDir_port . lcdD0 ] := [input] ;
	cbi	_lcd_DataDir_port,lcdD1		; [lcd_DataDir_port . lcdD1 ] := [input] ;
	cbi	_lcd_DataDir_port,lcdD2		; [lcd_DataDir_port . lcdD2 ] := [input] ;
	cbi	_lcd_DataDir_port,lcdD3		; [lcd_DataDir_port . lcdD3 ] := [input] ;
.else
	ldi	acc,0				; [lcd_DataDir_port] := 0 ;
	out	_lcd_DataDir_port,acc		;
.endif
	ret					;END (* _SetInputDataBus *) ;
;-----------------------------------------------------------------------------
_LCDDelay :					;PROCEDURE _LCDDelay ( ) ;
;*********					;BEGIN (* _LCDDelay *)
      	nop					;
	nop					;
	nop					;
	nop					;
	nop					;
	ret					;END (* _LCDDelay *) ;
;-----------------------------------------------------------------------------
.ifdef lcd_Interf4b
_SetDataBus :					;PROCEDURE _SetDataBus ( Data [param] : BYTE ) ;
;************					;BEGIN (* _SetDataBus *)
	cbi	lcd_Data_port,lcdD0		; [lcd_Data_port . lcdD0 ] := 0 ;
	cbi	lcd_Data_port,lcdD1		; [lcd_Data_port . lcdD1 ] := 0 ;
	cbi	lcd_Data_port,lcdD2		; [lcd_Data_port . lcdD2 ] := 0 ;
	cbi	lcd_Data_port,lcdD3		; [lcd_Data_port . lcdD3 ] := 0 ;
	sbrc	param,0				; IF Data . 0 THEN
	sbi	lcd_Data_port,lcdD0		;  [lcd_Data_port . lcdD0 ] := 1 ;
	sbrc	param,1				; IF Data . 1 THEN
	sbi	lcd_Data_port,lcdD1		;  [lcd_Data_port . lcdD1 ] := 1 ;
	sbrc	param,2				; IF Data . 2 THEN
	sbi	lcd_Data_port,lcdD2		;  [lcd_Data_port . lcdD2 ] := 1 ;
	sbrc	param,3				; IF Data . 3 THEN
	sbi	lcd_Data_port,lcdD3		;  [lcd_Data_port . lcdD3 ] := 1 ;
	ret					;END (* _SetDataBus *) ;
.endif
;-----------------------------------------------------------------------------
_EStrobe :					;PROCEDURE _EStrobe ( ) ;
;********					;BEGIN (* _EStrobe *)
	sbi	lcd_E_port,lcd_E_pin		; [E=1] ;
	rcall	_LCDDelay 			; _LCDDelay ( ) ;
	cbi	lcd_E_port,lcd_E_pin		; [E=0] ;
	ret					;END (* _EStrobe *) ;
;-----------------------------------------------------------------------------
_LCDOutC :					;PROCEDURE _LCDOutC ( Data [param] : BYTE ) ;
;********					;BEGIN (* _LCDOutC *)
	rcall	_SetOutputDataBus		; _SetOutputDataBus ( ) ;
	cbi	lcd_RS_port,lcd_RS_pin		; [RS=0] ;
	cbi	lcd_RW_port,lcd_RW_pin		; [R/W=0] ;
.ifdef lcd_Interf4b
	push	param				;
	swap	param				; _SetDataBus ( Data[4..7] ) ;
	rcall	_SetDataBus			;
	rcall	_EStrobe			; _EStrobe ( ) ;
	pop	param				; _SetDataBus ( Data[0..3] ) ;
	rcall	_SetDataBus			;
	rcall	_EStrobe			; _EStrobe ( ) ;
.else
	out	lcd_Data_port,param		; [lcd_Data_port] := Data ;
	rcall	_EStrobe			; _EStrobe ( ) ;
.endif
	ret					;END (* _LCDOutC *) ;
;-----------------------------------------------------------------------------
_LCDInC :					;PROCEDURE _LCDInC ( ) : BYTE ;
;*******					;VAR
;						; Temp[acc] : BYTE ;
;						; Temp4b[acc2] : BYTE ;
;						;BEGIN (* _LCDInC *)
	rcall	_SetInputDataBus		; _SetInputDataBus ( ) ;
	cbi	lcd_RS_port,lcd_RS_pin		; [RS=0] ;
	sbi	lcd_RW_port,lcd_RW_pin		; [R/W=1] ;
.ifdef lcd_Interf4b
	ldi	acc,0				; Temp := 0 ;
	sbi	lcd_E_port,lcd_E_pin		; [E=1] ;
	rcall	_LCDDelay			; _LCDDelay ( ) ;
	in	acc2,_lcd_DataPin_port		; Temp4b := [lcd_DataPin_port] ;
	cbi	lcd_E_port,lcd_E_pin		; [E=0] ;
	sbrc	acc2,lcdD0			; IF Temp4b . lcdD0 THEN
	sbr	acc,0x10			;  Temp . 4 := 1 ;
	sbrc	acc2,lcdD1			; IF Temp4b . lcdD1 THEN
	sbr	acc,0x20			;  Temp . 5 := 1 ;
	sbrc	acc2,lcdD2			; IF Temp4b . lcdD2 THEN
	sbr	acc,0x40			;  Temp . 6 := 1 ;
	sbrc	acc2,lcdD3			; IF Temp4b . lcdD3 THEN
	sbr	acc,0x80			;  Temp . 7 := 1 ;
	sbi	lcd_E_port,lcd_E_pin		; [E=1] ;
	rcall	_LCDDelay			; _LCDDelay ( ) ;
	in	acc2,_lcd_DataPin_port		; Temp4b := [lcd_DataPin_port] ;
	cbi	lcd_E_port,lcd_E_pin		; [E=0] ;
	sbrc	acc2,lcdD0			; IF Temp4b . lcdD0 THEN
	sbr	acc,0x01			;  Temp . 0 := 1 ;
	sbrc	acc2,lcdD1			; IF Temp4b . lcdD1 THEN
	sbr	acc,0x02			;  Temp . 1 := 1 ;
	sbrc	acc2,lcdD2			; IF Temp4b . lcdD2 THEN
	sbr	acc,0x04			;  Temp . 2 := 1 ;
	sbrc	acc2,lcdD3			; IF Temp4b . lcdD3 THEN
	sbr	acc,0x08			;  Temp . 3 := 1 ;
.else
	sbi	lcd_E_port,lcd_E_pin		; [E=1] ;
	rcall	_LCDDelay			; _LCDDelay ( ) ;
	in	acc,_lcd_DataPin_port		; Temp := [lcd_DataPin_port] ;
	cbi	lcd_E_port,lcd_E_pin		; [E=0] ;
						; _LCDInC := Temp ;
.endif
	ret					;END (* _LCDInC *) ;
;-----------------------------------------------------------------------------
_LCDOutD :					;PROCEDURE _LCDOutD ( Data [param] : BYTE ) ;
;********					;BEGIN (* _LCDOutD *)
	rcall	_SetOutputDataBus		; _SetOutputDataBus ( ) ;
	sbi	lcd_RS_port,lcd_RS_pin		; [RS=1] ;
	cbi	lcd_RW_port,lcd_RW_pin		; [R/W=0] ;
.ifdef lcd_Interf4b
	push	param				;
	swap	param				; _SetDataBus ( Data[4..7] ) ;
	rcall	_SetDataBus			;
	rcall	_EStrobe			; _EStrobe ( ) ;
	pop	param				; _SetDataBus ( Data[0..3] ) ;
	rcall	_SetDataBus			;
	rcall	_EStrobe			; _EStrobe ( ) ;
.else
	out	lcd_Data_port,param		; [lcd_Data_port] := Data ;
	rcall	_EStrobe			; _EStrobe ( ) ;
.endif
	ret					;END (* _LCDOutD *) ;
;-----------------------------------------------------------------------------
.ifdef lcd_Interf4b
_SpecialLCDOutC	:				;PROCEDURE _SpecialLCDOutC ( Data [param] : BYTE ) ;
;****************				;BEGIN (* _SpecialLCDOutC *)
	rcall	_SetOutputDataBus		; _SetOutputDataBus ( ) ;
	swap	param				; _SetDataBus ( Data[7..4] ) ;
	rcall	_SetDataBus			;
	cbi	lcd_RS_port,lcd_RS_pin		; [RS=0] ;
	cbi	lcd_RW_port,lcd_RW_pin		; [R/W=0] ;
	rcall	_EStrobe			; _EStrobe ( ) ;
	ret					;END (* _SpecialLCDOutC *) ;
.endif
;-----------------------------------------------------------------------------
_WaitAccept :					;PROCEDURE _WaitAccept ( ) ;
;***********					;BEGIN (* _WaitAccept *)
WaAcc_0	:					; WHILE ( _LCDInC ( ) & 0x80 ) = 0x80 DO
						;  BEGIN
	rcall	_LCDInC				;
	andi	acc,0x80			;
	cpi	acc,0x80			;
	breq	WaAcc_0				;  END (* WHILE *) ;
	ret					;END (* _WaitAccept *) ;
;-----------------------------------------------------------------------------
InitialiseLCD :					;PROCEDURE InitialiseLCD ( ) ;
;						; VAR
;						;  Data[param] : BYTE ;
;						;  DataPtr[z] : POINTER TO BYTE ;
;**************					;BEGIN (* InitialiseLCD *)
.ifdef lcd_Interf4b
	ldi	param,0b00100000		; _SpecialLCDOutC ( 0b00100000 ) ;  
						;    (* set : dl = 4 bits *)
	rcall	_SpecialLCDOutC			;
.endif
	ldz	LCDCtrlData<<1			; DataPtr := ADR ( LCDCtrlData ) ;
IniL_1	:					; LOOP
	lpm					;  Data := DataPtr ^ ;
	mov	param,r0			;
	adiw	zl,1				;  DataPtr := DataPtr + 1 ;
	cpi	param,0				;  IF Data = 0 THEN
	breq	IniL_0				;  BEGIN
						;   EXIT ;
						;  END (* IF *) ;
	rcall	_LCDOutC			;  _LCDOutC ( Data ) ;
	rcall	_WaitAccept			;  _WaitAccept ( ) ;
	rjmp	IniL_1				; END (* LOOP *) ;
IniL_0	:					;
.ifdef	V1x16_as_2x8
	clr	acc				; _CharPos := 0 ;
	sts	_CharPos,acc			;
.endif
;	rcall	InitGraphChars			; InitGraphChars ( ) ;
	ret					;END (* InitialiseLCD *) ;
;-----------------------------------------------------------------------------
ClrScrLCD :					;PROCEDURE ClrScrLCD ( ) ;
;**********					;BEGIN (* ClrScrLCD *)
	ldi	param,0b00000001		; _LCDOutC ( 0b00000001 ) ;
	rcall	_LCDOutC			;
	rcall	_WaitAccept			; _WaitAccept ( ) ;
.ifdef	V1x16_as_2x8
	clr	acc				; _CharPos := 0 ;
	sts	_CharPos,acc			;
.endif
	ret					;END (* ClrScrLCD *) ;
;-----------------------------------------------------------------------------
HomeLCD :					;PROCEDURE HomeLCD ( ) ;
;********					;BEGIN (* HomeLCD *)
	ldi	param,0b00000010		; _LCDOutC ( 0b00000010 ) ;
	rcall	_LCDOutC			;
	rcall	_WaitAccept			; _WaitAccept ( ) ;
.ifdef	V1x16_as_2x8
	clr	acc				; _CharPos := 0 ;
	sts	_CharPos,acc			;
.endif
	ret					;END (* HomeLCD *) ;
;-----------------------------------------------------------------------------
SetCursonOn :					;PROCEDURE SetCursonOn ( ) ;
;************					;BEGIN (* SetCursonOn *)
	ldi	param,0b00001110		; _LCDOutC ( 0b00001110 ) ;
						;    set : display on
						;          cursor on
						;          cursor blink off
	rcall	_LCDOutC			;
	rcall	_WaitAccept			; _WaitAccept ( ) ;
	ret					;END (* SetCursonOn *) ;
;-----------------------------------------------------------------------------
SetCursonOff :					;PROCEDURE SetCursonOff ( ) ;
;*************					;BEGIN (* SetCursonOff *)
	ldi	param,0b00001100		; _LCDOutC ( 0b00001100 ) ;
						;    set : display on
						;          cursor off
						;          cursor blink off
	rcall	_LCDOutC			;
	rcall	_WaitAccept			; _WaitAccept ( ) ;
	ret					;END (* SetCursonOff *) ;
;-----------------------------------------------------------------------------
.ifdef	V1x16_as_2x8
SetCursorPos :					;PROCEDURE SetCursorPos ( Col[param] : BYTE ) ;
;*************					;VAR
;						; Control[param] : BYTE ;
;						;BEGIN (* SetCursorPos *)
	andi	param,0x3F			; Col := Col MOD 64 ;
	sts	_CharPos,param			; _CharPos := Col ;
	cpi	param,8				; IF Col < 8 THEN
	brcc	SetCPo_0			; BEGIN
	ldi	acc,0x80			;  Control := 0b10000000 + Col ;
	add	param,acc			;
	rjmp	SetCPo_1			; END (* IF ... *)
						; ELSE
SetCPo_0 :					; BEGIN
	ldi	acc,0xC0 - 8			;  Control := 0b11000000 + ( Col - 8 ) ;
	add	param,acc			;
SetCPo_1 :					; END * IF ... ELSE *) ;
	rcall	_LCDOutC			; _LCDOutC ( Control ) ;
	rcall	_WaitAccept			; _WaitAccept ( ) ;
	ret					;END (* SetCursorPos *) ;
.else
	.ifdef	lcd_as1line
SetCursorPos :					;PROCEDURE SetCursorPos ( Col[param] : BYTE ) ;
;*************					;BEGIN (* SetCursorPos *)
	andi	param,0x3F			; Col := Col MOD 64 ;
	ldi	acc,0x80			; _LCDOutC ( 0b10000000 + Col ) ;
	add	param,acc			;
	rcall	_LCDOutC			;
	rcall	_WaitAccept			; _WaitAccept ( ) ;
	ret					;END (* SetCursorPos *) ;
	.else
SetCursorPos :					;PROCEDURE SetCursorPos ( Col[param] : BYTE ;
;*************					;			  Row[acc] : BYTE ) ;
;						;VAR
;						; Control[acc] : BYTE ;
;						;BEGIN (* SetCursorPos *)
	andi	param,0x3F			; Col := Col MOD 64 ;
	cpi	acc,0				; IF Row = 0 THEN
	brne	SetCPo_0			; BEGIN
	ldi	acc,0x80			;  Control := 0x80 ;
	rjmp	SetCPo_1			; END (* IF ... *)
						; ELSE
SetCPo_0 :					; BEGIN
	ldi	acc,0xC0			;  Control := 0xC0 ;
SetCPo_1 :					; END (* IF ... ESE *) ;
	add	param,acc			; _LCDOutC ( Control + Col ) ;
	rcall	_LCDOutC			;
	rcall	_WaitAccept			; _WaitAccept ( ) ;
	ret					;END (* SetCursorPos *) ;
	.endif
.endif
;-----------------------------------------------------------------------------
NewLineLCD :					;PROCEDURE NewLineLCD ( ) ;
;***********					;BEGIN
	ldi	param,0b11000000		; _LCDOutC ( 0b11000000 ) ;
	rcall	_LCDOutC			;
	rcall	_WaitAccept			; _WaitAccept ( ) ;
.ifdef	V1x16_as_2x8
	clr	acc				; _CharPos := 0 ;
	sts	_CharPos,acc			;
.endif
	ret					;END (* NewLineLCD *) ;
;-----------------------------------------------------------------------------
WriteChLCD :					;PROCEDURE WriteChLCD ( Char [param] : BYTE ) ;
;***********					;BEGIN (* WriteChLCD *)
	rcall	_LCDOutD			; _LCDOutD ( Char ) ;
	rcall	_WaitAccept			; _WaitAccept ( ) ;
.ifdef	V1x16_as_2x8
	lds	acc,_CharPos			; _CharPos := _CharPos + 1 ;
	inc	acc				;
	sts	_CharPos,acc			;
	cpi	acc,0x08			; IF _CharPos = 8 THEN
	brne	WrCLCD_0			;  BEGIN
	ldi	param,0b11000000		;   _LCDOutC ( 0b11000000 ) ;
	rcall	_LCDOutC			;
	rcall	_WaitAccept			;   _WaitAccept ( ) ;
WrCLCD_0 :					;  END (* IF *) ;
.endif
	ret                             	;END (* WriteChLCD *) ;
;-----------------------------------------------------------------------------
WriteTextLCDFlash :				;PROCEDURE WriteTextLCDFlash ( Text [z] : POINTER TO CHAR ) ;
;******************				; VAR
;						;  Ch : CHAR ;
;						;BEGIN (* WriteTextLCDFlash *)
	push	param				;
WrTxLCDFl_0 :					; LOOP
	lpm					;  Ch := Text ^ ;
	mov	param,r0			;
	adiw	zl,1				;  Text := Text + 1 ;
	cpi	param,0				;  IF Ch = 0 THEN
	brne	WrTxLCDFl_1			;   BEGIN
	pop	param				;
	ret					;    RETURN ;
WrTxLCDFl_1 :					;   END (* IF *) ;
	rcall	WriteChLCD			;  WriteChLCD ( Ch ) ;
	rjmp	WrTxLCDFl_0			; END (* LOOP *) ;
						;END (* WriteTextLCDFlash *) ;
;-----------------------------------------------------------------------------
WriteTextLCD :					;PROCEDURE WriteTextLCD ( Text [z] : POINTER TO CHAR ) ;
;****************				; VAR
;						;  Ch : CHAR ;
;						;BEGIN (* WriteTextLCD *)
	push	param				;
WrTxLCDRAM_0 :					; LOOP
	ld	param,z+			;  Ch := Text ^ ;
						;  Text := Text + 1 ;
	cpi	param,0				;  IF Ch = 0 THEN
	brne	WrTxLCDRAM_1			;   BEGIN
	pop	param				;
	ret					;    RETURN ;
WrTxLCDRAM_1 :					;   END (* IF *) ;
	rcall	WriteChLCD			;  WriteChLCD ( Ch ) ;
	rjmp	WrTxLCDRAM_0			; END (* LOOP *) ;
						;END (* WriteTextLCD *) ;
;-----------------------------------------------------------------------------
;SetFont	:					;PROCEDURE SetFont ( Char [param] : BYTE ;
;;****************				;		     CharTab [z] : POINTER TO CHAR ) ;
;						;VAR
;						; Inx : BYTE ;
;						; CurCh : CHAR ;
;						;BEGIN (* SetFont *)
;	andi	param,0x07			; Char := Char MOD 8 ;
;	lsl	param				; Char := 0x40 + 8 * Char ;
;	lsl	param				;
;	lsl	param				;
;	ori	param,0b01000000		;
;	rcall	_LCDOutC			; _LCDOutC ( Char ) ; 
;	rcall	_WaitAccept			; _WaitAccept ( ) ;
;	ldi	acc2,8				; FOR Inx := 8 DOWNDO 1 DO
;SetFon_0 :					;  BEGIN
;	lpm					;   CurCh := CharTab ^ ;
;	mov	param,r0			;
;	adiw	zl,1				;   CharTab := CharTab + 1 ;
;	push	acc2				;
;	rcall	_LCDOutD			;   _LCDOutD ( CurCh ) ;
;	rcall	_LCDDelay			;   _LCDDelay ( ) ;
;	pop	acc2				;
;	dec	acc2				;
;	brne	SetFon_0			;  END (* FOR *) ;
;	ret					;END (* SetFont *) ;
;-----------------------------------------------------------------------------
.ifdef lcd_Interf4b
	.ifdef	V1x16_as_2x8
LCDCtrlData :	.db	0b00101000 , 0b00001100
			;set : dl = 4 bits
			;      li = 2 line
			;      5 X 7 dot
				     ;set : display on
				     ;      cursor off
				     ;      cursor blink off
		.db	0b00000110 , 0b00000000
			;set : autoincrement
			;      non shift mode
	.else
		.ifdef	lcd_as1line
LCDCtrlData :	.db	0b00100000 , 0b00001100
			;set : dl = 4 bits
			;      li = 1 line
			;      5 X 7 dot
				     ;set : display on
				     ;      cursor off
				     ;      cursor blink off
		.db	0b00000110 , 0b00000000
			;set : autoincrement
			;      non shift mode
		.else
LCDCtrlData :	.db	0b00101000 , 0b00001100
			;set : dl = 4 bits
			;      li = 2 line
			;      5 X 7 dot
				     ;set : display on
				     ;      cursor off
				     ;      cursor blink off
		.db	0b00000110 , 0b00000000
			;set : autoincrement
			;      non shift mode
		.endif
	.endif
.else
	.ifdef	V1x16_as_2x8
LCDCtrlData :	.db	0b00111000 , 0b00001100
			;set : dl = 8 bits
			;      li = 2 line
			;      5 X 7 dot
				     ;set : display on
				     ;      cursor off
				     ;      cursor blink off
		.db	0b00000110 , 0b00000000
			;set : autoincrement
			;      non shift mode
	.else
		.ifdef	lcd_as1line
LCDCtrlData :	.db	0b00110000 , 0b00001100
			;set : dl = 8 bits
			;      li = 1 line
			;      5 X 7 dot
				     ;set : display on
				     ;      cursor off
				     ;      cursor blink off
		.db	0b00000110 , 0b00000000
			;set : autoincrement
			;      non shift mode
		.else
LCDCtrlData :	.db	0b00111000 , 0b00001100
			;set : dl = 8 bits
			;      li = 2 line
			;      5 X 7 dot
				     ;set : display on
				     ;      cursor off
				     ;      cursor blink off
		.db	0b00000110 , 0b00000000
			;set : autoincrement
			;      non shift mode
		.endif
	.endif
.endif		
.exit
