;
;*******************************************************
;*******************************************************
;**                                                   **
;** Program prezentacji operacji na acuchach znakw **
;**                                                   **
;**         Andrzej Pawluczuk                         **
;**                                                   **
;**         Mikrokontroler: ATMEGA8515                **
;**                                                   **
;**             Bialystok, 2006                       **
;**                                                   **
;*******************************************************
;*******************************************************
;
	.nolist
;*******************************************************
.include "m8515def.inc"
;*******************************************************
	.list
	.listmac
;-----------------------------------------------------------------------------
.macro	ldz
	ldi	zl,low(@0)
	ldi	zh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	ldx
	ldi	xl,low(@0)
	ldi	xh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	pushz
	push	zl
	push	zh
.endm
;-----------------------------------------------------------------------------
.macro	popz
	pop	zh
	pop	zl
.endm
;-----------------------------------------------------------------------------
.macro	pushf
	push	acc
	in	acc,sreg
	push	acc
.endm
;-----------------------------------------------------------------------------
.macro	popf
	pop	acc
	out	sreg,acc
	pop	acc
.endm
;-----------------------------------------------------------------------------
.def DivLoopCounter	= r1
.def Argument1LL	= r2
.def Argument1LH	= r3
.def Argument1HL	= r4
.def Argument1HH	= r5
.def Argument2LL	= r6
.def Argument2LH	= r7
.def Argument2HL	= r8
.def Argument2HH	= r9
.def WorkRegLL		= r10
.def WorkRegLH		= r11
.def WorkRegHL		= r12
.def WorkRegHH		= r13
.def acc		= r16
.def acc2		= r17
.def param		= r18
.def ResultLL		= r22
.def ResultLH		= r23
.def ResultHL		= r24
.def ResultHH		= r25
;-----------------------------------------------------------------------------
.equ		Stand_TIMSK	=	0b00000010
;	Timer/counter Interrupt Mask	  ||| | |
;					  ||| | +--- TOIE0 : Timer/Counter 0 Overflow Interrupt Enable
;					  ||| +----- TICIE1 : Timer/Counter1 Input Capture Interrupt Enable
;					  ||+------- OCIE1B : Timer/counter1 Output Compare B Match Interrupt Enable
;					  |+-------- OCE1A : Timer/counter1 Output Compare A Match Interrupt Enable
;					  +--------- TOIE1 : Timer/Counter1 Overflow Interrupt Enable
;-----------------------------------------------------------------------------
.equ		Stand_TCCR0	=	0b00000011
;	Timer/Counter0 Control Register	       |||
;					       ||+-- CS00
;					       |+--- CS01
;					       +---- CS02
;		CS02	CS01	CS00
;		0	0	0	timer/counter0 is stopped
;		0	0	1	clk
;		0	1	0	clk/8
;		0	1	1	clk/64
;		1	0	0	clk/256
;		1	0	1	clk/1024
;		1	1	0	external pin T0; falling edge
;		1	1	1	external pin T0; rising edge
;-----------------------------------------------------------------------------
.equ		Stand_PADir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output : 
;					  ||||||+--- output : 
;					  |||||+---- output : 
;					  ||||+----- output : 
;					  |||+------ output : 
;					  ||+------- output : 
;					  |+-------- output : 
;					  +--------- output : 
.equ		Stand_PAData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PBDir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output : 
;					  ||||||+--- output : 
;					  |||||+---- output : 
;					  ||||+----- output : 
;					  |||+------ output : 
;					  ||+------- output : 
;					  |+-------- output : 
;					  +--------- output : 
.equ		Stand_PBData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PCDir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output : 
;					  ||||||+--- output : 
;					  |||||+---- output : 
;					  ||||+----- output : 
;					  |||+------ output : 
;					  ||+------- output : 
;					  |+-------- output : 
;					  +--------- output : 
.equ		Stand_PCData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PDDir	=	0b11111110
;					  ||||||||
;					  |||||||+-- input :	[RXD]
;					  ||||||+--- output :	[TXD]
;					  |||||+---- output : 
;					  ||||+----- output : 
;					  |||+------ output : 
;					  ||+------- output : 
;					  |+-------- output : 
;					  +--------- output : 
.equ		Stand_PDData	=	0b11111111
;-----------------------------------------------------------------------------
	.dseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  DATA  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
;						;CONST
.equ	Cr		= 0x0D			; Cr = 0x0D ;
.equ	Lf		= 0x0A			; Lf = 0x0A ;
.equ	Bs		= 0x08			; Bs = 08H ;
.equ	TextLineSize	= 64			; TextLineSize = 64 ;
;						;
;						;VAR
TextLine:	.byte	TextLineSize+1		;  TextLine : ARRAY [ 0 .. TextLineSize ] OF CHAR ;
CopyTextLine:	.byte	TextLineSize+1		;  CopyTextLine : ARRAY [ 0 .. TextLineSize ] OF CHAR ;
TextLineInx:	.byte	1			;  TextLineInx : BYTE ;
TextSize:	.byte	2			;  TextSize : WORD ;
Number:		.byte	8			;  Number : ARRAY [ 0 .. 7 ] OF CHAR ;
NumbInx:	.byte	1			;  NumbInx : BYTE ;
;-----------------------------------------------------------------------------
	.cseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  CODE  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
	.org	0				;
	rjmp	ResetProcessor			;
;-----------------------------------------------------------------------------
	.org	INT0addr			;External Interrupt0 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT1addr			;External Interrupt1 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ICP1addr			;Input Capture1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Aaddr			;Output Compare1A Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Baddr			;Output Compare1B Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF1addr			;Overflow1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF0addr			;Overflow0 Interrupt Vector Address
	rjmp	TC0OvfInterrupt			;
;-----------------------------------------------------------------------------
	.org	SPIaddr				;SPI Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	URXCaddr			;UART Receive Complete Interrupt Vector Address
	rjmp	USARTRXDInterrupt		;
;-----------------------------------------------------------------------------
	.org	UDREaddr			;UART Data Register Empty Interrupt Vector Address
	rjmp	USARTDataRegEmptInterrupt	;
;-----------------------------------------------------------------------------
	.org	UTXCaddr			;UART Transmit Complete Interrupt Vector Address
	rjmp	USARTTXDInterrupt		;
;-----------------------------------------------------------------------------
	.org	ACIaddr				;Analog Comparator Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT2addr			;External Interrupt2 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC0addr				;Output Compare0 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ERDYaddr			;EEPROM Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	SPMaddr				;SPM complete Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
.include "util.inc"
;-----------------------------------------------------------------------------
.include "strlib.inc"
;-----------------------------------------------------------------------------
.include "arithmetic.inc"
;-----------------------------------------------------------------------------
.include "strings.inc"
;-----------------------------------------------------------------------------
.equ	USART_Speed	= 47	; USART BAUD Rate Generator fosc=7.3728 MHz , 9600 bps
.equ	USART_LED_Trace	= 1
.equ	TXDBuffSize	= 32
.equ	RXDBuffSize	= 16
.equ	RXDLED		= 7
.equ	TXDLED		= 6
.equ	USARTTracePort	= PORTD
.equ	LEDOnState	= 0
.include "usartlib.inc"
;-----------------------------------------------------------------------------
HardwareInit :					;PROCEDURE HardwareInit ;
;*************					;BEGIN
	ldi	acc,Stand_TIMSK			; [TIMSK] := Stand_TIMSK ;
	out	TIMSK,acc			;
	ldi	acc,Stand_TCCR0			; [TCCR0] := Stand_TCCR0 ;
	out	TCCR0,acc			;
	ret					;END (* HardwareInit *) ;
;-----------------------------------------------------------------------------
EnvirInit :					;PROCEDURE EnvirInit ;
;**********					;BEGIN
	ldi	acc,Stand_PADir			; [DDRA] := Stand_PADir ;
	out	DDRA,Acc			;
	ldi	acc,Stand_PAData		; [PORTA] := Stand_PAData ;
	out	PORTA,acc			;
	ldi	acc,Stand_PBDir			; [DDRB] := Stand_PBDir ;
	out	DDRB,Acc			;
	ldi	acc,Stand_PBData		; [PORTB] := Stand_PBData ;
	out	PORTB,acc			;
	ldi	acc,Stand_PCDir			; [DDRC] := Stand_PCDir ;
	out	DDRC,Acc			;
	ldi	acc,Stand_PCData		; [PORTC] := Stand_PCData ;
	out	PORTC,acc			;
	ldi	acc,Stand_PDDir			; [DDRD] := Stand_PDDir ;
	out	DDRD,Acc			;
	ldi	acc,Stand_PDData		; [PORTD] := Stand_PDData ;
	out	PORTD,acc			;
	ldz	USART_Speed			; USARTHardwareInit ( USART_Speed ) ;
	rcall	USARTHardwareInit		;
	ret					;END (* EnvirInit *) ;
;-----------------------------------------------------------------------------
SoftwareInit :					;PROCEDURE SoftwareInit ;
;*************					;BEGIN
	rcall	USARTSoftwareInit		; USARTSoftwareInit ( ) ;
	ret					;END (* SoftwareInit *) ;
;-----------------------------------------------------------------------------
TC0OvfInterrupt :				;PROCEDURE TC0OvfInterrupt ;
;****************				;BEGIN (* TC0OvfInterrupt *)
	pushf					;
	pushz					;
.if USART_LED_Trace == 1
	rcall	USARTTimeService		; USARTTimeService ( ) ;
.endif
	popz					;
	popf					;
	reti					;END (* TC0OvfInterrupt *) ;
;-----------------------------------------------------------------------------
RangeErrorMessage :				;PROCEDURE RangeErrorMessage ( ) ;
;******************				;BEGIN (* RangeErrorMessage *)
	ldz	RangeErrText<<1			; SendSerialFlashTxt ( RangeErrText ) ;
	rcall	SendSerialFlashTxt		;
	ret					;END (* RangeErrorMessage *) ;
;-----------------------------------------------------------------------------
FindByteInArr :					;PROCEDURE FindByteInArr ( Data [param] : BYTE ;
;****************			   	;		Array [z] : ARRAY OF BYTE ) : BYTE[acc] ;
;						;VAR
;						; Loop[acc2] : BYTE ;
;						; Ch[acc] : CHAR ;
;						;BEGIN (* FindByteInArr *)
	clr	acc2				; Loop := 0 ;
FiByIA_2 :					; REPEAT
	lpm					;  Ch := Array [ Loop ] ;
	mov	acc,r0				;
	cpi	acc,0xFF			;  IF Ch = 0x0FF THEN
	brne	FiByIA_0			;  BEGIN
	ldi	acc,0xFF			;   RETURN 0xFF ;
	ret					;
FiByIA_0 :					;  END (* IF *) ;
	cp	acc,param			;  IF Ch = Data THEN
	brne	FiByIA_1			;  BEGIN
	mov	acc,acc2			;   RETURN Loop ;
	ret					;
FiByIA_1 :					;  END (* IF *) ;
	adiw	zl,1				;
	inc	acc2				;
	rjmp	FiByIA_2			; UNTIL FALSE ;
						;END (* FindByteInArr *) ;
;-----------------------------------------------------------------------------
RunService :					;PROCEDURE RunService ( Inx [param] ; BYTE :
;****************				;			Serv [z] : ADDRESS ) ;
;						;BEGIN (* RunService *)
	lsl	param				; Serv [ Inx ] ( ) ;
	clr	acc				;
	add	zl,param			;
	adc	zh,acc				;
	lpm					;
	push	r0				;
	adiw	zl,1				;
	lpm					;
	mov	zh,r0				;
	pop	zl				;
	icall					;
	ret					;END (* RunService *) ;
;-----------------------------------------------------------------------------
InputKey :					;FUNCTION InputKey ( ) : CHAR ;
;***************				;BEGIN (* InputKey *)
InpKy_1	:					; REPEAT
	rcall	SerialDataPresent		; UNTIL SerialDataPresent ( ) ;
	brcc	InpKy_1				;
	rcall	GetSerial			; InputKey := GetSerial ( ) ;
	ret					;END (* InputKey *) ;
;-----------------------------------------------------------------------------
SendNewLine :					;PROCEDURE SendNewLine ( ) ;
;************					;BEGIN (* SendNewLine *)
	ldi	param,Cr			; SendSerial ( Cr ) ;
	rcall	SendSerial			;
	ldi	param,Lf			; SendSerial ( Lf ) ;
	rcall	SendSerial			;
	ret					;END (* SendNewLine *) ;
;-----------------------------------------------------------------------------
NextChar :					;FUNCTION NextChar ( ) : CHAR [acc] ;
;*********					;VAR
;						; Ch[acc] : CHAR ;
;						;BEGIN (* NextChar *)
	rcall	InputKey			; Ch := InputKey ( ) ;
	push	acc				;
	cpi	acc,' '				; IF Ch >= ' ' THEN
	brcs	NeCh_1				; BEGIN
	mov	param,acc			;  SendSerial ( Ch ) ;
	rcall	SendSerial			;
NeCh_1	:					; END (* IF *) ;
	pop	acc				; NextChar := Ch ;
	ret					;END (* NextChar *) ;
;-----------------------------------------------------------------------------
NextDigitChar :					;FUNCTION NextDigitChar ( ) : CHAR [acc] ;
;*********					;VAR
;						; Ch[acc] : CHAR ;
;						;BEGIN (* NextDigitChar *)
NxDigCh_1 :					; LOOP
	rcall	InputKey			;  Ch := InputKey ( ) ;
	cpi	acc,Cr				;  IF Ch = Cr THEN
	brne	NxDigCh_3			;  BEGIN
	ret					;   NextDigitChar := Ch ;
						;   RETURN ;
NxDigCh_3 :					;  END (* IF *) ;
	cpi	acc,Bs				;  IF Ch = Bs THEN
	brne	NxDigCh_4			;  BEGIN
	ret					;   NextDigitChar := Ch ;
						;   RETURN ;
NxDigCh_4 :					;  END (* IF *) ;
	cpi	acc,'0'				;  IF ( Ch >= '0' ) AND ( Ch <= '9' ) THEN
	brcs	NxDigCh_0			;
	cpi	acc,'9'+1			;
	brcc	NxDigCh_0			;  BEGIN
	rjmp	NxDigCh_2			;   EXIT ;
NxDigCh_0 :					;  END (* IF *) ;
	rjmp	NxDigCh_1			; END (* LOOP *) ;
NxDigCh_2 :					;
	push	acc				;
	mov	param,acc			; SendSerial ( Ch ) ;
	rcall	SendSerial			;
	pop	acc				; NextDigitChar := Ch ;
	ret					;END (* NextDigitChar *) ;
;-----------------------------------------------------------------------------
RefreashText :					;PROCEDURE RefreashText ( ) ;
;*************					;VAR
;						; Loop [stack] : BYTE ;
;						;BEGIN (* RefreashText *)
	ldi	param,Cr			; SendSerial ( CR ) ;
	rcall	SendSerial			;
	ldi	acc,0				; FOR Loop := 0 TO TextLineSize DO
TefTx_1	:					; BEGIN
	push	acc				;
	ldi	param,' '			;  SendSerial ( ' ' ) ;
	rcall	SendSerial			;
	pop	acc				;
	cpi	acc,TextLineSize		;
	breq	RefTx_0				;
	inc	acc				;
	rjmp	TefTx_1				;
RefTx_0	:					; END (* FOR *) ;
	ldi	param,Cr			; SendSerial ( CR ) ;
	rcall	SendSerial			;
	ldz	TextLine			; SendSerialRAMTxt ( TextLine ) ;
	rcall	SendSerialRAMTxt		;
	ret					;END (* RefreashText ) ;
;-----------------------------------------------------------------------------
DispResult :					;PROCEDURE DispResult ( ) ;
;***********					;BEGIN (* DispResult *)
	ldz	ResultPrompt<<1			; SendSerialFlashTxt ( ResultPrompt ) ;
	rcall	SendSerialFlashTxt		;
	ldi	param,'"'			; SendSerial ( '"' ) ;
	rcall	SendSerial			;
	ldz	TextLine			; SendSerialRAMTxt ( TextLine ) ;
	rcall	SendSerialRAMTxt		;
	ldi	param,'"'			; SendSerial ( '"' ) ;
	rcall	SendSerial			;
	rcall	SendNewLine			; SendNewLine ( ) ;
	rcall	SendNewLine			; SendNewLine ( ) ;
	ret					;END (* DispResult *) ;
;-----------------------------------------------------------------------------
AddChToTextLine	:				;PROCEDURE AddChToTextLine ( Ch [param] : CHAR ) ;
;****************				;BEGIN (* AddChToTextLine *)
	lds	acc,TextLineInx			; IF TextLineInx < TextLineSize THEN
	cpi	acc,TextLineSize		;
	brcc	AChTTxL_0			; BEGIN
	ldz	TextLine			;  TextLine [ TextLineInx ] := Ch ;
	add	zl,acc				;
	ldi	acc,0				;
	adc	zh,acc				;
	st	z+,param			;
	st	z,acc				;  TextLine [ TextLineInx + 1 ] := 0 ;
	ldz	TextLineInx			;  TextLineInx := TextLineInx + 1 ;
	ld	acc,z				;
	inc	acc				;
	st	z,acc				;
AChTTxL_0 :					; END (* IF *) ;
	ret					;END (* AddChToTextLine *) ;
;-----------------------------------------------------------------------------
ReadNumber :					;FUNCTION ReadNumber ( ) : WORD[x] ;
;***********					;VAR
;						; Ch [acc] : CHAR ;
;						; ChPtr[z] : POINTER TO CHAR ;
;						;BEGIN (* ReadNumber *)
	ldz	InpNumberPrompt<<1		; SendSerialFlashTxt ( InpNumberPrompt ) ;
	rcall	SendSerialFlashTxt		;
	ldi	acc,0				; NumbInx := 0 ;
	sts	NumbInx,acc			;
	sts	Number,acc			; Number [ 0 ] := 0 ;
RdNumb_0 :					; LOOP
	rcall	NextDigitChar			;  Ch := NextDigitChar ( ) ;
	cpi	acc,Cr				;  IF Ch = Cr THEN
	brne	RdNumb_1			;  BEGIN
	rjmp	RdNumb_4			;   EXIT ;
RdNumb_1 :					;  END (* IF *) ;
	cpi	acc,Bs				;  IF Ch = Bs THEN
	brne	RdNumb_2			;  BEGIN
	ldz	NumbInx				;   IF NumbInx <> 0 THEN
	ld	acc2,z				;
	cpi	acc2,0				;
	breq	RdNumb_5			;   BEGIN
	dec	acc2				;    NumbInx := NumbInx - 1 ;
	st	z,acc2				;
	ldz	Number				;    Number [ NumbInx ] := 0 ;
	add	zl,acc2				;
	ldi	acc2,0				;
	adc	zh,acc2				;
	st	z,acc2				;
	ldi	param,Cr			;    SendSerial ( Cr ) ;
	rcall	SendSerial			;
	ldz	Number				;    SendSerialRAMTxt ( Number ) ;
	rcall	SendSerialRAMTxt		;
	ldi	param,' '			;    SendSerial ( ' ' ) ;
	rcall	SendSerial			;
	ldi	param,' '			;    SendSerial ( ' ' ) ;
	rcall	SendSerial			;
RdNumb_5 :					;   END (* IF *) ;
RdNumb_2 :					;  END (* IF *) ;
	lds	acc2,NumbInx			;  IF NumbInx < 7 THEN
	cpi	acc2,7				;
	brcc	RdNumb_3			;  BEGIN
	ldz	Number				;   Number [ NumbInx ] := Ch ;
	add	zl,acc2				;
	ldi	acc2,0				;
	add	zh,acc2				;
	st	z+,acc				;
	st	z,acc2				;   Nmber [ NumbInx + 1 ] := 0 ;
	ldz	NumbInx				;   NumbInx := NumbInx + 1 ;
	ld	acc,z				;
	inc	acc				;
	st	z,acc				;
RdNumb_3 :					;  END (* IF *) ;
	rjmp	RdNumb_0			; END (* LOOP *) ;
RdNumb_4 :					;
	ldi	acc,0				; Argument1 := 0 ;
	mov	Argument1LL,acc			;
	mov	Argument1LH,acc			;
	mov	Argument1HL,acc			;
	mov	Argument1HH,acc			;
	ldz	Number				; ChPtr := ADR ( Number [ 0 ] ) ;
RdNumb_6 :					; LOOP
	ld	acc,z+				;  Ch := ChPtr ^ ;
						;  ChPtr := ChPtr + 1 ;
	cpi	acc,0				;  IF Ch = 0 THEN
	brne	RdNumb_7			;  BEGIN
	mov	xl,Argument1LL			;   ReadNumber := Argument1 ;
	mov	xh,Argument1LH			;
	ret					;   RETURN ;
RdNumb_7 :					;  END (* IF *) ;
	rcall	MultAgr1And10			;  Argument1 := Argument1 * 10 ;
	subi	acc,'0'				;  Argument1 := Argument1 + ( Ch - '0' ) ;
	add	Argument1LL,acc			;
	ldi	acc,0				;
	adc	Argument1LH,acc			;
	adc	Argument1HL,acc			;
	adc	Argument1HH,acc			;
	rjmp	RdNumb_6			; END (* LOOP *) ;
						;END (* ReadNumber *) ;
;-----------------------------------------------------------------------------
GetTextLine :					;PROCEDURE GetTextLine ( ) ;
;************					;VAR
;						; Ch [acc] : CHAR ;
;						;BEGIN (* GetTextLine *)
	ldz	InpDataPrompt<<1		; SendSerialFlashTxt ( InpDataPrompt ) ;
	rcall	SendSerialFlashTxt		;
	ldi	acc,0				; TextLineInx := 0 ;
	sts	TextLineInx,acc			;
	sts	TextLine,acc			; TextLine [ 0 ] := 0 ;
GeTL_0	:					; LOOP
	rcall	NextChar			;  Ch := NextChar ( ) ;
	cpi	acc,Cr				;  IF Ch = Cr THEN
	brne	GeTL_1				;  BEGIN
	rcall	SendNewLine			;   SendNewLine ( ) ;
	ldz	InpTextPrompt<<1		;   SendSerialFlashTxt ( InpTextPrompt ) ;
	rcall	SendSerialFlashTxt		;
	ldi	param,'"'			;   SendSerial ( '"' ) ;
	rcall	SendSerial			;
	ldz	TextLine			;   SendSerialRAMTxt ( TextLine ) ;
	rcall	SendSerialRAMTxt		;
	ldi	param,'"'			;   SendSerial ( '"' ) ;
	rcall	SendSerial			;
	rcall	SendNewLine			;   SendNewLine ( ) ;
	rcall	SendNewLine			;   SendNewLine ( ) ;
	ret					;   RETURN ;
GeTL_1	:					;  END (* IF *) ;
	cpi	acc,Bs				;  IF Ch = Bs THEN
	brne	GeTL_2				;  BEGIN
	ldz	TextLineInx			;   IF TextLineInx <> 0 THEN
	ld	acc,z				;
	cpi	acc,0				;
	breq	GeTL_4				;   BEGIN
	dec	acc				;    TextLineInx := TextLineInx - 1 ;
	st	z,acc				;
	ldz	TextLine			;    TextLine [ TextLineInx ] := 0 ;
	add	zl,acc				;
	ldi	acc,0				;
	adc	zh,acc				;
	st	z,acc				;
	rcall	RefreashText			;    RefreashText ( ) ;
GeTL_4	:					;   END (* IF *) ;
	rjmp	GeTL_3				;  END (* IF ... *)
GeTL_2	:					;  ELSE
						;  BEGIN
	mov	param,acc			;   AddChToTextLine ( Ch ) ;
	rcall	AddChToTextLine			;
GeTL_3	:					;  END (* IF ... ELSE *) ;
	rjmp	GeTL_0				;
						;END GetTextLine ;
;-----------------------------------------------------------------------------
StringLengthPresentation :			;PROCEDURE StringLengthPresentation ( ) ;
;*************************			;BEGIN (* StringLengthPresentation *)
	rcall	GetTextLine			; GetTextLine ( ) ;
	ldz	TextLine			; TextSize := StrLen ( TextLine ) ;
	rcall	StrLen				;
	ldz	TextSize			;
	st	z+,xl				;
	st	z,xh				;
	ldz	StrLgtPrompt<<1			; SendSerialFlashTxt ( StrLgtPrompt ) ;
	rcall	SendSerialFlashTxt		;
	ldz	TextSize			; WordToString ( ADR ( TextSize ) , Number , 5 , ' ' ) ;
	ldx	Number				;
	ldi	param,5				;
	ldi	acc,' '				;
	rcall	WordToString			;
	ldz	Number				; SendSerialRAMTxt ( BegOfString ( Number ) ) ;
	rcall	BegOfString			;
	rcall	SendSerialRAMTxt		;
	rcall	SendNewLine			; SendNewLine ( ) ;
	ret					;END (* StringLengthPresentation *) ;
;-----------------------------------------------------------------------------
DeleteCharsPresetation :			;PROCEDURE DeleteCharsPresetation ( ) ;
;***********************			;VAR
;						; CharPos[x] : BYTE ;
;						;BEGIN (* DeleteCharsPresetation *)
	rcall	GetTextLine			; GetTextLine ( ) ;
	ldz	TextLine			; TextSize := StrLen ( TextLine ) ;
	rcall	StrLen				;
	ldz	TextSize			;
	st	z+,xl				;
	st	z,xh				;
	ldi	acc,4				; IF TextSize < 4 THEN
	cp	xl,acc				;
	ldi	acc,0				;
	cpc	xh,acc				;
	brcc	DelChars_1			; BEGIN
	rcall	RangeErrorMessage		;  RangeErrorMessage ( ) ;
	ret					;  REURN ;
DelChars_1 :					; END (* IF *) ;
	rcall	ReadNumber			; CharPos := ReadNumber ( ) ;
	ldz	TextSize			; IF ( TextSize - 3 ) < CharPos THEN
	ld	acc,z+				;
	ld	acc2,z				;
	subi	acc,3				;
	sbci	acc2,0				;
	cp	acc,xl				;
	cpc	acc2,xh				;
	brcc	DelChars_0			; BEGIN
	rcall	RangeErrorMessage		;  RangeErrorMessage ( ) ;
	ret					;  REURN ;
DelChars_0 :					; END (* IF *) ;
	ldz	TextLine			; StrDelete ( TextLine , CharPos , 3 ) ;
	mov	param,xl			;
	ldi	acc,3				;
	rcall	StrDelete			;
	ldz	StrDelFunctionTxt<<1		; SendSerialFlashTxt ( StrDelFunctionTxt ) ;
	rcall	SendSerialFlashTxt		;
	rcall	DispResult			; DispResult ( ) ;
	ret					;END (* DeleteCharsPresetation *) ;
;-----------------------------------------------------------------------------
AppendStringsPresentation :			;PROCEDURE AppendStringsPresentation ( ) ;
;**************************			;BEGIN (* AppendStringsPresentation *)
	rcall	GetTextLine			; GetTextLine ( ) ;
	ldx	CopyTextLine			; StrCpy ( CopyTextLine , TextLine ) ;
	ldz	TextLine			;
	rcall	StrCpy				;
	rcall	GetTextLine			; GetTextLine ( ) ;
	ldz	TextLine			; TextSize := StrLen ( TextLine ) + StrLen ( CopyTextLine )
	rcall	StrLen				;
	push	xh				;
	push	xl				;
	ldz	CopyTextLine			;
	rcall	StrLen				;
	pop	acc				;
	add	xl,acc				;
	pop	acc				;
	adc	xh,acc				;
	ldz	TextSize			;
	st	z+,xl				;
	st	z,xh				;
	ldi	acc,TextLineSize		; IF TextSize >= TextLineSize THEN
	cp	xl,acc				;
	ldi	acc,0				;
	cpc	xh,acc				;
	brcs	AppStr_0			; BEGIN
	rcall	RangeErrorMessage		;  RangeErrorMessage ( ) ;
	ret					;  REURN ;
AppStr_0 :					; END (* IF *) ;
	ldx	TextLine			; AppendString ( CopyTextLine , TextLine ) ;
	ldz	CopyTextLine			;
	rcall	AppendString			;
	ldz	ResultPrompt<<1			; SendSerialFlashTxt ( ResultPrompt ) ;
	rcall	SendSerialFlashTxt		;
	ldi	param,'"'			; SendSerial ( '"' ) ;
	rcall	SendSerial			;
	ldz	CopyTextLine			; SendSerialRAMTxt ( CopyTextLine ) ;
	rcall	SendSerialRAMTxt		;
	ldi	param,'"'			; SendSerial ( '"' ) ;
	rcall	SendSerial			;
	rcall	SendNewLine			; SendNewLine ( ) ;
	rcall	SendNewLine			; SendNewLine ( ) ;
	ret					;END (* AppendStringsPresentation *) ;
;-----------------------------------------------------------------------------
AppendFlashStrPresentation :			;PROCEDURE AppendFlashStrPresentation ( ) ;
;***************************			;BEGIN (* AppendFlashStrPresentation *)
	rcall	GetTextLine			; GetTextLine ( ) ;
	ldz	TextLine			; IF ( StrLen ( TextLine ) + 10 ) >= TextLineSize THEN
	rcall	StrLen				;
	ldi	acc,10				;
	add	xl,acc				;
	ldi	acc,0				;
	add	xh,acc				;
	ldi	acc,TextLineSize		;
	cp	xl,acc				;
	ldi	acc,0				;
	cpc	xh,acc				;
	brcs	AppFlStr_0			; BEGIN
	rcall	RangeErrorMessage		;  RangeErrorMessage ( ) ;
	ret					;  REURN ;
AppFlStr_0 :					; END (* IF *) ;
	ldz	TextLine			; AppendStringFlash ( TextLine , FlashText ) ;
	ldx	FlashText<<1			;
	rcall	AppendStringFlash		;
	rcall	DispResult			; DispResult ( ) ;
	ret					;END (* AppendFlashStrPresentation *) ;
;-----------------------------------------------------------------------------
AppendCharPresentation :			;PROCEDURE AppendCharPresentation ( ) ;
;***********************			;BEGIN (* AppendCharPresentation *)
	rcall	GetTextLine			; GetTextLine ( ) ;
	ldz	TextLine			; IF ( StrLen ( TextLine ) + 1 ) >= TextLineSize THEN
	rcall	StrLen				;
	ldi	acc,1				;
	add	xl,acc				;
	ldi	acc,0				;
	add	xh,acc				;
	ldi	acc,TextLineSize		;
	cp	xl,acc				;
	ldi	acc,0				;
	cpc	xh,acc				;
	brcs	AppCh_0				; BEGIN
	rcall	RangeErrorMessage		;  RangeErrorMessage ( ) ;
	ret					;  REURN ;
AppCh_0 :					; END (* IF *) ;
	ldz	TextLine			; AppendChar ( TextLine , '*' ) ;
	ldi	param,'*'			;
	rcall	AppendChar			;
	rcall	DispResult			; DispResult ( ) ;
	ret					;END (* AppendCharPresentation *) ;
;-----------------------------------------------------------------------------
InsertCharPresentation :			;PROCEDURE InsertCharPresentation ( ) ;
;***********************			;VAR
;						; CharPos[x] : BYTE ;
;						;BEGIN (* InsertCharPresentation *)
	rcall	GetTextLine			; GetTextLine ( ) ;
	ldz	TextLine			; TextSize := StrLen ( TextLine ) ;
	rcall	StrLen				;
	ldz	TextSize			;
	st	z+,xl				;
	st	z,xh				;
	rcall	ReadNumber			; CharPos := ReadNumber ( ) ;
	ldz	TextSize			; IF TextSize <= CharPos THEN
	ld	acc,z+				;
	ld	acc2,z				;
	cp	acc,xl				;
	cpc	acc2,xh				;
	brcc	InsChars_0			; BEGIN
	rcall	RangeErrorMessage		;  RangeErrorMessage ( ) ;
	ret					;  REURN ;
InsChars_0 :					; END (* IF *) ;
	ldz	TextLine			; StrInsertCh ( TextLine , '*' , CharPos ) ;
	ldi	param,'*'			;
	mov	acc,xl				;
	rcall	StrInsertCh			;
	rcall	DispResult			; DispResult ( ) ;
	ret					;END (* InsertCharPresentation *) ;
;-----------------------------------------------------------------------------
SubStrCopyPresentation :			;PROCEDURE SubStrCopyPresentation ( ) ;
;***********************			;VAR
;						; CharPos[x] : BYTE ;
;						;BEGIN (* SubStrCopyPresentation *)
	rcall	GetTextLine			; GetTextLine ( ) ;
	ldx	CopyTextLine			; StrCpy ( CopyTextLine , TextLine ) ;
	ldz	TextLine			;
	rcall	StrCpy				;
	ldz	TextLine			; TextSize := StrLen ( TextLine ) ;
	rcall	StrLen				;
	ldz	TextSize			;
	st	z+,xl				;
	st	z,xh				;
	rcall	ReadNumber			; CharPos := ReadNumber ( ) ;
	ldz	TextSize			; IF TextSize <= CharPos THEN
	ld	acc,z+				;
	ld	acc2,z				;
	cp	acc,xl				;
	cpc	acc2,xh				;
	brcc	StExt_0				; BEGIN
	rcall	RangeErrorMessage		;  RangeErrorMessage ( ) ;
	ret					;  REURN ;
StExt_0 :					; END (* IF *) ;
	mov	acc,xl				; StrExtract ( TextLine , CopyTextLine , CharPos , 5 ) ;
	ldx	TextLine			;
	ldz	CopyTextLine			;
	ldi	param,5				;
	rcall	StrExtract			;
	rcall	DispResult			; DispResult ( ) ;
	ret					;END (* SubStrCopyPresentation *) ;
;-----------------------------------------------------------------------------
NormStrPresentation :				;PROCEDURE NormStrPresentation ( ) ;
;********************				;BEGIN (* NormStrPresentation *)
	rcall	GetTextLine			; GetTextLine ( ) ;
	ldz	TextLine			; StrNorm ( TextLine ) ;
	rcall	StrNorm				;
	rcall	DispResult			; DispResult ( ) ;
	ret					;END (* NormStrPresentation *) ;
;-----------------------------------------------------------------------------
CapcharPresentation :				;PROCEDURE CapcharPresentation ( ) ;
;********************				;VAR
;						; ChPtr[z] : POINTER TO CHAR ;
;						;BEGIN (* CapcharPresentation *)
	rcall	GetTextLine			; GetTextLine ( ) ;
	ldz	TextLine			; ChPtr := ADR ( TextLine [ 0 ] ) ;
Cap_1	:					; WHILE ChPtr ^ <> 0 DO
	ld	acc,z				;
	cpi	acc,0				;
	breq	Cap_0				; BEGIN
	rcall	CapChar				;  ChPtr ^ := CapChar ( ChPtr ^ ) ;
	st	z+,acc				;  ChPtr := ChPtr + 1 ;
	rjmp	Cap_1				;
Cap_0	:					; END (* WHILE *) ;
	rcall	DispResult			; DispResult ( ) ;
	ret					;END (* CapcharPresentation *) ;
;-----------------------------------------------------------------------------
ComparePresentation :				;PROCEDURE ComparePresentation ( ) ;
;********************				;VAR
;						; CmpRes[acc] : SHORTINT ;
;						;BEGIN (* ComparePresentation *)
	ldz	CompareInfoTxt<<1		; SendSerialFlashTxt ( CompareInfoTxt ) ;
	rcall	SendSerialFlashTxt		;
	rcall	GetTextLine			; GetTextLine ( ) ;
	ldx	CopyTextLine			; StrCpy ( CopyTextLine , TextLine ) ;
	ldz	TextLine			;
	rcall	StrCpy				;
	rcall	GetTextLine			; GetTextLine ( ) ;
	ldx	CopyTextLine			; CmpRes := StrComp ( CopyTextLine , TextLine ,
	ldz	TextLine			;                     TextLineSize , FALSE ) ;
	ldi	param,TextLineSize		;
	ldi	acc,0				;
	rcall	StrComp				;
	cpi	acc,0				; IF CmpRes = 0 THEN
	brne	CompPre_0			; BEGIN
	ldz	CompareEQTxt<<1			;  SendSerialFlashTxt ( CompareEQTxt ) ;
	rcall	SendSerialFlashTxt		;
	ret					;  RETURN ;
CompPre_0 :					; END (* IF *) ;
	rol	acc				; IF CmpRes < 0 THEN
	brcc	CompPre_1			; BEGIN
	ldz	CompareLTTxt<<1			;  SendSerialFlashTxt ( CompareLTTxt ) ;
	rcall	SendSerialFlashTxt		;
	ret					;  RETURN ;
CompPre_1 :					; END (* IF *) ;
	ldz	CompareGTTxt<<1			; SendSerialFlashTxt ( CompareGTTxt ) ;
	rcall	SendSerialFlashTxt		;
	ret					;END (* ComparePresentation *) ;
;-----------------------------------------------------------------------------
ResetProcessor :				;
	ldi	acc,HIGH(RAMEND)		;
	out	SPH,acc				;
	ldi	acc,LOW(RAMEND)			;
	out	SPL,acc				;
						;BEGIN (* Main *)
	rcall	ClearRAM			; ClearRAM ( ) ;
	rcall	ClearRegs			; ClearRegs ( ) ;
	rcall	HardwareInit			; HardwareInit ( ) ;
	rcall	EnvirInit			; EnvirInit ( ) ;
	rcall	SoftwareInit			; SoftwareInit ( ) ;
	sei					; EI ( ) ;
	ldz	HelloText<<1			; SendSerialFlashTxt ( HelloText ) ;
	rcall	SendSerialFlashTxt		;
Main_0	:					; LOOP
	ldz	MenuTextEntry<<1		;  SendSerialFlashTxt ( MenuTextEntry ) ;
	rcall	SendSerialFlashTxt		;
	rcall	SendNewLine			;  SendNewLine ( ) ;
	rcall	InputKey			;  Inx := FindByteInArr ( InputKey ( ) , MenuCodeArray ) ;
	mov	param,acc			;
	ldz	MenuCodeArray<<1		;
	rcall	FindByteInArr			;
	cpi	acc,0xFF			;  IF Inx <> 0xFF THEN
	breq	Main_1				;  BEGIN
	ldz	MenuServiceArray<<1		;   RunService ( Inx , MenuServiceArray ) ;
	mov	param,acc			;
	rcall	RunService			;
Main_1	:					;  END (* IF *) ;
	rjmp	Main_0				; END (* LOOP *) ;
						;END (* Main *).
;-----------------------------------------------------------------------------
HelloText:		.db	Cr,Lf,Cr,Lf,"Hello !",Cr,Lf,Cr,Lf,0
MenuTextEntry:		.db	Cr,Lf,"Wybierz opcje:",Cr,Lf
			.db	" A - dlugosc lancucha znakow",Cr,Lf
			.db	" B - usuwanie znakow ze srodka",Cr,Lf
			.db	" C - dodanie lancucha na koncu",Cr,Lf
			.db	" D - dodanie stalego lancucha na koncu",Cr,Lf
			.db	" E - dodanie znaku na koncu ",Cr,Lf
			.db	" F - wstawienie znaku w srodku",Cr,Lf
			.db	" G - kopiowanie fragmentu lancucha",Cr,Lf
			.db	" H - normalizacja lancucha znakow ",Cr,Lf
			.db	" I - normalizacja do duzych liter ",Cr,Lf
			.db	" J - porownanie lancuchow znakow",Cr,Lf,0,0
MenuCodeArray:		.db	'A' , 'a'	;
			.db	'B' , 'b'	;
			.db	'C' , 'c'	;
			.db	'D' , 'd'	;
			.db	'E' , 'e'	;
			.db	'F' , 'f'	;
			.db	'G' , 'g'	;
			.db	'H' , 'h'	;
			.db	'I' , 'i'	;
			.db	'J' , 'j'	;
			.db	0xFF , 0	;
MenuServiceArray:	.dw	StringLengthPresentation	; 'A'
			.dw	StringLengthPresentation	; 'a'
			.dw	DeleteCharsPresetation		; 'B'
			.dw	DeleteCharsPresetation		; 'b'
			.dw	AppendStringsPresentation	; 'C'
			.dw	AppendStringsPresentation	; 'c'
			.dw	AppendFlashStrPresentation	; 'D'
			.dw	AppendFlashStrPresentation	; 'd'
			.dw	AppendCharPresentation		; 'E'
			.dw	AppendCharPresentation		; 'e'
			.dw	InsertCharPresentation		; 'F'
			.dw	InsertCharPresentation		; 'f'
			.dw	SubStrCopyPresentation		; 'G'
			.dw	SubStrCopyPresentation		; 'G'
			.dw	NormStrPresentation		; 'H'
			.dw	NormStrPresentation		; 'h'
			.dw	CapcharPresentation		; 'I'
			.dw	CapcharPresentation		; 'i'
			.dw	ComparePresentation		; 'J'
			.dw	ComparePresentation		; 'j'
InpDataPrompt:		.db	Cr,Lf,"Wprowadz lancuch znakow (Enter-konczy):",Cr,Lf,0
InpTextPrompt:		.db	"Wprowadzono :",0
StrLgtPrompt:		.db	"Dlugosc lancucha=",0
InpNumberPrompt:	.db	Cr,Lf,"Wprowadz liczbe:",Cr,Lf,0,0
RangeErrText:		.db	Cr,Lf,"Blad zakresu.",Cr,Lf,0
ResultPrompt:		.db	Cr,Lf,"Po operacji :",0
FlashText:		.db	"<dodatek>",0
StrDelFunctionTxt:	.db	Cr,Lf,"Usuniecie 3 znakow od miejsca wskazanego.",Cr,Lf,0
CompareInfoTxt:		.db	Cr,Lf,"Porownanie dwoch lancuchow znakow "
			.db	"z dokladnoscia do lancucha krotszego.",Cr,Lf,Cr,Lf,0
CompareEQTxt:		.db	Cr,Lf,"Lancuchy sa rowne",Cr,Lf,0
CompareLTTxt:		.db	Cr,Lf,"Pierwszy jest alfabetycznie mlodszy",Cr,Lf,0
CompareGTTxt:		.db	Cr,Lf,"Pierwszy jest alfabetycznie starszy",Cr,Lf,0
;-----------------------------------------------------------------------------
.exit
