;
;*******************************************************
;*******************************************************
;**                                                   **
;**   Program prezentacji obslugi wielopoziomowego    **
;**                    menu                           **
;**                                                   **
;**         Andrzej Pawluczuk                         **
;**                                                   **
;**         Mikrokontroler: ATMEGA8515                **
;**                                                   **
;*******************************************************
;*******************************************************
;
	.nolist
;*******************************************************
.include "m8515def.inc"
;*******************************************************
	.list
	.listmac
;-----------------------------------------------------------------------------
.macro	ldz
	ldi	zl,low(@0)
	ldi	zh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	ldx
	ldi	xl,low(@0)
	ldi	xh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	pushz
	push	zl
	push	zh
.endm
;-----------------------------------------------------------------------------
.macro	popz
	pop	zh
	pop	zl
.endm
;-----------------------------------------------------------------------------
.macro	pushf
	push	acc
	in	acc,sreg
	push	acc
.endm
;-----------------------------------------------------------------------------
.macro	popf
	pop	acc
	out	sreg,acc
	pop	acc
.endm
;-----------------------------------------------------------------------------
.macro	pushvar
	ldi	zl,low(@0)			;
	ldi	zh,high(@0)			;
	ld	acc,z+				;
	ld	acc2,z				;
	push	acc2				;
	push	acc				;
.endm
;	funkcja		skladuje na stosie zawartosc dwubajtowej zmiennej z pamieci RAM
;	uzycie		pushvar	<variable>
;	przyklad	pushvar	CurrentMenu
;-----------------------------------------------------------------------------
.macro	popvar
	pop	acc				;
	pop	acc2				;
	ldi	zl,low(@0)			;
	ldi	zh,high(@0)			;
	st	z+,acc				;
	st	z,acc2				;
.endm
;	funkcja		odtwarza ze stosu zawartosc dwubajtowej zmiennej w pamieci RAM
;	uzycie		popvar	<variable>
;	przyklad	popvar	CurrentMenu
;-----------------------------------------------------------------------------
.macro	WriFTxt
	ldi	zl,low(@0<<1)			; SendSerialFlashTxt ( @0 << 1 ) ;
	ldi	zh,high(@0<<1)			;
	rcall	SendSerialFlashTxt		;
.endm
;	funkcja		wywolanie procedury 'SendSerialFlashTxt' z bezposrednim wskazaniem
;			na wysylany test do pamieci flash (ze wzgledu na specyfike instrukcji
;			'lpm' ladowany do rejestru 'z' wskaznik, jest pomnozony przez 2)
;	uzycie		WriFTxt	<adres obszaru .DB>
;	przyklad	WriFTxt	HelloText
;-----------------------------------------------------------------------------
.macro	SendCh
	ldi	param,@0			; SendSerial ( @0 ) ;
	rcall	SendSerial			;
.endm
;	funkcja		wywolanie procedury 'SendSerial' z bezposrednim operandem
;	uzycie		SendCh	<znak>
;	przyklad	SendCh	'?'
;-----------------------------------------------------------------------------
.macro	SetMenu
	ldi	zl,low(CurrentMenu)		; CurrentMenu := @0 << 1 ;
	ldi	zh,high(CurrentMenu)		;
	ldi	acc,low(@0<<1)			;
	st	z+,acc				;
	ldi	acc,high(@0<<1)			;
	st	z,acc				;
.endm
;	funkcja		instrukcja realizuje podstawienie 'CurrentMenu := @0 << 1 ;'
;	uzycie		SetMenu	<adres obszaru opisujacy opcje menu zdefiniowany przez .DW>
;	przyklad	SetMenu	GlobalMenu
;-----------------------------------------------------------------------------
.macro	shiftz
	lsl	zl				;
	rol	zh				;
.endm
;	funkcja		mnozy zawartozc rejestru zh:zl przez 2
;	uzycie		shiftz
;-----------------------------------------------------------------------------
.macro	GetFB
	mov	zl,yl				; [acc] := [yh:yl] . @0
	mov	zh,yh				;
	.if @0 != 0
	adiw	zl,@0				;
	.endif
	lpm					;
	mov	acc,r0				;
.endm
;	funkcja		laduje do rejestru acc zawartosc pola bajtowego z rekordu typu
;			MenuDescrRecT (patrz nizej) opisujacego jedna opcje menu
;			identyfikowanego przez parametr @0, podstawa adresowa rekordu
;			jest zawarta w parze rejestrow yh:yl, wskazywany rekord jest
;			w pamieci FLASH
;	uzycie		GetFB	<identyfikator pola>
;	przyklad	GetFB	MenuDesRT_Service
;-----------------------------------------------------------------------------
.macro	GetF
	mov	zl,yl				; [acc2:acc] := [yh:yl] . @0
	mov	zh,yh				;
	.if @0 != 0
	adiw	zl,@0				;
	.endif
	lpm					;
	mov	acc,r0				;
	adiw	zl,1				;
	lpm					;
	mov	acc2,r0				;
.endm
;	funkcja		laduje do pary rejestrow acc2:acc zawartosc pola dwubajtwego z
;			rekordu typu MenuDescrRecT (patrz nizej) opisujacego jedna opcje menu
;			identyfikowanego przez parametr @0, podstawa adresowa rekordu
;			jest zawarta w parze rejestrow yh:yl, wskazywany rekord jest
;			w pamieci FLASH
;	uzycie		GetF	<identyfikator pola>
;	przyklad	GetF	MenuDesRT_Service
;-----------------------------------------------------------------------------
.macro	CallSrv
	mov	zl,acc				;
	mov	zh,acc2				;
	icall					;
.endm
;	funkcja		realizuje wywolanie obslugi opcji 'MenuDescrRecT.Service:PROCEDURE()'
;			uzywana w parze z maroinstrukcja 'GetF MenuDesRT_Service'
;	uzycie		CallSrv
;-----------------------------------------------------------------------------
.macro	GetMenu
	ldi	zl,low(CurrentMenu)		; [yh:yl] := CurrentMenu ;
	ldi	zh,high(CurrentMenu)		;
	ld	yl,z+				;
	ld	yh,z				;
.endm
;	funkcja		instrukcja realizuje podstawienie '[yh:yl] := CurrentMenu'
;	uzycie		GetMenu
;-----------------------------------------------------------------------------
.def acc		= r16
.def acc2		= r17
.def param		= r18
;-----------------------------------------------------------------------------
.equ		Stand_TIMSK	=	0b00000010
;	Timer/counter Interrupt Mask	  ||| | |
;					  ||| | +--- TOIE0 : Timer/Counter 0 Overflow Interrupt Enable
;					  ||| +----- TICIE1 : Timer/Counter1 Input Capture Interrupt Enable
;					  ||+------- OCIE1B : Timer/counter1 Output Compare B Match Interrupt Enable
;					  |+-------- OCE1A : Timer/counter1 Output Compare A Match Interrupt Enable
;					  +--------- TOIE1 : Timer/Counter1 Overflow Interrupt Enable
;-----------------------------------------------------------------------------
.equ		Stand_TCCR0	=	0b00000011
;	Timer/Counter0 Control Register	       |||
;					       ||+-- CS00
;					       |+--- CS01
;					       +---- CS02
;		CS02	CS01	CS00
;		0	0	0	timer/counter0 is stopped
;		0	0	1	clk
;		0	1	0	clk/8
;		0	1	1	clk/64
;		1	0	0	clk/256
;		1	0	1	clk/1024
;		1	1	0	external pin T0; falling edge
;		1	1	1	external pin T0; rising edge
;-----------------------------------------------------------------------------
.equ		Stand_PADir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output : 
;					  ||||||+--- output : 
;					  |||||+---- output : 
;					  ||||+----- output : 
;					  |||+------ output : 
;					  ||+------- output : 
;					  |+-------- output : 
;					  +--------- output : 
.equ		Stand_PAData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PBDir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output : 
;					  ||||||+--- output : 
;					  |||||+---- output : 
;					  ||||+----- output : 
;					  |||+------ output : 
;					  ||+------- output : 
;					  |+-------- output : 
;					  +--------- output : 
.equ		Stand_PBData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PCDir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output : 
;					  ||||||+--- output : 
;					  |||||+---- output : 
;					  ||||+----- output : 
;					  |||+------ output : 
;					  ||+------- output : 
;					  |+-------- output : 
;					  +--------- output : 
.equ		Stand_PCData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PDDir	=	0b11111110
;					  ||||||||
;					  |||||||+-- input :	[RXD]
;					  ||||||+--- output :	[TXD]
;					  |||||+---- output : 
;					  ||||+----- output : 
;					  |||+------ output : 
;					  ||+------- output : 
;					  |+-------- output : 
;					  +--------- output : 
.equ		Stand_PDData	=	0b11111111
;-----------------------------------------------------------------------------
	.dseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  DATA  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
;					;CONST
.equ	Cr		= 0x0D		; Cr = 0x0D ;
.equ	Lf		= 0x0A		; Lf = 0x0A ;
.equ	Esc		= 0x1B		; Esc = 0x1B ;
;					;
;					;TYPE
;					; MenuDescrRecT = RECORD
.equ	MenuDesRT_KeyCode	= 0	;                   KeyCode  : WORD ;
.equ	MenuDesRT_TextAdr	= 2	;                   TextAdr  : POINTER TO ARRAY OF CHAR ;
.equ	MenuDesRT_Service	= 4	;                   Service  : PROCEDURE ( ) ;
.equ	MenuDesctRecTSize	= 6	;                 END (* RECORD *) ;
;
;					;VAR
CurrentMenu :		.byte	2	; CurrentMenu : POINTER TO MenuDescrRecT ;
Recur:			.byte	1	; Recur : BOOLEAN ;
RedisplayMenu:		.byte	1	; RedisplayMenu : BOOLEAN ;
;-----------------------------------------------------------------------------
	.cseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  CODE  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
	.org	0				;
	rjmp	ResetProcessor			;
;-----------------------------------------------------------------------------
	.org	INT0addr			;External Interrupt0 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT1addr			;External Interrupt1 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ICP1addr			;Input Capture1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Aaddr			;Output Compare1A Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Baddr			;Output Compare1B Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF1addr			;Overflow1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF0addr			;Overflow0 Interrupt Vector Address
	rjmp	TC0OvfInterrupt			;
;-----------------------------------------------------------------------------
	.org	SPIaddr				;SPI Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	URXCaddr			;USART Receive Complete Interrupt Vector Address
	rjmp	USARTRXDInterrupt		;
;-----------------------------------------------------------------------------
	.org	UDREaddr			;USART Data Register Empty Interrupt Vector Address
	rjmp	USARTDataRegEmptInterrupt	;
;-----------------------------------------------------------------------------
	.org	UTXCaddr			;USART Transmit Complete Interrupt Vector Address
	rjmp	USARTTXDInterrupt		;
;-----------------------------------------------------------------------------
	.org	ACIaddr				;Analog Comparator Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT2addr			;External Interrupt2 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC0addr				;Output Compare0 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ERDYaddr			;EEPROM Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	SPMaddr				;SPM complete Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
.include "util.inc"
;-----------------------------------------------------------------------------
.equ	USART_Speed	= 47	; USART BAUD Rate Generator fosc=7.3728 MHz , 9600 bps
.equ	USART_LED_Trace	= 1
.equ	TXDBuffSize	= 32
.equ	RXDBuffSize	= 16
.equ	RXDLED		= 7
.equ	TXDLED		= 6
.equ	USARTTracePort	= PORTD
.equ	LEDOnState	= 0
.include "usartlib.inc"
;-----------------------------------------------------------------------------
HardwareInit :					;PROCEDURE HardwareInit ;
;*************					;BEGIN
	ldi	acc,Stand_TIMSK			; [TIMSK] := Stand_TIMSK ;
	out	TIMSK,acc			;
	ldi	acc,Stand_TCCR0			; [TCCR0] := Stand_TCCR0 ;
	out	TCCR0,acc			;
	ret					;END (* HardwareInit *) ;
;-----------------------------------------------------------------------------
EnvirInit :					;PROCEDURE EnvirInit ;
;**********					;BEGIN
	ldi	acc,Stand_PADir			; [DDRA] := Stand_PADir ;
	out	DDRA,Acc			;
	ldi	acc,Stand_PAData		; [PORTA] := Stand_PAData ;
	out	PORTA,acc			;
	ldi	acc,Stand_PBDir			; [DDRB] := Stand_PBDir ;
	out	DDRB,Acc			;
	ldi	acc,Stand_PBData		; [PORTB] := Stand_PBData ;
	out	PORTB,acc			;
	ldi	acc,Stand_PCDir			; [DDRC] := Stand_PCDir ;
	out	DDRC,Acc			;
	ldi	acc,Stand_PCData		; [PORTC] := Stand_PCData ;
	out	PORTC,acc			;
	ldi	acc,Stand_PDDir			; [DDRD] := Stand_PDDir ;
	out	DDRD,Acc			;
	ldi	acc,Stand_PDData		; [PORTD] := Stand_PDData ;
	out	PORTD,acc			;
	ldz	USART_Speed			; USARTHardwareInit ( USART_Speed ) ;
	rcall	USARTHardwareInit		;
	ret					;END (* EnvirInit *) ;
;-----------------------------------------------------------------------------
SoftwareInit :					;PROCEDURE SoftwareInit ;
;*************					;BEGIN
	ldi	acc,0				; Recur := FALSE ;
	sts	Recur,acc			;
	ret					;END (* SoftwareInit *) ;
;-----------------------------------------------------------------------------
TC0OvfInterrupt :				;PROCEDURE TC0OvfInterrupt ;
;****************				;BEGIN
	pushf					;
	pushz					;
.if USART_LED_Trace == 1
	rcall	USARTTimeService		; USARTTimeService ( ) ;
.endif
	popz					;
	popf					;
	reti					;END (* TC0OvfInterrupt *) ;
;-----------------------------------------------------------------------------
InputKey :					;FUNCTION InputKey ( ) : CHAR ;
;***************				;BEGIN (* InputKey *)
InpKy_1	:					; REPEAT
	rcall	SerialDataPresent		; UNTIL SerialDataPresent ( ) ;
	brcc	InpKy_1				;
	rcall	GetSerial			; InputKey := GetSerial ( ) ;
	ret					;END InputKey ;
;-----------------------------------------------------------------------------
NewLine		:				;PROCEDURE NewLine ( ) ;
;****************				;BEGIN (* NewLine *)
	SendCh	Cr				; SendCh ( Cr ) ;
	SendCh	Lf				; SendCh ( Lf ) ;
	ret					;END NewLine ;
;-----------------------------------------------------------------------------
EscapeKey :					;PROCEDURE EscapeKey ( Key[param] : CHAR ) : BOOLEAN ;
;**********					;BEGIN (* EscapeKey *)
	cpi	param,Esc			; IF Key = Esc THEN
	brne	EsKe_0				; BEGIN
	sec					;  RETURN TRUE ;
	ret					;
EsKe_0	:					; END (* IF *) ;
	clc					; RETURN FALSE ;
	ret					;END EscapeKey ;
;-----------------------------------------------------------------------------
DisplayMenu	:				;PROCEDURE DisplayMenu ( ) ;
;****************				;VAR
;						; Menu[yh:yl] : POINTER TO MenuDescrRecT ;
;						; OptionCh[acc] : CHAR ;
;						;BEGIN (* DisplayMenu *)
	GetMenu					; Menu := CurrentMenu ;
	GetF	MenuDesRT_Service		; IF Menu ^ . Service <> NIL THEN
	mov	param,acc			;
	or	param,acc2			;
	breq	Dis_Me2				; BEGIN
	CallSrv					;  Menu ^ . Service ( ) ;
Dis_Me2	:					; END (* IF *) ;
	GetMenu					; Menu := CurrentMenu ;
	GetF	MenuDesRT_TextAdr		; IF Menu ^ . TextAdr <> NIL THEN
	mov	param,acc			;
	or	param,acc2			;
	breq	Dis_Me1				; BEGIN
	mov	zl,acc				;  SendSerialFlashTxt ( Menu ^ . TextAdr<<1 ) ;
	mov	zh,acc2				;
	shiftz					;
	rcall	SendSerialFlashTxt		;
	rcall	NewLine				;  NewLine ( ) ;
Dis_Me1	:					; END (* IF *) ;
Dis_Me0	:					; LOOP
	adiw	yl,MenuDesctRecTSize		;  Menu := Menu + 1 ; (* Menu ++ *) ;
	GetFB	MenuDesRT_KeyCode		;  OptionCh := Menu ^ . KeyCode ;
	or	acc,acc				;  IF OptionCh = 0 THEN
	brne	Dis_Me3				;  BEGIN
	rjmp	Dis_Me4				;   EXIT ;
Dis_Me3	:					;  END (* IF *) ;
	mov	param,acc			;  SendSerial ( OptionCh ) ;
	rcall	SendSerial			;
	SendCh	' '				;  SendCh ( ' ' ) ;
	SendCh	'-'				;  SendCh ( '-' ) ;
	SendCh	' '				;  SendCh ( ' ' ) ;
	GetF	MenuDesRT_TextAdr		;  SendSerialFlashTxt ( Menu ^ . TextAdr<<1 ) ;
	mov	zl,acc				;
	mov	zh,acc2				;
	shiftz					;
	rcall	SendSerialFlashTxt		;
	rcall	NewLine				;  NewLine ( ) ;
	rjmp	Dis_Me0				;
Dis_Me4	:					; END (* LOOP *) ;
	ret					;END (* DisplayMenu *) ;
;-----------------------------------------------------------------------------
RunMenuSerivce	:				;PROCEDURE RunMenuSerivce ( ) ;
;****************				;VAR
;						; Ch[param] : CHAR ;
;						; OptionCh[acc] : CHAR ;
;						; Menu[yh:yl] : POINTER TO MenuDescrRecT ;
;						;BEGIN (* RunMenuSerivce *)
	ldi	acc,1				; RedisplayMenu := TRUE ;
	sts	RedisplayMenu,acc		;
RuMeSe_0 :					; LOOP
	lds	acc,RedisplayMenu		;  IF RedisplayMenu THEN
	cpi	acc,0				;
	breq	RuMeSe_2			;  BEGIN
	rcall	DisplayMenu			;   DisplayMenu ( ) ;
	rcall	NewLine				;   NewLine ( ) ;
	ldi	acc,0				;   RedisplayMenu := FALSE ;
	sts	RedisplayMenu,acc		;
RuMeSe_2 :					;  END (* IF *) ;
	rcall	InputKey			;  Ch = InputKey ( ) ;
	mov	param,acc			;  IF EscapeKey ( Ch ) THEN
	rcall	EscapeKey			;
	brcc	RuMeSe_1			;  BEGIN
	ret					;   RETURN ;
RuMeSe_1 :					;  END (* IF *) ;
	GetMenu					;  Menu := CurrentMenu ;
RuMeSe_6 :					;  LOOP
	adiw	yl,MenuDesctRecTSize		;   Menu := Menu + 1 ;
	GetFB	MenuDesRT_KeyCode		;   OptionCh := Menu ^ . KeyCode ;
	or	acc,acc				;   IF OptionCh = 0 THEN
	brne	RuMeSe_4			;   BEGIN
	rjmp	RuMeSe_5			;    EXIT ;
RuMeSe_4 :					;   END (* IF *) ;
	cp	acc,param			;   IF Ch = OptionCh THEN
	brne	RuMeSe_7			;   BEGIN
	GetF	MenuDesRT_Service		;    Menu ^ . Service ( ) ;
	CallSrv					;
	ldi	acc,1				;    RedisplayMenu := TRUE ;
	sts	RedisplayMenu,acc		;
	rjmp	RuMeSe_5			;    EXIT ;
RuMeSe_7 :					;   END (* IF *) ;
	rjmp	RuMeSe_6			;
RuMeSe_5 :					;  END (* LOOP *) ;
	rjmp	RuMeSe_0			; END (* LOOP *) ;
	ret					;END RunMenuSerivce ;
;-----------------------------------------------------------------------------
GlobMenu0OptSrv :				;PROCEDURE GlobMenu0OptSrv ( ) ;
;****************				;BEGIN (* GlobMenu0OptSrv *)
	pushvar	CurrentMenu			; PushVar ( CurrentMenu ) ;
	SetMenu	Lvl1_0Menu			; SetMenu ( Lvl1_0Menu ) ;
	rcall	RunMenuSerivce			; RunMenuSerivce ( ) ;
	popvar	CurrentMenu			; PopVar ( CurrentMenu ) ;
	ret					;END (* GlobMenu0OptSrv *) ;
;-----------------------------------------------------------------------------
GlobMenu1OptSrv :				;PROCEDURE GlobMenu1OptSrv ( ) ;
;****************				;BEGIN (* GlobMenu1OptSrv *)
	WriFTxt	GlobalMenu1			; WriteFlashText ( GlobalMenu1 ) ;
	ret					;END (* GlobMenu1OptSrv *) ;
;-----------------------------------------------------------------------------
GlobMenu2OptSrv :				;PROCEDURE GlobMenu2OptSrv ( ) ;
;****************				;BEGIN (* GlobMenu2OptSrv *)
	WriFTxt	GlobalMenu2			; WriteFlashText ( GlobalMenu2 ) ;
	ret					;END (* GlobMenu2OptSrv *) ;
;-----------------------------------------------------------------------------
GlobMenu3OptSrv :				;PROCEDURE GlobMenu3OptSrv ( ) ;
;****************				;BEGIN (* GlobMenu3OptSrv *)
	pushvar	CurrentMenu			; PushVar ( CurrentMenu ) ;
	SetMenu	Lvl1_3Menu			; SetMenu ( Lvl1_3Menu ) ;
	rcall	RunMenuSerivce			; RunMenuSerivce ( ) ;
	popvar	CurrentMenu			; PopVar ( CurrentMenu ) ;
	ret					;END (* GlobMenu3OptSrv *) ;
;-----------------------------------------------------------------------------
GlobMenu4OptSrv :				;PROCEDURE GlobMenu4OptSrv ( ) ;
;****************				;BEGIN (* GlobMenu4OptSrv *)
	WriFTxt	GlobalMenu4			; WriteFlashText ( GlobalMenu4 ) ;
	ret					;END (* GlobMenu4OptSrv *) ;
;-----------------------------------------------------------------------------
GlobMenu5OptSrv :				;PROCEDURE GlobMenu5OptSrv ( ) ;
;****************				;BEGIN (* GlobMenu5OptSrv *)
	WriFTxt	GlobalMenu5			; WriteFlashText ( GlobalMenu5 ) ;
	ret					;END (* GlobMenu5OptSrv *) ;
;-----------------------------------------------------------------------------
Lvl1_0Menu0OptSrv :				;PROCEDURE Lvl1_0Menu0OptSrv ( ) ;
;******************				;BEGIN (* Lvl1_0Menu0OptSrv *)
	WriFTxt	Lvl1_0Menu0			; WriteFlashText ( Lvl1_0Menu0 ) ;
	ret					;END (* Lvl1_0Menu0OptSrv *) ;
;-----------------------------------------------------------------------------
Lvl1_0Menu1OptSrv :				;PROCEDURE Lvl1_0Menu1OptSrv ( ) ;
;******************				;BEGIN (* Lvl1_0Menu1OptSrv *)
	WriFTxt	Lvl1_0Menu1			; WriteFlashText ( Lvl1_0Menu1 ) ;
	ret					;END (* Lvl1_0Menu1OptSrv *) ;
;-----------------------------------------------------------------------------
Lvl1_0Menu2OptSrv :				;PROCEDURE Lvl1_0Menu2OptSrv ( ) ;
;******************				;BEGIN (* Lvl1_0Menu2OptSrv *)
	WriFTxt	Lvl1_0Menu2			; WriteFlashText ( Lvl1_0Menu2 ) ;
	ret					;END (* Lvl1_0Menu2OptSrv *) ;
;-----------------------------------------------------------------------------
Lvl1_0Menu3OptSrv :				;PROCEDURE Lvl1_0Menu3OptSrv ( ) ;
;******************				;BEGIN (* Lvl1_0Menu3OptSrv *)
	WriFTxt	Lvl1_0Menu3			; WriteFlashText ( Lvl1_0Menu3 ) ;
	ret					;END (* Lvl1_0Menu3OptSrv *) ;
;-----------------------------------------------------------------------------
Lvl1_0Menu4OptSrv :				;PROCEDURE Lvl1_0Menu4OptSrv ( ) ;
;******************				;BEGIN (* Lvl1_0Menu4OptSrv *)
	WriFTxt	Lvl1_0Menu4			; WriteFlashText ( Lvl1_0Menu4 ) ;
	ret					;END (* Lvl1_0Menu4OptSrv *) ;
;-----------------------------------------------------------------------------
Lvl1_0Menu5OptSrv :				;PROCEDURE Lvl1_0Menu5OptSrv ( ) ;
;******************				;BEGIN (* Lvl1_0Menu5OptSrv *)
	WriFTxt	Lvl1_0Menu5			; WriteFlashText ( Lvl1_0Menu5 ) ;
	ret					;END (* Lvl1_0Menu5OptSrv *) ;
;-----------------------------------------------------------------------------
Lvl1_0Menu6OptSrv :				;PROCEDURE Lvl1_0Menu6OptSrv ( ) ;
;******************				;BEGIN (* Lvl1_0Menu6OptSrv *)
	WriFTxt	Lvl1_0Menu6			; WriteFlashText ( Lvl1_0Menu6 ) ;
	ret					;END (* Lvl1_0Menu6OptSrv *) ;
;-----------------------------------------------------------------------------
Lvl1_0Menu7OptSrv :				;PROCEDURE Lvl1_0Menu7OptSrv ( ) ;
;******************				;BEGIN (* Lvl1_0Menu7OptSrv *)
	WriFTxt	Lvl1_0Menu7			; WriteFlashText ( Lvl1_0Menu7 ) ;
	ret					;END (* Lvl1_0Menu7OptSrv *) ;
;-----------------------------------------------------------------------------
Lvl1_0Menu8OptSrv :				;PROCEDURE Lvl1_0Menu8OptSrv ( ) ;
;******************				;BEGIN (* Lvl1_0Menu8OptSrv *)
	WriFTxt	Lvl1_0Menu8			; WriteFlashText ( Lvl1_0Menu8 ) ;
	ret					;END (* Lvl1_0Menu8OptSrv *) ;
;-----------------------------------------------------------------------------
Lvl1_0Menu9OptSrv :				;PROCEDURE Lvl1_0Menu9OptSrv ( ) ;
;******************				;BEGIN (* Lvl1_0Menu9OptSrv *)
	WriFTxt	Lvl1_0Menu9			; WriteFlashText ( Lvl1_0Menu9 ) ;
	ret					;END (* Lvl1_0Menu9OptSrv *) ;
;-----------------------------------------------------------------------------
Lvl1_3Menu0OptSrv :				;PROCEDURE Lvl1_3Menu0OptSrv ( ) ;
;******************				;BEGIN (* Lvl1_3Menu0OptSrv *)
	WriFTxt	Lvl1_3Menu0			; WriteFlashText ( Lvl1_3Menu0 ) ;
	ret					;END (* Lvl1_3Menu0OptSrv *) ;
;-----------------------------------------------------------------------------
Lvl1_3Menu1OptSrv :				;PROCEDURE Lvl1_3Menu1OptSrv ( ) ;
;******************				;BEGIN (* Lvl1_3Menu1OptSrv *)
	WriFTxt	Lvl1_3Menu1			; WriteFlashText ( Lvl1_3Menu1 ) ;
	ret					;END (* Lvl1_3Menu1OptSrv *) ;
;-----------------------------------------------------------------------------
Lvl1_3Menu2OptSrv :				;PROCEDURE Lvl1_3Menu2OptSrv ( ) ;
;******************				;BEGIN (* Lvl1_3Menu2OptSrv *)
	WriFTxt	Lvl1_3Menu2			; WriteFlashText ( Lvl1_3Menu2 ) ;
	ret					;END (* Lvl1_3Menu2OptSrv *) ;
;-----------------------------------------------------------------------------
Lvl1_3Menu3OptSrv :				;PROCEDURE Lvl1_3Menu3OptSrv ( ) ;
;******************				;BEGIN (* Lvl1_3Menu3OptSrv *)
	lds	acc,Recur			; IF Recur THEN
	cpi	acc,0				;
	breq	Lv13OS_0			; BEGIN
	WriFTxt	DisableCallMesgTxt		;  WriteFlashText ( DisableCallMesgTxt ) ;
	ret					;  RETURN ;
Lv13OS_0 :					; END (* IF *) ;
	ldi	acc,1				; Recur := TRUE ;
	sts	Recur,acc			;
	pushvar	CurrentMenu			; PushVar ( CurrentMenu ) ;
	rcall	SetupService			; SetupService ( ) ;
	popvar	CurrentMenu			; PopVar ( CurrentMenu ) ;
	ldi	acc,0				; Recur := FALSE ;
	sts	Recur,acc			;
	ret					;END (* Lvl1_3Menu3OptSrv *) ;
;-----------------------------------------------------------------------------
ExtraMenuService :				;PROCEDURE ExtraMenuService ( ) ;
;*****************				;BEGIN (* ExtraMenuService *)
	WriFTxt	ExtraText1			; WriteFlashText ( ExtraText1 ) ;
	lds	acc,Recur			; IF Recur THEN
	cpi	acc,0				;
	breq	ExMeSe_0			; BEGIN
	WriFTxt	ExtraText2			;  WriteFlashText ( ExtraText2 ) ;
ExMeSe_0 :					; END (* IF *) ;
	WriFTxt	ExtraText3			; WriteFlashText ( ExtraText3 ) ;
	ret					;END (* ExtraMenuService *) ;
;-----------------------------------------------------------------------------
SetupService :					;PROCEDURE SetupService ( ) ;
;*************					;BEGIN (* SetupService *)
	SetMenu	GlobalMenu			; SetMenu ( GlobalMenu ) ;
	rcall	RunMenuSerivce			; RunMenuSerivce ( ) ;
	ret					;END SetupService ;
;-----------------------------------------------------------------------------
ResetProcessor :				;
	ldi	acc,HIGH(RAMEND)		;
	out	SPH,acc				;
	ldi	acc,LOW(RAMEND)			;
	out	SPL,acc				;
						;BEGIN (* Main *)
	rcall	ClearRAM			; ClearRAM ( ) ;
	rcall	ClearRegs			; ClearRegs ( ) ;
	rcall	HardwareInit			; HardwareInit ( ) ;
	rcall	EnvirInit			; EnvirInit ( ) ;
	rcall	SoftwareInit			; SoftwareInit ( ) ;
	sei					; EI ( ) ;
	WriFTxt	HelloText			; WriteFlashText ( HelloText ) ;
Main_0	:					; LOOP
Main_1	:					;  REPEAT
	rcall	SerialDataPresent		;  UNTIL SerialDataPresent ( ) ;
	brcc	Main_1				;
	WriFTxt	StartInfoText			;  WriteFlashText ( StartInfoText ) ;
	rcall	GetSerial			;  ? := GetSerial ( ) ;
	rcall	SetupService			;  SetupService ( ) ;
	WriFTxt	StopInfoText			;  WriteFlashText ( StopInfoText ) ;
	WriFTxt	HelloText2			;  WriteFlashText ( HelloText2 ) ;
	rjmp	Main_0				; END (* LOOP *) ;
						;END (* Main *).
;-----------------------------------------------------------------------------
HelloText:		.db	Cr,Lf,Cr,Lf,"Witaj!",Cr,Lf
HelloText2:		.db	"Dowolny klawisz aktywuje menu",Cr,Lf,0
StartInfoText:		.db	"Startuje menu",Cr,Lf,Cr,Lf,0
StopInfoText:		.db	"Koniec menu",Cr,Lf,Cr,Lf,0
DisableCallMesgTxt:	.db	Cr,Lf,"Blokada rekurencyjnego wywolania.",Cr,Lf,0
ExtraText1:		.db	Cr,Lf
			.db	"**************************************",Cr,Lf
			.db	"*  Opcja <d> wywoluje rekurencyjnie  *",Cr,Lf
			.db	"*    cale menu (istnieje blokada     *",Cr,Lf
			.db	"*       pelnej rekurencyjnosci)      *",Cr,Lf,0,0
ExtraText2:		.db	"* To jest juz wywolanie rekurencyjne *",Cr,Lf,0,0
ExtraText3:		.db	"**************************************",Cr,Lf,0,0
;-----------------------------------------------------------------------------
GlobalMenu1:		.db	Cr,Lf,"Wykonanie opcji 1 menu glownego",Cr,Lf,Cr,Lf,0
GlobalMenu2:		.db	Cr,Lf,"Wykonanie opcji 2 menu glownego",Cr,Lf,Cr,Lf,0
GlobalMenu4:		.db	Cr,Lf,"Wykonanie opcji 4 menu glownego",Cr,Lf,Cr,Lf,0
GlobalMenu5:		.db	Cr,Lf,"Wykonanie opcji 5 menu glownego",Cr,Lf,Cr,Lf,0
GlobalMenuTitle:	.db	Cr,Lf,"Menu: poziom glowny",0
GlobMenu0OptTxt:	.db	">>> Opcja 0 <<< (---> podmenu)",0,0
GlobMenu1OptTxt:	.db	">>> Opcja 1 <<<",0
GlobMenu2OptTxt:	.db	">>> Opcja 2 <<<",0
GlobMenu3OptTxt:	.db	">>> Opcja 3 <<< (---> podmenu)",0,0
GlobMenu4OptTxt:	.db	">>> Opcja 4 <<<",0
GlobMenu5OptTxt:	.db	">>> Opcja 5 <<<",0
;
;GlobalMenu : ARRAY OF MenuDescrRecT ;
;
GlobalMenu:		.dw	0xFFFF , GlobalMenuTitle , 0
			.dw	'0'    , GlobMenu0OptTxt , GlobMenu0OptSrv
			.dw	'1'    , GlobMenu1OptTxt , GlobMenu1OptSrv
			.dw	'2'    , GlobMenu2OptTxt , GlobMenu2OptSrv
			.dw	'3'    , GlobMenu3OptTxt , GlobMenu3OptSrv
			.dw	'4'    , GlobMenu4OptTxt , GlobMenu4OptSrv
			.dw	'5'    , GlobMenu5OptTxt , GlobMenu5OptSrv
			.dw	0
;-----------------------------------------------------------------------------
Lvl1_0Menu0:		.db	Cr,Lf,"Wykonanie opcji 0",Cr,Lf,Cr,Lf,0
Lvl1_0Menu1:		.db	Cr,Lf,"Wykonanie opcji 1",Cr,Lf,Cr,Lf,0
Lvl1_0Menu2:		.db	Cr,Lf,"Wykonanie opcji 2",Cr,Lf,Cr,Lf,0
Lvl1_0Menu3:		.db	Cr,Lf,"Wykonanie opcji 3",Cr,Lf,Cr,Lf,0
Lvl1_0Menu4:		.db	Cr,Lf,"Wykonanie opcji 4",Cr,Lf,Cr,Lf,0
Lvl1_0Menu5:		.db	Cr,Lf,"Wykonanie opcji 5",Cr,Lf,Cr,Lf,0
Lvl1_0Menu6:		.db	Cr,Lf,"Wykonanie opcji 6",Cr,Lf,Cr,Lf,0
Lvl1_0Menu7:		.db	Cr,Lf,"Wykonanie opcji 7",Cr,Lf,Cr,Lf,0
Lvl1_0Menu8:		.db	Cr,Lf,"Wykonanie opcji 8",Cr,Lf,Cr,Lf,0
Lvl1_0Menu9:		.db	Cr,Lf,"Wykonanie opcji 9",Cr,Lf,Cr,Lf,0
Lvl1_0MenuTitle:	.db	Cr,Lf,"Podmenu opcji 0 menu glownego",0
Lvl1_0Menu0OptTxt:	.db	">>> Opcja 0 <<<",0
Lvl1_0Menu1OptTxt:	.db	">>> Opcja 1 <<<",0
Lvl1_0Menu2OptTxt:	.db	">>> Opcja 2 <<<",0
Lvl1_0Menu3OptTxt:	.db	">>> Opcja 3 <<<",0
Lvl1_0Menu4OptTxt:	.db	">>> Opcja 4 <<<",0
Lvl1_0Menu5OptTxt:	.db	">>> Opcja 5 <<<",0
Lvl1_0Menu6OptTxt:	.db	">>> Opcja 6 <<<",0
Lvl1_0Menu7OptTxt:	.db	">>> Opcja 7 <<<",0
Lvl1_0Menu8OptTxt:	.db	">>> Opcja 8 <<<",0
Lvl1_0Menu9OptTxt:	.db	">>> Opcja 9 <<<",0
;
;Lvl1_0Menu : ARRAY OF MenuDescrRecT ;
;
Lvl1_0Menu:		.dw	0xFFFF , Lvl1_0MenuTitle , 0
			.dw	'0'    , Lvl1_0Menu0OptTxt , Lvl1_0Menu0OptSrv
			.dw	'1'    , Lvl1_0Menu1OptTxt , Lvl1_0Menu1OptSrv
			.dw	'2'    , Lvl1_0Menu2OptTxt , Lvl1_0Menu2OptSrv
			.dw	'3'    , Lvl1_0Menu3OptTxt , Lvl1_0Menu3OptSrv
			.dw	'4'    , Lvl1_0Menu4OptTxt , Lvl1_0Menu4OptSrv
			.dw	'5'    , Lvl1_0Menu5OptTxt , Lvl1_0Menu5OptSrv
			.dw	'6'    , Lvl1_0Menu6OptTxt , Lvl1_0Menu6OptSrv
			.dw	'7'    , Lvl1_0Menu7OptTxt , Lvl1_0Menu7OptSrv
			.dw	'8'    , Lvl1_0Menu8OptTxt , Lvl1_0Menu8OptSrv
			.dw	'9'    , Lvl1_0Menu9OptTxt , Lvl1_0Menu9OptSrv
			.dw	0
;-----------------------------------------------------------------------------
Lvl1_3Menu0:		.db	Cr,Lf,"Wykonanie opcji a",Cr,Lf,Cr,Lf,0
Lvl1_3Menu1:		.db	Cr,Lf,"Wykonanie opcji b",Cr,Lf,Cr,Lf,0
Lvl1_3Menu2:		.db	Cr,Lf,"Wykonanie opcji c",Cr,Lf,Cr,Lf,0
Lvl1_3MenuTitle:	.db	Cr,Lf,"Podmenu opcji 3 menu glownego",0
Lvl1_3Menu0OptTxt:.db	">>> Opcja a <<<",0
Lvl1_3Menu1OptTxt:.db	">>> Opcja b <<<",0
Lvl1_3Menu2OptTxt:.db	">>> Opcja c <<<",0
Lvl1_3Menu3OptTxt:.db	">>> Opcja d <<< (--> podmenu)",0
;
;Lvl1_3Menu : ARRAY OF MenuDescrRecT ;
;
Lvl1_3Menu:		.dw	0xFFFF , Lvl1_3MenuTitle , ExtraMenuService
			.dw	'a'    , Lvl1_3Menu0OptTxt , Lvl1_3Menu0OptSrv
			.dw	'b'    , Lvl1_3Menu1OptTxt , Lvl1_3Menu1OptSrv
			.dw	'c'    , Lvl1_3Menu2OptTxt , Lvl1_3Menu2OptSrv
			.dw	'd'    , Lvl1_3Menu3OptTxt , Lvl1_3Menu3OptSrv
			.dw	0
;-----------------------------------------------------------------------------
.exit
