;
;*******************************************************
;*******************************************************
;**                                                   **
;**      Program prezentacji obsugi wyswietlacza     **
;**     LED 7-segmentowego, wspolna anoda (6 cyfr)    **
;**                                                   **
;**         Andrzej Pawluczuk                         **
;**                                                   **
;**         Mikrokontroler: ATMEGA8515                **
;**                                                   **
;**             Bialystok, 2006                       **
;**                                                   **
;*******************************************************
;*******************************************************
;
	.nolist
;*******************************************************
.include "m8515def.inc"
;*******************************************************
	.list
	.listmac
;-----------------------------------------------------------------------------
.macro	ldz
	ldi	zl,low(@0)
	ldi	zh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	ldx
	ldi	xl,low(@0)
	ldi	xh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	pushz
	push	zl
	push	zh
.endm
;-----------------------------------------------------------------------------
.macro	popz
	pop	zh
	pop	zl
.endm
;-----------------------------------------------------------------------------
.macro	pushf
	push	acc
	in	acc,sreg
	push	acc
.endm
;-----------------------------------------------------------------------------
.macro	popf
	pop	acc
	out	sreg,acc
	pop	acc
.endm
;-----------------------------------------------------------------------------
.def DivLoopCounter	= r1
.def Argument1LL	= r2
.def Argument1LH	= r3
.def Argument1HL	= r4
.def Argument1HH	= r5
.def Argument2LL	= r6
.def Argument2LH	= r7
.def Argument2HL	= r8
.def Argument2HH	= r9
.def WorkRegLL		= r10
.def WorkRegLH		= r11
.def WorkRegHL		= r12
.def WorkRegHH		= r13
.def acc		= r16
.def acc2		= r17
.def param		= r18
.def ResultLL		= r22
.def ResultLH		= r23
.def ResultHL		= r24
.def ResultHH		= r25
;-----------------------------------------------------------------------------
.equ	DivideConst	= 0x80
;-----------------------------------------------------------------------------
.equ		Stand_TIMSK	=	0b00000010
;	Timer/counter Interrupt Mask	  ||| | |
;					  ||| | +--- TOIE0 : Timer/Counter 0 Overflow Interrupt Enable
;					  ||| +----- TICIE1 : Timer/Counter1 Input Capture Interrupt Enable
;					  ||+------- OCIE1B : Timer/counter1 Output Compare B Match Interrupt Enable
;					  |+-------- OCE1A : Timer/counter1 Output Compare A Match Interrupt Enable
;					  +--------- TOIE1 : Timer/Counter1 Overflow Interrupt Enable
;-----------------------------------------------------------------------------
.equ		Stand_TCCR0	=	0b00000011
;	Timer/Counter0 Control Register	       |||
;					       ||+-- CS00
;					       |+--- CS01
;					       +---- CS02
;		CS02	CS01	CS00
;		0	0	0	timer/counter0 is stopped
;		0	0	1	clk
;		0	1	0	clk/8
;		0	1	1	clk/64
;		1	0	0	clk/256
;		1	0	1	clk/1024
;		1	1	0	external pin T0; falling edge
;		1	1	1	external pin T0; rising edge
;-----------------------------------------------------------------------------
.equ		Stand_PADir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output : LED - sterowanie cyfra
;					  ||||||+--- output : LED - sterowanie cyfra
;					  |||||+---- output : LED - sterowanie cyfra
;					  ||||+----- output : LED - sterowanie cyfra
;					  |||+------ output : LED - sterowanie cyfra
;					  ||+------- output : LED - sterowanie cyfra
;					  |+-------- output :
;					  +--------- output :
.equ		Stand_PAData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PBDir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output : LED - sterowanie segmentem
;					  ||||||+--- output : LED - sterowanie segmentem
;					  |||||+---- output : LED - sterowanie segmentem
;					  ||||+----- output : LED - sterowanie segmentem
;					  |||+------ output : LED - sterowanie segmentem
;					  ||+------- output : LED - sterowanie segmentem
;					  |+-------- output : LED - sterowanie segmentem
;					  +--------- output : LED - sterowanie segmentem
.equ		Stand_PBData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PCDir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output :
;					  ||||||+--- output :
;					  |||||+---- output :
;					  ||||+----- output :
;					  |||+------ output :
;					  ||+------- output :
;					  |+-------- output :
;					  +--------- output :
.equ		Stand_PCData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PDDir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output :
;					  ||||||+--- output :
;					  |||||+---- output :
;					  ||||+----- output :
;					  |||+------ output :
;					  ||+------- output :
;					  |+-------- output :
;					  +--------- output :
.equ		Stand_PDData	=	0b11111111
;-----------------------------------------------------------------------------
	.dseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  DATA  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
;						; VAR
Number:		.byte	8			;  Number : ARRAY [ 0 .. 7 ] OF CHAR ;
Counter:	.byte	4			;  Counter : LONGWORD ;
;-----------------------------------------------------------------------------
	.cseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  CODE  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
	.org	0				;
	rjmp	ResetProcessor			;
;-----------------------------------------------------------------------------
	.org	INT0addr			;External Interrupt0 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT1addr			;External Interrupt1 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ICP1addr			;Input Capture1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Aaddr			;Output Compare1A Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Baddr			;Output Compare1B Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF1addr			;Overflow1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF0addr			;Overflow0 Interrupt Vector Address
	rjmp	TC0OvfInterrupt			;
;-----------------------------------------------------------------------------
	.org	SPIaddr				;SPI Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	URXCaddr			;UART Receive Complete Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	UDREaddr			;UART Data Register Empty Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	UTXCaddr			;UART Transmit Complete Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ACIaddr				;Analog Comparator Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
.include "util.inc"
;-----------------------------------------------------------------------------
.equ	LED_SelDigit_Port	= PORTA
.equ	LED_SegData_Port	= PORTB
.include "7segmlib.inc"
;-----------------------------------------------------------------------------
.include "arithmetic.inc"
;-----------------------------------------------------------------------------
HardwareInit :					;PROCEDURE HardwareInit ;
;*************					;BEGIN (* HardwareInit *)
	ldi	acc,Stand_TIMSK			; [TIMSK] := Stand_TIMSK ;
	out	TIMSK,acc			;
	ldi	acc,Stand_TCCR0			; [TCCR0] := Stand_TCCR0 ;
	out	TCCR0,acc			;
	ret					;END (* HardwareInit *) ;
;-----------------------------------------------------------------------------
EnvirInit :					;PROCEDURE EnvirInit ;
;**********					;BEGIN (* EnvirInit *)
	ldi	acc,Stand_PADir			; [DDRA] := Stand_PADir ;
	out	DDRA,Acc			;
	ldi	acc,Stand_PAData		; [PORTA] := Stand_PAData ;
	out	PORTA,acc			;
	ldi	acc,Stand_PBDir			; [DDRB] := Stand_PBDir ;
	out	DDRB,Acc			;
	ldi	acc,Stand_PBData		; [PORTB] := Stand_PBData ;
	out	PORTB,acc			;
	ldi	acc,Stand_PCDir			; [DDRC] := Stand_PCDir ;
	out	DDRC,Acc			;
	ldi	acc,Stand_PCData		; [PORTC] := Stand_PCData ;
	out	PORTC,acc			;
	ldi	acc,Stand_PDDir			; [DDRD] := Stand_PDDir ;
	out	DDRD,Acc			;
	ldi	acc,Stand_PDData		; [PORTD] := Stand_PDData ;
	out	PORTD,acc			;
	rcall	Init7SegmEnvir			; Init7SegmEnvir ( ) ;
	ret					;END (* EnvirInit *) ;
;-----------------------------------------------------------------------------
SoftwareInit :					;PROCEDURE SoftwareInit ;
;*************					;BEGIN (* SoftwareInit *)
	ldi	acc,0				; Counter :=0 ;
	ldz	Counter				;
	st	z+,acc				;
	st	z+,acc				;
	st	z+,acc				;
	st	z,acc				;
	ret					;END (* SoftwareInit *) ;
;-----------------------------------------------------------------------------
TC0OvfInterrupt :				;PROCEDURE TC0OvfInterrupt ;
;****************				;BEGIN (* TC0OvfInterrupt *)
	pushf					;
	pushz					;
	rcall	LEDDisplTimeService		; LEDDisplTimeService ( ) ;
	popz					;
	popf					;
	reti					;END (* TC0OvfInterrupt *) ;
;-----------------------------------------------------------------------------
BinToStrDec	:				;PROCEDURE BinToStrDec ( Width[param] : BYTE ) ;
;****************				;VAR
;						; ChLoop [acc2] : BYTE ;
;						;BEGIN
	ldi	acc2,0				; FOR ChLoop := 0 TO Width-1 DO
	ldz	Number				;
	ldi	acc,'0'				;
BiToSD_0 :					; BEGIN
	st	z+,acc				;  Number [ ChLoop ] := '0' ;
	inc	acc2				;
	cp	acc2,param			; END (* FOR *) ;
	brne	BiToSD_0			;
	ldi	acc,0				; Number [ Width ] := 0 ;
	st	z,acc				;
	ldz	Counter				; Argument1 := Counter ;
	ld	acc,z+				;
	mov	Argument1LL,acc			;
	ld	acc,z+				;
	mov	Argument1LH,acc			;
	ld	acc,z+				;
	mov	Argument1HL,acc			;
	ld	acc,z+				;
	mov	Argument1HH,acc			;
	mov	acc2,param			; FOR ChLoop := Width - 1 DOWNTO 0 DO
	ldz	Number				;
	dec	acc2				;
	add	zl,param			;
	ldi	acc,0				;
	adc	zh,acc				;
BiToSD_2 :					; BEGIN
	push	acc2				;
	rcall	DivModArg1By10			;  Argument2 := Argument1 DIV 10 ;
						;  ModValue := Argument1 MOD 10 ;
	pop	acc2				;
	ldi	acc,'0'				;  Number [ ChLoop ] := WorkReg + '0' ;
	add	acc,WorkRegLL			;
	st	-z,acc				;
	mov	acc,Argument2LL			;  IF Argument2 = 0 THEN
	or	acc,Argument2LH			;
	or	acc,Argument2HL			;
	or	acc,Argument2HH			;
	brne	BiToSD_3			;  BEGIN
	ret					;   RETURN ;
BiToSD_3 :					;  END (* IF *) ;
	mov	Argument1LL,Argument2LL		;  Argument1 := Argument2 ;
	mov	Argument1LH,Argument2LH		;
	mov	Argument1HL,Argument2HL		;
	mov	Argument1HH,Argument2HH		;
	cpi	acc2,0				;
	breq	BiToSD_1			;
	dec	acc2				;
	rjmp	BiToSD_2			;
BiToSD_1 :					; END (* FOR *) ;
	ret					;END (* BinToStrDec *) ;
;-----------------------------------------------------------------------------
ResetProcessor :				;
	cli					;
	ldi	acc,HIGH(RAMEND)		;
	out	SPH,acc				;
	ldi	acc,LOW(RAMEND)			;
	out	SPL,acc				;
						;BEGIN (* Main *)
	rcall	ClearRAM			; ClearRAM ( ) ;
	rcall	ClearRegs			; ClearRegs ( ) ;
	rcall	HardwareInit			; HardwareInit ( ) ;
	rcall	EnvirInit			; EnvirInit ( ) ;
	rcall	SoftwareInit			; SoftwareInit ( ) ;
	sei					; EI ( ) ;
Main_0	:					; LOOP
	ldi	param,6				;  BinToStrDec ( 6 ) ;
	rcall	BinToStrDec			;
	rcall	LEDClearDisplay			;  LEDClearDisplay ( ) ;
	lds	param,Number			;  LEDDisplWriteCh ( Number [ 0 ] ) ;
	rcall	LEDDisplWriteCh			;
	lds	param,Number+1			;  LEDDisplWriteCh ( Number [ 1 ] ) ;
	rcall	LEDDisplWriteCh			;
	lds	param,Number+2			;  LEDDisplWriteCh ( Number [ 2 ] ) ;
	rcall	LEDDisplWriteCh			;
	lds	param,Number+3			;  LEDDisplWriteCh ( Number [ 3 ] ) ;
	rcall	LEDDisplWriteCh			;
	lds	param,Number+4			;  LEDDisplWriteCh ( Number [ 4 ] ) ;
	rcall	LEDDisplWriteCh			;
	lds	param,Number+5			;  LEDDisplWriteCh ( Number [ 5 ] ) ;
	rcall	LEDDisplWriteCh			;
	rcall	LongDelay			;  LongDelay ( ) ;
	ldz	Counter				;  Counter := Counter + 1 ;
	ld	acc,z				;
	ldi	acc2,1				;
	add	acc,acc2			;
	st	z+,acc				;
	ld	acc,z				;
	ldi	acc2,0				;
	adc	acc,acc2			;
	st	z+,acc				;
	ld	acc,z				;
	adc	acc,acc2			;
	st	z+,acc				;
	ld	acc,z				;
	adc	acc,acc2			;
	st	z+,acc				;
	rjmp	Main_0				; END (* LOOP *) ;
						;END (* Main *) ;
;-----------------------------------------------------------------------------
.exit
