;
;*******************************************************
;*******************************************************
;**                                                   **
;**      Program prezentacji uzycia zewnetrznej       **
;**                  pamieci RAM                      **
;**                                                   **
;**         Andrzej Pawluczuk                         **
;**                                                   **
;**         Mikrokontroler: ATMEGA8515                **
;**                                                   **
;**             Bialystok, 2006                       **
;**                                                   **
;*******************************************************
;*******************************************************
;
	.nolist
;*******************************************************
.include "m8515def.inc"
;*******************************************************
	.list
	.listmac
;-----------------------------------------------------------------------------
.macro	ldz
	ldi	zl,low(@0)
	ldi	zh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	ldy
	ldi	yl,low(@0)
	ldi	yh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	ldx
	ldi	xl,low(@0)
	ldi	xh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	pushz
	push	zl
	push	zh
.endm
;-----------------------------------------------------------------------------
.macro	popz
	pop	zh
	pop	zl
.endm
;-----------------------------------------------------------------------------
.macro	pushf
	push	acc
	in	acc,sreg
	push	acc
.endm
;-----------------------------------------------------------------------------
.macro	popf
	pop	acc
	out	sreg,acc
	pop	acc
.endm
;-----------------------------------------------------------------------------
.macro	arrelem
	ldi	zl,low(@1)
	ldi	zh,high(@1)
	ld	acc,z+
	ld	acc2,z
	lsl	acc
	rol	acc2
	ldi	zl,low(@0)
	ldi	zh,high(@0)
	add	zl,acc
	adc	zh,acc2
.endm
;	function	z := adr ( @0 [ @1 ] )
;			arrelem	<table>,<index>
;	example		arrelem	DataTable,DataTableInx
;-----------------------------------------------------------------------------
.macro	incword
	ldi	zl,low(@0)
	ldi	zh,high(@0)
	ld	acc,z
	ldi	acc2,1
	add	acc,acc2
	st	z+,acc
	ld	acc,z
	ldi	acc2,0
	adc	acc,acc2
	st	z,acc
.endm
;	function	@0 := @0 + 1 ;
;			incword	<variable>
;	example		incword	Loop
;-----------------------------------------------------------------------------
.macro	clrword
	ldi	zl,low(@0)
	ldi	zh,high(@0)
	ldi	acc,0
	st	z+,acc
	st	z,acc
.endm
;	function	@0 := 0 ;
;			clrword	<variable>
;	example		clrword	Loop
;-----------------------------------------------------------------------------
.macro	mvword
	ldi	zl,low(@1)
	ldi	zh,high(@1)
	ld	acc,z+
	ld	acc2,z
	ldi	zl,low(@0)
	ldi	zh,high(@0)
	st	z+,acc
	st	z,acc2
.endm
;	function	@0 := @1 ;
;			mvword	<variable>,<variable>
;	example		mvword	MinMaxInx,Loop
;-----------------------------------------------------------------------------
.macro	getarr
	ld	acc,z+
	ld	acc2,z
	ldi	zl,low(@0)
	ldi	zh,high(@0)
	st	z+,acc
	st	z,acc2
.endm
;	function	@0 := Z ^ ;
;			getarr	<variable>
;	example		getarr	MinMaxVal
;-----------------------------------------------------------------------------
.macro	putarr
	ldi	xl,low(@0)
	ldi	xh,high(@0)
	ld	acc,x+
	st	z+,acc
	ld	acc,x
	st	z,acc
.endm
;	function	Z ^ := @0 ;
;			putarr	<variable>
;	example		putarr	MinMaxVal
;-----------------------------------------------------------------------------
.macro	clfor
	ldi	zl,low(@0)
	ldi	zh,high(@0)
	ldi	xl,low(@1)
	ldi	xh,high(@1)
	ld	acc,z+
	ld	acc2,x+
	cp	acc,acc2
	ld	acc,z
	ld	acc2,x
	cpc	acc,acc2
	breq	@2
	rjmp	@3
.endm
;	function	zamkniecie petli for
;			clfor	<index>,<end index>,<exit label>,<cycle label>
;	example		clfor	Loop,DataTableInx,SeMin_2,SeMin_1
;-----------------------------------------------------------------------------
.macro	jmpxltz
	ld	acc,z+
	ld	acc2,x+
	cp	acc,acc2
	ld	acc,z
	ld	acc2,x
	cpc	acc,acc2
	brcs	@0
.endm
;	function	weryfikacja warunku: skok jezeli x^ < z^
;			jmpxltz	<label>
;	example		jmpxltz	SeMin_3
;-----------------------------------------------------------------------------
.macro	jmpxgtz
	ld	acc,x+
	ld	acc2,z+
	cp	acc,acc2
	ld	acc,x
	ld	acc2,z
	cpc	acc,acc2
	brcs	@0
.endm
;	function	weryfikacja warunku: skok jezeli x^ > z^
;			jmpxgtz	<label>
;	example		jmpxgtz	SeMax_3
;-----------------------------------------------------------------------------
.macro	jmpxeqz
	ld	acc,z+
	ld	acc2,x+
	cp	acc,acc2
	ld	acc,z
	ld	acc2,x
	cpc	acc,acc2
	breq	@0
.endm
;	function	weryfikacja warunku: skok jezeli x^ = z^
;			jmpxeqz	<label>
;	example		jmpxeqz	SeMax_3
;-----------------------------------------------------------------------------
.def DivLoopCounter	= r1
.def Argument1LL	= r2
.def Argument1LH	= r3
.def Argument1HL	= r4
.def Argument1HH	= r5
.def Argument2LL	= r6
.def Argument2LH	= r7
.def Argument2HL	= r8
.def Argument2HH	= r9
.def WorkRegLL		= r10
.def WorkRegLH		= r11
.def WorkRegHL		= r12
.def WorkRegHH		= r13
.def acc		= r16
.def acc2		= r17
.def param		= r18
.def ResultLL		= r22
.def ResultLH		= r23
.def ResultHL		= r24
.def ResultHH		= r25
;-----------------------------------------------------------------------------
.equ		Stand_TIMSK	=	0b00000010
;	Timer/counter Interrupt Mask	  ||| | |
;					  ||| | +--- TOIE0 : Timer/Counter 0 Overflow Interrupt Enable
;					  ||| +----- TICIE1 : Timer/Counter1 Input Capture Interrupt Enable
;					  ||+------- OCIE1B : Timer/counter1 Output Compare B Match Interrupt Enable
;					  |+-------- OCE1A : Timer/counter1 Output Compare A Match Interrupt Enable
;					  +--------- TOIE1 : Timer/Counter1 Overflow Interrupt Enable
;-----------------------------------------------------------------------------
.equ		Stand_MCUCR	=	0b10000000
;	MCU Control Register		  ||||||||
;					  |||||||+-- ISC00 : Interrupt 0 Sense Control bit 0
;					  ||||||+--- ISC01 : Interrupt 0 Sense Control bit 1
;					  |||||+---- ISC10 : Interrupt 1 Sense Control bit 0
;					  ||||+----- ISC11 : Interrupt 1 Sense Control bit 1
;					  |||+------ SM : Sleep Mode
;					  ||+------- SE : Sleep Enable
;					  |+-------- SRW : External SRAM Wait Stare
;					  +--------- SRE : External SRAM Enable
;			ISCx0	ISCx1
;			  0	  0	the low lewel of INTx generates an interrupt request
;			  0	  1	reserved
;			  1	  0	the falling edge of INTx generates an interrupt request
;			  1	  1	the rising edge of INTx generates an interrupt request
;-----------------------------------------------------------------------------
.equ		Stand_TCCR0	=	0b00000100
;	Timer/Counter0 Control Register	       |||
;					       ||+-- CS00
;					       |+--- CS01
;					       +---- CS02
;		CS02	CS01	CS00
;		0	0	0	timer/counter0 is stopped
;		0	0	1	clk
;		0	1	0	clk/8
;		0	1	1	clk/64
;		1	0	0	clk/256
;		1	0	1	clk/1024
;		1	1	0	external pin T0; falling edge
;		1	1	1	external pin T0; rising edge
;-----------------------------------------------------------------------------
.equ		Stand_TCCR1B	=		0b00000001
;	Timer/Counter1 Control Register B	  ||  ||||
;						  ||  |||+-- CS10
;						  ||  ||+--- CS11
;						  ||  |+---- CS12
;						  ||  +----- CTC1 : Clear Timer/Counter1 on Compare Match
;						  |+-------- ICES1 : Input Capture 1 Edge Select
;						  +--------- ICNC1 : Input Capture 1 Noise Canceler
;		CS12	CS11	CS10
;		0	0	0	timer/counter1 is stopped
;		0	0	1	clk
;		0	1	0	clk/8
;		0	1	1	clk/64
;		1	0	0	clk/256
;		1	0	1	clk/1024
;		1	1	0	external pin T0; falling edge
;		1	1	1	external pin T0; rising edge
;-----------------------------------------------------------------------------
.equ		Stand_PADir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output : memory interface
;					  ||||||+--- output : memory interface
;					  |||||+---- output : memory interface
;					  ||||+----- output : memory interface
;					  |||+------ output : memory interface
;					  ||+------- output : memory interface
;					  |+-------- output : memory interface
;					  +--------- output : memory interface
.equ		Stand_PAData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PBDir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output : LCD D0
;					  ||||||+--- output : LCD D1
;					  |||||+---- output : LCD D2
;					  ||||+----- output : LCD D3
;					  |||+------ output : LCD RS
;					  ||+------- output : LCD RW
;					  |+-------- output : LCD E
;					  +--------- output :
.equ		Stand_PBData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PCDir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output : memory interface
;					  ||||||+--- output : memory interface
;					  |||||+---- output : memory interface
;					  ||||+----- output : memory interface
;					  |||+------ output : memory interface
;					  ||+------- output : memory interface
;					  |+-------- output : memory interface
;					  +--------- output : memory interface
.equ		Stand_PCData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PDDir	=	0b11000000
;					  ||||||||
;					  |||||||+-- input :	[RXD]
;					  ||||||+--- output :	[TXD]
;					  |||||+---- input :
;					  ||||+----- input :
;					  |||+------ input :
;					  ||+------- input :
;					  |+-------- output : memory interface
;					  +--------- output : memory interface
.equ		Stand_PDData	=	0b11111111
;-----------------------------------------------------------------------------
.equ	ExtRAMOrgin	= 0x8000
.equ	ExtRamSize	= 0x8000
;-----------------------------------------------------------------------------
.equ	KeybRAMPortAdr	= 0x1002
.equ	Key0Pin		= 7
.equ	Key1Pin		= 6
.equ	Key2Pin		= 5
.equ	Key3Pin		= 4
.equ	Key4Pin		= 3
.equ	Key5Pin		= 2
.equ	Key6Pin		= 1
.equ	Key7Pin		= 0
;-----------------------------------------------------------------------------
.equ	Key0Code	= '0'
.equ	Key1Code	= '1'
.equ	Key2Code	= '2'
.equ	Key3Code	= '3'
.equ	Key4Code	= '4'
.equ	Key5Code	= '5'
.equ	Key6Code	= '6'
.equ	Key7Code	= '7'
;-----------------------------------------------------------------------------
	.dseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  DATA  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
;						; CONST
.equ	TableSize	=	1024		;  TableSize = 1024 ;
.equ	Cr		=	0x0D		;  Cr = 0x0D ;
.equ	Lf		=	0x0A		;  Lf = 0x0A ;
;						;
;						; VAR
	.org	ExtRAMOrgin			;
DataTable :		.byte	2*TableSize	;  DataTable : ARRAY [ 0 .. TableSize-1 ] OF WORD ;
DataTableInx :		.byte	2		;  DataTableInx : WORD ;
NewData :		.byte	2		;  NewData : WORD ;
Loop :			.byte	2		;  Loop : WORD ;
StartLoop :		.byte	2		;  StartLoop : WORD ;
MinMaxInx :		.byte	2		;  MinMaxInx : WORD ;
MinMaxVal :		.byte	2		;  MinMaxVal : WORD ;
Sum :			.byte	2		;  Sum : WORD ;
	.org	0x60				;
RamErrorAdr :		.byte	2		;  RamErrorAdr : ADDRESS ;
Number :		.byte	8		;  Number : ARRAY [ 0 .. 7 ] OF CHAR ;
;-----------------------------------------------------------------------------
	.cseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  CODE  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
	.org	0				;
	rjmp	ResetProcessor			;
;-----------------------------------------------------------------------------
	.org	INT0addr			;External Interrupt0 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT1addr			;External Interrupt1 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ICP1addr			;Input Capture1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Aaddr			;Output Compare1A Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Baddr			;Output Compare1B Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF1addr			;Overflow1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF0addr			;Overflow0 Interrupt Vector Address
	rjmp	TC0OvfInterrupt			;
;-----------------------------------------------------------------------------
	.org	SPIaddr				;SPI Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	URXCaddr			;UART Receive Complete Interrupt Vector Address
	rjmp	USARTRXDInterrupt		;
;-----------------------------------------------------------------------------
	.org	UDREaddr			;UART Data Register Empty Interrupt Vector Address
	rjmp	USARTDataRegEmptInterrupt	;
;-----------------------------------------------------------------------------
	.org	UTXCaddr			;UART Transmit Complete Interrupt Vector Address
	rjmp	USARTTXDInterrupt		;
;-----------------------------------------------------------------------------
	.org	ACIaddr				;Analog Comparator Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
.include "util.inc"
;-----------------------------------------------------------------------------
.equ	V1x16_as_2x8	= 1
.equ	lcd_Interf4b	= 1
.equ	lcd_E_port	= PORTB
.equ	lcd_E_pin	= 6
.equ	lcd_RW_port	= PORTB
.equ	lcd_RW_pin	= 5
.equ	lcd_RS_port	= PORTB
.equ	lcd_RS_pin	= 4
.equ	lcdD0		= 0
.equ	lcdD1		= 1
.equ	lcdD2		= 2
.equ	lcdD3		= 3
.equ	lcd_Data_port	= PORTB
.include "lcdlib.inc"
;-----------------------------------------------------------------------------
.include "keybsrv.inc"
;-----------------------------------------------------------------------------
.include "arithmetic.inc"
;-----------------------------------------------------------------------------
.equ	USART_Speed	= 47	; UART BAUD Rate Generator fosc=7.3728 MHz , 9600 bps
.equ	USART_LED_Trace	= 0
.equ	TXDBuffSize	= 128
.equ	RXDBuffSize	= 16
.include "usartlib.inc"
;-----------------------------------------------------------------------------
.include "strings.inc"
;-----------------------------------------------------------------------------
HardwareInit :					;PROCEDURE HardwareInit ;
;*************					;BEGIN (* HardwareInit *)
	ldi	acc,Stand_TIMSK			; [TIMSK] := Stand_TIMSK ;
	out	TIMSK,acc			;
	ldi	acc,Stand_MCUCR			; [MCUCR] := Stand_MCUCR ;
	out	MCUCR,acc			;
	ldi	acc,Stand_TCCR0			; [TCCR0] := Stand_TCCR0 ;
	out	TCCR0,acc			;
	ldi	acc,Stand_TCCR1B		; [TCCR1B] := Stand_TCCR1B ;
	out	TCCR1B,acc			;
	ldz	USART_Speed			; USARTHardwareInit ( USART_Speed ) ;
	rcall	USARTHardwareInit		;
	ret					;END (* HardwareInit *) ;
;-----------------------------------------------------------------------------
EnvirInit :					;PROCEDURE EnvirInit ;
;**********					;BEGIN (* EnvirInit *)
	ldi	acc,Stand_PBDir			; [DDRB] := Stand_PBDir ;
	out	DDRB,Acc			;
	ldi	acc,Stand_PBData		; [PORTB] := Stand_PBData ;
	out	PORTB,acc			;
	ldi	acc,Stand_PDDir			; [DDRD] := Stand_PDDir ;
	out	DDRD,Acc			;
	ldi	acc,Stand_PDData		; [PORTD] := Stand_PDData ;
	out	PORTD,acc			;
	rcall	InitLCDEnvir			; InitLCDEnvir ( ) ;
	ret					;END (* EnvirInit *) ;
;-----------------------------------------------------------------------------
SoftwareInit :					;PROCEDURE SoftwareInit ;
;*************					;BEGIN (* SoftwareInit *)
	rcall	USARTSoftwareInit		; USARTSoftwareInit ( ) ;
	rcall	KeybSoftInit			; KeybSoftInit ( ) ;
	ret					;END (* SoftwareInit *) ;
;-----------------------------------------------------------------------------
DecodeKey :					;PROCEDURE DecodeKey ( ) ;
;**********					;VAR
;						; Kbd [acc2] : BYTE ;
;						; Temp [acc] : BYTE ;
						;BEGIN (* DecodeKey *)
	lds	acc2,KeybRAMPortAdr		; Kbd := KeyPInput ;
	ldi	acc,0				; Temp := 0 ;
	sbrs	acc2,Key0Pin			; IF Kbd[Key0Pin] THEN
	ldi	acc,Key0Code			;  Temp := Key0Code ;
	sbrs	acc2,Key1Pin			; IF Kbd[Key1Pin] THEN
	ldi	acc,Key1Code			;  Temp := Key1Code ;
	sbrs	acc2,Key2Pin			; IF Kbd[Key2Pin] THEN
	ldi	acc,Key2Code			;  Temp := Key2Code ;
	sbrs	acc2,Key3Pin			; IF Kbd[Key3Pin] THEN
	ldi	acc,Key3Code			;  Temp := Key3Code ;
	sbrs	acc2,Key4Pin			; IF Kbd[Key4Pin] THEN
	ldi	acc,Key4Code			;  Temp := Key4Code ;
	sbrs	acc2,Key5Pin			; IF Kbd[Key5Pin] THEN
	ldi	acc,Key5Code			;  Temp := Key5Code ;
	sbrs	acc2,Key6Pin			; IF Kbd[Key6Pin] THEN
	ldi	acc,Key6Code			;  Temp := Key6Code ;
	sbrs	acc2,Key7Pin			; IF Kbd[Key7Pin] THEN
	ldi	acc,Key7Code			;  Temp := Key7Code ;
	rcall	StoreKey			; StoreKey ( Temp ) ;
	ret					;END (* DecodeKey *) ;
;-----------------------------------------------------------------------------
AnyKeyActive :					;FUNCTION AnyKeyActive ( ) : BOOLEAN [C] ;
;*************					;BEGIN (* AnyKeyActive *)
	lds	acc,KeybRAMPortAdr		; IF KeyPort = 0xFF THEN
	cpi	acc,0xFF			;
	brne	AnKeAct_0			; BEGIN
	clc					;  AnyKeyActive := FALSE ;
	ret					;  RETURN ;
AnKeAct_0 :					; END (* IF *) ;
	sec					; AnyKeyActive := TRUE ;
	ret					;END (* AnyKeyActive *) ;
;-----------------------------------------------------------------------------
TC0OvfInterrupt :				;PROCEDURE TC0OvfInterrupt ;
;****************				;BEGIN (* TC0OvfInterrupt *)
	pushf					;
	pushz					;
	rcall	KeybService			; KeybService ( ) ;
	popz					;
	popf					;
	reti					;END (* TC0OvfInterrupt *) ;
;-----------------------------------------------------------------------------
ClearExtRam :					;PROCEDURE ClearExtRam ( ) ;
;****************				;VAR
;						; BPtr[z] : ^ BYTE ;
;						; Loop[y] : WORD ;
;						;BEGIN (* ClearExtRam *)
	ldz	ExtRAMOrgin			; BPtr := ExtRAMOrgin ;
	ldy	0				; FOR Loop := 0 TO ExtRamSize - 1 DO
	ldi	acc,0				;
ClXR_0	:					; BEGIN
	st	z+,acc				;  BPtr ^ := 0 ;
						;  BPtr := BPtr + 1 ;
	adiw	yl,1				; END (*& FOR *) ;
	cpi	yl,LOW(ExtRamSize)		;
	brne	ClXR_0				;
	cpi	yh,HIGH(ExtRamSize)		;
	brne	ClXR_0				;
	ret					;END (* ClearExtRam *) ;
;-----------------------------------------------------------------------------
ExtRAMTest :					;PROCEDURE ExtRAMTest ( ) : BOOLEAN ;
;****************				;VAR
;						; BPtr[z] : ^ BYTE ;
;						; xx[acc2] : BYTE ;
;						; Loop[y] : WORD ;
;						;BEGIN (* ExtRAMTest *)
	ldi	acc2,0				; xx := 0 ;
	ldz	ExtRAMOrgin			; BPtr := ExtRAMOrgin ;
	ldy	0				; Loop := 0 ;
ExtRT_0	:					; LOOP
	mov	acc,acc2			;  BPtr ^ := fajna_liczba ( xx ) ;
	com	acc				;
	swap	acc				;
	andi	acc,0xF0			;
	or	acc,acc2			;
	st	z+,acc				;
						;  BPtr := BPtr + 1 ;
	adiw	yl,1				;  Loop := Loop + 1 ;
	inc	acc2				;  xx := xx + 1 ;
	cpi	acc2,0x0D			;  IF xx = 13 THEN
	brne	ExtRT_1				;  BEGIN
	ldi	acc2,0				;   xx := 0 ;
ExtRT_1	:					;  END (* IF *) ;
	cpi	yl,LOW(ExtRamSize)		;  IF Loop = ExtRamSize THEN
						;  BEGIN
	brne	ExtRT_0				;   EXIT ;
	cpi	yh,HIGH(ExtRamSize)		;
	brne	ExtRT_0				;  END (* IF *) ;
						; END (* LOOP *) ;
	ldi	acc2,0				; xx := 0 ;
	ldz	ExtRAMOrgin			; BPtr := ExtRAMOrgin ;
	ldy	0				; Loop := 0 ;
ExtRT_2	:					; LOOP
	mov	acc,acc2			;  IF BPtr <> fajna_liczba ( liczba ) THEN
	com	acc				;
	swap	acc				;
	andi	acc,0xF0			;
	or	acc,acc2			;
	ld	param,z				;
	cp	param,acc			;  BEGIN
	breq	ExtRT_3				;
	ldy	RamErrorAdr			;   RamErrorAdr := BPtr ;
	st	y+,zl				;
	st	y,zh				;
	clc					;   ExtRAMTest := FALSE ;
	ret					;   RETURN ;
ExtRT_3	:					;  END (* IF *) ;
	adiw	zl,1				;
	adiw	yl,1				;  Loop := Loop + 1 ;
	inc	acc2				;  xx := xx + 1 ;
	cpi	acc2,0x0D			;  IF xx = 13 THEN
	brne	ExtRT_4				;  BEGIN
	ldi	acc2,0				;   xx := 0 ;
ExtRT_4	:					;  END (* IF *) ;
	cpi	yl,LOW(ExtRamSize)		;  IF Loop = ExtRamSize THEN
						;  BEGIN
	brne	ExtRT_2				;   EXIT ;
	cpi	yh,HIGH(ExtRamSize)		;
	brne	ExtRT_2				;  END (* IF *) ;
						; END (* LOOP *) ;
	sec					; ExtRAMTest := TRUE ;
	ret					;END (* ExtRAMTest *) ;
;-----------------------------------------------------------------------------
RAMErrorMessage :				;PROCEDURE RAMErrorMessage ( ) ;
;****************				;VAR
;						; XCh[x] : ARRAY [ 0 .. 1 ] OF CHAR ;
;						;BEGIN (* RAMErrorMessage *)
	ldz	RAMError<<1			; WriteTextLCDFlash ( RAMError ) ;
	rcall	WriteTextLCDFlash		;
	lds	param,RamErrorAdr+1		; XCh := ByteToHex ( RamErrorAdr DIV 256 ) ;
	rcall	ByteToHex			;
	push	xl				; WriteChLCD ( XCh [ 0 ] ) ;
	mov	param,xh			;
	rcall	WriteChLCD			;
	pop	param				; WriteChLCD ( XCh [ 1 ] ) ;
	rcall	WriteChLCD			;
	lds	param,RamErrorAdr		; XCh := ByteToHex ( RamErrorAdr MOD 256 ) ;
	rcall	ByteToHex			;
	push	xl				; WriteChLCD ( XCh [ 0 ] ) ;
	mov	param,xh			;
	rcall	WriteChLCD			;
	pop	param				; WriteChLCD ( XCh [ 1 ] ) ;
	rcall	WriteChLCD			;
RAMErM_0 :					; STOP ;
	rjmp	RAMErM_0			;
						;END (* RAMErrorMessage *) ;
;-----------------------------------------------------------------------------
GetNextData :					;FUNCTION GetNextData ( ) : BOOLEAN ;
;************					;VAR
;						;BEGIN (* GetNextData *)
	ldz	NewData				; NewData := [TCNT1] ;
	in	acc,TCNT1L			;
	st	z+,acc				;
	in	acc,TCNT1H			;
	st	z,acc				;
	arrelem	DataTable,DataTableInx		; DataTable [ DataTableInx ] := NewData ;
	putarr	NewData				;
	incword	DataTableInx			; DataTableInx := DataTableInx + 1 ;
	rcall	ClrScrLCD			; ClrScrLCD ( ) ;
	ldi	param,'['			; WriteChLCD ( '[' ) ;
	rcall	WriteChLCD			;
	ldz	DataTableInx			; WordToString ( ADR ( DataTableInx ) , Number , 5 , ' ' ) ;
	ldx	Number				;
	ldi	param,5				;
	ldi	acc,' '				;
	rcall	WordToString			;
	ldz	Number				; WriteTextLCD ( Number ) ;
	rcall	WriteTextLCD			;
	ldi	param,']'			; WriteChLCD ( ']' ) ;
	rcall	WriteChLCD			;
	ldi	param,'='			; WriteChLCD ( '=' ) ;
	rcall	WriteChLCD			;
	ldz	NewData				; WordToString ( ADR ( NewData ) , Number , 5 , '0' ) ;
	ldx	Number				;
	ldi	param,5				;
	ldi	acc,'0'				;
	rcall	WordToString			;
	ldz	Number				; WriteTextLCD ( Number ) ;
	rcall	WriteTextLCD			;
	ldz	DataTableInx			; IF DataTableInx = TableSize THEN
	ld	acc,z+				;
	cpi	acc,low(TableSize)		;
	ld	acc,z				;
	ldi	acc2,high(TableSize)		;
	cpc	acc,acc2			;
	brne	GeNeDa_0			; BEGIN
	clc					;  GetNextData := FALSE ;
	ret					;  RETURN ;
GeNeDa_0 :					; END (* IF *) ;
	sec					; GetNextData := TRUE ;
	ret					;END (* GetNextData *) ;
;-----------------------------------------------------------------------------
GetData	:					;PROCEDURE GetData ( ) ;
;********					;VAR
;						; Ch[acc] : CHAR ;
;						;BEGIN (* GetData *)
	rcall	ClrScrLCD			; ClrScrLCD ( ) ;
	clrword	DataTableInx			; DataTableInx := 0 ;
GeDa_0	:					; LOOP
	rcall	GetNextData			;  IF NOT GetNextData ( ) THEN
	brcs	GeDa_5				;  BEGIN
	ret					;   RETURN ;
GeDa_5	:					;  END (* IF *) ;
	rcall	LongDelay			;  LongDelay ( ) ;
	rcall	LongDelay			;  LongDelay ( ) ;
	rcall	LongDelay			;  LongDelay ( ) ;
	rcall	LongDelay			;  LongDelay ( ) ;
	rcall	KeyPressed			;  IF KeyPressed ( ) THEN
	brcc	GeDa_1				;  BEGIN
	rcall	ReadKey				;   ? := ReadKey ( ) ;
	ret					;   RETURN ;
GeDa_1	:					;  END (* IF *) ;
	rjmp	GeDa_0				; END (* LOOP *) ;
	ret					;END (* GetData *) ;
;-----------------------------------------------------------------------------
DisplayMessage :				;PROCEDURE DisplayMessage ( MsgText[z] : ARRAY OF CHAR ) ;
;***************				;BEGIN (* DisplayMessage *)
	pushz					;
	rcall	ClrScrLCD			; ClrScrLCD ( ) ;
	popz					; WriteTextLCDFlash ( MsgText ) ;
	rcall	WriteTextLCDFlash		;
	rcall	LongDelay			; LongDelay ( ) ;
	rcall	LongDelay			; LongDelay ( ) ;
	rcall	LongDelay			; LongDelay ( ) ;
	rcall	LongDelay			; LongDelay ( ) ;
	rcall	LongDelay			; LongDelay ( ) ;
	rcall	LongDelay			; LongDelay ( ) ;
	rcall	LongDelay			; LongDelay ( ) ;
	rcall	LongDelay			; LongDelay ( ) ;
	ret					;END (* DisplayMessage *) ;
;-----------------------------------------------------------------------------
NewLine	:					;PROCEDURE NewLine ( ) ;
;********					;BEGIN (* NewLine *)
	ldi	param,Cr			; SendSerial ( Cr ) ;
	rcall	SendSerial			;
	ldi	param,Lf			; SendSerial ( Lf ) ;
	rcall	SendSerial			;
	ret					;END (* NewLine *) ;
;-----------------------------------------------------------------------------
WaitOnF7Key :					;PROCEDURE WaitOnF7Key ( ) ;
;************					;BEGIN (* WaitOnF7Key *)
WaONKe_0 :					; LOOP
WaONKe_1 :					;  REPEAT
	rcall	KeyPressed			;  UNTIL KeyPressed ( ) ;
	brcc	WaONKe_1			;
	rcall	ReadKey				;  IF ReadKey ( ) = Key7Code THEN
	cpi	acc,Key7Code			;
	brne	WaONKe_2			;  BEGIN
	ret					;   RETURN ;
WaONKe_2 :					;  END (* IF *) ;
	rjmp	WaONKe_0			; END (* LOOP *) ;
	ret					;END (* WaitOnF7Key *) ;
;-----------------------------------------------------------------------------
WaitOnAccKey :					;FUNCTION WaitOnAccKey ( ) : BOOLEAN ;
;************					;VAR
;						; Ch[acc] : CHAR ;
;						;BEGIN (* WaitOnAccKey *)
WaONAccKe_0 :					; LOOP
WaONAccKe_1 :					;  REPEAT
	rcall	KeyPressed			;  UNTIL KeyPressed ( ) ;
	brcc	WaONAccKe_1			;
	rcall	ReadKey				;  Ch := ReadKey ( );
	cpi	acc,Key7Code			;  IF Ch = Key7Code THEN
	brne	WaONAccKe_2			;  BEGIN
	clc					;   WaitOnAccKey := FALSE ;
	ret					;   RETURN ;
WaONAccKe_2 :					;  END (* IF *) ;
	cpi	acc,Key6Code			;  IF Ch = Key6Code THEN
	brne	WaONAccKe_3			;  BEGIN
	sec					;   WaitOnAccKey := TRUE ;
	ret					;   RETURN ;
WaONAccKe_3 :					;  END (* IF *) ;
	rjmp	WaONAccKe_0			; END (* LOOP *) ;
	ret					;END (* WaitOnAccKey *) ;
;-----------------------------------------------------------------------------
DisplayMinMax :					;PROCEDURE DisplayMinMax ( ) ;
;**************					;BEGIN (* DisplayMinMax *)
	ldi	param,'['			; WriteChLCD ( '[' ) ;
	rcall	WriteChLCD			;
	ldz	MinMaxInx			; WordToString ( ADR ( MinMaxInx ) , Number , 3 , ' ' ) ;
	ldx	Number				;
	ldi	param,3				;
	ldi	acc,' '				;
	rcall	WordToString			;
	ldz	Number				; WriteTextLCD ( Number ) ;
	rcall	WriteTextLCD			;
	ldi	param,']'			; WriteChLCD ( ']' ) ;
	rcall	WriteChLCD			;
	ldi	param,'='			; WriteChLCD ( '=' ) ;
	rcall	WriteChLCD			;
	ldz	MinMaxVal			; WordToString ( ADR ( MinMaxVal ) , Number , 5 , ' ' ) ;
	ldx	Number				;
	ldi	param,5				;
	ldi	acc,' '				;
	rcall	WordToString			;
	ldz	Number				; WriteTextLCD ( Number ) ;
	rcall	WriteTextLCD			;
	ret					;END (* DisplayMinMax *) ;
;-----------------------------------------------------------------------------
DisplayMin :					;PROCEDURE DisplayMin ( ) ;
;***********					;BEGIN (* DisplayMin *)
	rcall	ClrScrLCD			; ClrScrLCD ( ) ;
	ldz	MinText<<1			; WriteTextLCDFlash ( MinText ) ;
	rcall	WriteTextLCDFlash		;
	rcall	DisplayMinMax			; DisplayMinMax ( ) ;
	rcall	WaitOnF7Key			; WaitOnF7Key ( ) ;
	ret					;END (* DisplayMin *) ;
;-----------------------------------------------------------------------------
DisplayMax :					;PROCEDURE DisplayMax ( ) ;
;***********					;BEGIN (* DisplayMax *)
	rcall	ClrScrLCD			; ClrScrLCD ( ) ;
	ldz	MaxText<<1			; WriteTextLCDFlash ( MaxText ) ;
	rcall	WriteTextLCDFlash		;
	rcall	DisplayMinMax			; DisplayMinMax ( ) ;
	rcall	WaitOnF7Key			; WaitOnF7Key ( ) ;
	ret					;END (* DisplayMax *) ;
;-----------------------------------------------------------------------------
DataTableValid :				;FUNCTION DataTableValid ( ) : BOOLEAN ;
;***************				;BEGIN (* DataTableValid *)
	ldz	DataTableInx			; IF DataTableInx = 0 THEN
	ld	acc,z+				;
	ld	acc2,z				;
	or	acc,acc2			;
	brne	DaTaVa_0			; BEGIN
	ldz	NoDataMessage<<1		;  DisplayMessage ( NoDataMessage ) ;
	rcall	DisplayMessage			;
	clc					;  DataTableValid := FALSE ;
	ret					;  RETURN ;
DaTaVa_0 :					; END (* IF *) ;
	sec					; DataTableValid := TRUE ;
	ret					;END (* DataTableValid *) ;
;-----------------------------------------------------------------------------
SerialDispDataTable :				;PROCEDURE SerialDispDataTable ( ) ;
;********************				;BEGIN (* SerialDispDataTable *)
	rcall	DataTableValid			; IF NOT DataTableValid ( ) THEN
	brcs	SeDiDT_0			; BEGIN
	ret					;  RETURN ;
SeDiDT_0 :					; END (* IF *) ;
	rcall	NewLine				; NewLine ( ) ;
	ldz	DataCntText<<1			; SendSerialFlashTxt ( DataCntText ) ;
	rcall	SendSerialFlashTxt		;
	ldz	DataTableInx			; WordToString ( ADR ( DataTableInx ) , Number , 5 , ' ' ) ;
	ldx	Number				;
	ldi	param,5				;
	ldi	acc,' '				;
	rcall	WordToString			;
	ldz	Number				; SendSerialRAMTxt ( Number ) ;
	rcall	SendSerialRAMTxt		;
	rcall	NewLine				; NewLine ( ) ;
	clrword	Loop				; FOR Loop := 0 TO DataTableInx - 1 DO
SeDiDT_1 :					; BEGIN
	ldi	param,'['			;  SendSerial ( '[' ) ;
	rcall	SendSerial			;
	ldz	Loop				;  WordToString ( ADR ( Loop ) , Number , 3 , ' ' ) ;
	ldx	Number				;
	ldi	param,3				;
	ldi	acc,' '				;
	rcall	WordToString			;
	ldz	Number				;  SendSerialRAMTxt ( Number ) ;
	rcall	SendSerialRAMTxt		;
	ldi	param,']'			;  SendSerial ( ']' ) ;
	rcall	SendSerial			;
	ldi	param,'='			;  SendSerial ( '=' ) ;
	rcall	SendSerial			;
	arrelem	DataTable,Loop			;  WordToString ( ADR ( DataTable [ Loop ] ) , Number , 5 , ' ' ) ;
	ldx	Number				;
	ldi	param,5				;
	ldi	acc,' '				;
	rcall	WordToString			;
	ldz	Number				;  SendSerialRAMTxt ( Number ) ;
	rcall	SendSerialRAMTxt		;
	rcall	NewLine				;  NewLine ( ) ;
	incword	Loop				; END (* FOR *) ;
	clfor	Loop,DataTableInx,SeDiDT_2,SeDiDT_1
SeDiDT_2 :					;
	ret					;END (* SerialDispDataTable *) ;
;-----------------------------------------------------------------------------
LCDDispDataTable :				;PROCEDURE LCDDispDataTable ( ) ;
;*****************				;BEGIN (* LCDDispDataTable *)
	rcall	DataTableValid			; IF NOT DataTableValid ( ) THEN
	brcs	LCDDiDT_0			; BEGIN
	ret					;  RETURN ;
LCDDiDT_0 :					; END (* IF *) ;
	rcall	ClrScrLCD			; ClrScrLCD ( ) ;
	ldz	LCDDataCntText<<1		; WriteTextLCDFlash ( LCDDataCntText ) ;
	rcall	WriteTextLCDFlash		;
	ldz	DataTableInx			; WordToString ( ADR ( DataTableInx ) , Number , 4 , ' ' ) ;
	ldx	Number				;
	ldi	param,4				;
	ldi	acc,' '				;
	rcall	WordToString			;
	ldz	Number				; WriteTextLCD ( Number ) ;
	rcall	WriteTextLCD			;
	rcall	WaitOnAccKey			; IF NOT WaitOnAccKey ( ) THEN
	brcs	LCDDiDT_3			; BEGIN
	ret					;  RETURN ;
LCDDiDT_3 :					; END (* IF *) ;
	clrword	Loop				; FOR Loop := 0 TO DataTableInx - 1 DO
LCDDiDT_1 :					; BEGIN
	rcall	ClrScrLCD			;  ClrScrLCD ( ) ;
	ldi	param,'['			;  WriteChLCD ( '[' ) ;
	rcall	WriteChLCD			;
	ldz	Loop				;  WordToString ( ADR ( Loop ) , Number , 3 , ' ' ) ;
	ldx	Number				;
	ldi	param,3				;
	ldi	acc,' '				;
	rcall	WordToString			;
	ldz	Number				;  WriteTextLCD ( Number ) ;
	rcall	WriteTextLCD			;
	ldi	param,']'			;  WriteChLCD ( ']' ) ;
	rcall	WriteChLCD			;
	ldi	param,'='			;  WriteChLCD ( '=' ) ;
	rcall	WriteChLCD			;
	arrelem	DataTable,Loop			;  WordToString ( ADR ( DataTable [ Loop ] ) , Number , 5 , ' ' ) ;
	ldx	Number				;
	ldi	param,5				;
	ldi	acc,' '				;
	rcall	WordToString			;
	ldz	Number				;  WriteTextLCD ( Number ) ;
	rcall	WriteTextLCD			;
	rcall	WaitOnAccKey			;  IF NOT WaitOnAccKey ( ) THEN
	brcs	LCDDiDT_4			;  BEGIN
	ret					;   RETURN ;
LCDDiDT_4 :					;  END (* IF *) ;
	incword	Loop				; END (* FOR *) ;
	clfor	Loop,DataTableInx,LCDDiDT_2,LCDDiDT_1
LCDDiDT_2 :					;
	ret					;END (* LCDDispDataTable *) ;
;-----------------------------------------------------------------------------
StartMinMax :					;PROCEDURE StartMinMax ( ) ;
;************					;BEGIN (* StartMinMax *)
	ldz	DataTable			; MinMaxVal := DataTable [ 0 ] ;
	getarr	MinMaxVal			;
	clrword	MinMaxInx			; MinMaxInx := 0 ;
	ret					;END (* StartMinMax *) ;
;-----------------------------------------------------------------------------
SearchMin :					;PROCEDURE SearchMin ( ) ;
;**********					;BEGIN (* SearchMin *)
	rcall	DataTableValid			; IF NOT DataTableValid ( ) THEN
	brcs	SeMin_0				; BEGIN
	ret					;  RETURN ;
SeMin_0 :					; END (* IF *) ;
	rcall	StartMinMax			; StartMinMax ( ) ;
	clrword	Loop				; FOR Loop := 0 TO DataTableInx - 1 DO
SeMin_1 :					; BEGIN
	arrelem	DataTable,Loop			;  IF DataTable [ Loop ] <= MinMaxVal THEN
	ldx	MinMaxVal			;
	jmpxgtz	SeMin_3				;  BEGIN
	arrelem	DataTable,Loop			;   MinMaxVal := DataTable [ Loop ] ;
	getarr	MinMaxVal			;
	mvword	MinMaxInx,Loop			;   MinMaxInx := Loop ;
SeMin_3	:					;  END (* IF *) ;
	incword	Loop				; END (* FOR *) ;
	clfor	Loop,DataTableInx,SeMin_2,SeMin_1
SeMin_2 :					;
	rcall	DisplayMin			; DisplayMin ( ) ;
	ret					;END (* SearchMin *) ;
;-----------------------------------------------------------------------------
SearchMax :					;PROCEDURE SearchMax ( ) ;
;**********					;BEGIN (* SearchMax *)
	rcall	DataTableValid			; IF NOT DataTableValid ( ) THEN
	brcs	SeMax_0				; BEGIN
	ret					;  RETURN ;
SeMax_0 :					; END (* IF *) ;
	rcall	StartMinMax			; StartMinMax ( ) ;
	clrword	Loop				; FOR Loop := 0 TO DataTableInx - 1 DO
SeMax_1 :					; BEGIN
	arrelem	DataTable,Loop			;  IF DataTable [ Loop ] >= MinMaxVal THEN
	ldx	MinMaxVal			;
	jmpxltz	SeMax_3				;  BEGIN
	arrelem	DataTable,Loop			;   MinMaxVal := DataTable [ Loop ] ;
	getarr	MinMaxVal			;
	mvword	MinMaxInx,Loop			;   MinMaxInx := Loop ;
SeMax_3	:					;  END (* IF *) ;
	incword	Loop				; END (* FOR *) ;
	clfor	Loop,DataTableInx,SeMax_2,SeMax_1
SeMax_2 :					;
	rcall	DisplayMax			; DisplayMax ( ) ;
	ret					;END (* SearchMax *) ;
;-----------------------------------------------------------------------------
SortData :					;PROCEDURE SortData ( ) ;
;*********					;BEGIN (* SortData *)
	rcall	DataTableValid			; IF NOT DataTableValid ( ) THEN
	brcs	Sort_0				; BEGIN
	ret					;  RETURN ;
Sort_0	:					; END (* IF *) ;
	ldz	DataTableInx			; IF DataTableInx = 1 THEN
	ld	acc,z+				;
	cpi	acc,1				;
	ld	acc,z				;
	ldi	acc2,0				;
	cpc	acc,acc2			;
	brne	Sort_1				; BEGIN
	ldz	SortReadyTxt<<1			;  DisplayMessage ( SortReadyTxt ) ;
	rcall	DisplayMessage			;
	ret					;
Sort_1	:					; END (* IF *) ;
	ldz	SortingTxt<<1			; DisplayMessage ( SortingTxt ) ;
	rcall	DisplayMessage			;
	clrword	StartLoop			; FOR StartLoop := 0 TO DataTableInx - 1 DO
Sort_3	:					; BEGIN
	arrelem	DataTable,StartLoop		;  MinMaxVal := DataTable [ StartLoop ] ;
	getarr	MinMaxVal			;
	mvword	MinMaxInx,StartLoop		;  MinMaxInx := StartLoop ;
	mvword	Loop,StartLoop			;  FOR Loop := StartLoop TO DataTableInx - 1 DO
Sort_4	:					;  BEGIN
	arrelem	DataTable,Loop			;   IF MinMaxVal >= DataTable [ Loop ] THEN
	ldx	MinMaxVal			;
	jmpxltz	Sort_5				;   BEGIN
	arrelem	DataTable,Loop			;    MinMaxVal := DataTable [ Loop ] ;
	getarr	MinMaxVal			;
	mvword	MinMaxInx,Loop			;    MinMaxInx := Loop ;
Sort_5	:					;   END (* IF *) ;
	incword	Loop				;  END (* FOR *) ;
	clfor	Loop,DataTableInx,Sort_6,Sort_4	;
Sort_6	:					;
	ldz	MinMaxInx			;  IF MinMaxInx <> StartLoop THEN
	ldx	StartLoop			;
	jmpxeqz	Sort_7				;  BEGIN
	arrelem	DataTable,StartLoop		;   NewData := DataTable [ StartLoop ] ;
	getarr	NewData				;
	arrelem	DataTable,MinMaxInx		;   DataTable [ StartLoop ] := DataTable [ MinMaxInx ] ;
	mov	xl,zl				;
	mov	xh,zh				;
	arrelem	DataTable,StartLoop		;
	ld	acc,x+				;
	st	z+,acc				;
	ld	acc,x				;
	st	z,acc				;
	arrelem	DataTable,MinMaxInx		;   DataTable [ MinMaxInx ] := NewData ;
	putarr	NewData				;
Sort_7	:					;  END (* IF *) ;
	incword	StartLoop			; END (* FOR *) ;
	clfor	StartLoop,DataTableInx,Sort_2,Sort_3
Sort_2	:					;
	ret					;END (* SortData *) ;
;-----------------------------------------------------------------------------
SumData	:					;PROCEDURE SumData ( ) ;
;********					;BEGIN (* SumData *)
	rcall	DataTableValid			; IF NOT DataTableValid ( ) THEN
	brcs	Sum_0				; BEGIN
	ret					;  RETURN ;
Sum_0	:					; END (* IF *) ;
	ldi	acc,0				; Argument1 := 0 ;
	mov	Argument1LL,acc			;
	mov	Argument1LH,acc			;
	mov	Argument1HL,acc			;
	mov	Argument1HH,acc			;
	clrword	Loop				; FOR Loop := 0 TO DataTableInx - 1 DO
Sum_1 	:					; BEGIN
	arrelem	DataTable,Loop			;  Argument1 := Argument1 + DataTable [ Loop ] ;
	ld	acc,z+				;
	ld	acc2,z				;
	add	Argument1LL,acc			;
	adc	Argument1LH,acc2		;
	ldi	acc,0				;
	adc	Argument1HL,acc			;
	adc	Argument1HH,acc			;
	incword	Loop				; END (* FOR *) ;
	clfor	Loop,DataTableInx,Sum_2,Sum_1	;
Sum_2 	:					;
	ldz	DataTableInx			; Argument2 := DataTableInx ;
	ld	Argument2LL,z+			;
	ld	Argument2LH,z			;
	ldi	acc,0				;
	mov	Argument2HL,acc			;
	mov	Argument2HH,acc			;
	rcall	DivideArg1ByArg2Round		; Sum := Argument1 / Argument2 ;
	ldz	Sum				;
	st	z+,ResultLL			;
	st	z,ResultLH			;
	ldz	SumTxt<<1			; DisplayMessage ( SumTxt ) ;
	rcall	DisplayMessage			;
	ldz	Sum				; WordToString ( ADR ( Sum ) , Number , 5 , '0' ) ;
	ldx	Number				;
	ldi	param,5				;
	ldi	acc,'0'				;
	rcall	WordToString			;
	ldz	Number				; WriteTextLCD ( Number ) ;
	rcall	WriteTextLCD			;
	rcall	WaitOnF7Key			; WaitOnF7Key ( ) ;
	ret					;END (* SumData *) ;
;-----------------------------------------------------------------------------
RunCommand :					;PROCEDURE RunCommand ( ) ;
;***********					;BEGIN (* RunCommand *)
	rcall	ReadKey				; CASE ReadKey ( ) OF
	cpi	acc,Key0Code			; | Key0Code :
	brne	RuCo_0				;   BEGIN
	rcall	GetData				;    GetData ( ) ;
	ret					;    RETURN ;
RuCo_0	:					;   END (* CASE VARIANT *) ;
	cpi	acc,Key1Code			; | Key1Code :
	brne	RuCo_1				;   BEGIN
	rcall	SerialDispDataTable		;    SerialDispDataTable ( ) ;
	ret					;    RETURN ;
RuCo_1	:					;   END (* CASE VARIANT *) ;
	cpi	acc,Key2Code			; | Key2Code :
	brne	RuCo_2				;   BEGIN
	rcall	LCDDispDataTable		;    LCDDispDataTable ( ) ;
	ret					;    RETURN ;
RuCo_2	:					;   END (* CASE VARIANT *) ;
	cpi	acc,Key3Code			; | Key3Code :
	brne	RuCo_3				;   BEGIN
	rcall	SearchMin			;    SearchMin ( ) ;
	ret					;    RETURN ;
RuCo_3	:					;   END (* CASE VARIANT *) ;
	cpi	acc,Key4Code			; | Key4Code :
	brne	RuCo_4				;   BEGIN
	rcall	SearchMax			;    SearchMax ( ) ;
	ret					;    RETURN ;
RuCo_4	:					;   END (* CASE VARIANT *) ;
	cpi	acc,Key5Code			; | Key5Code :
	brne	RuCo_5				;   BEGIN
	rcall	SortData			;    SortData ( ) ;
	ret					;    RETURN ;
RuCo_5	:					;   END (* CASE VARIANT *) ;
	cpi	acc,Key6Code			; | Key6Code :
	brne	RuCo_6				;   BEGIN
	rcall	SumData				;    SumData ( ) ;
	ret					;    RETURN ;
RuCo_6	:					;   END (* CASE VARIANT *) ;
						; END (* CASE *) ;
	ldz	UnknownOption<<1		; DisplayMessage ( UnknownOption ) ;
	rcall	DisplayMessage			;
	ret					;END (* RunCommand *) ;
;-----------------------------------------------------------------------------
ResetProcessor :				;
	ldi	acc,HIGH(RAMEND)		;
	out	SPH,acc				;
	ldi	acc,LOW(RAMEND)			;
	out	SPL,acc				;
						;BEGIN (* Main *)
	rcall	ClearRAM			; ClearRAM ( ) ;
	rcall	ClearRegs			; ClearRegs ( ) ;
	rcall	HardwareInit			; HardwareInit ( ) ;
	rcall	EnvirInit			; EnvirInit ( ) ;
	rcall	SoftwareInit			; SoftwareInit ( ) ;
	rcall	LongDelay			; LongDelay ( ) ;
	rcall	InitialiseLCD			; InitialiseLCD ( ) ;
	sei					; EI ( ) ;
	rcall	ExtRAMTest			; IF ExtRAMTest ( ) <> NIL THEN
	brcs	Main_1				; BEGIN
	rcall	RAMErrorMessage			;  RAMErrorMessage ( ) ;
Main_1	:					; END (* IF *) ;
	ldz	RAMOK<<1			; WriteTextLCDFlash ( RAMOK ) ;
	rcall	WriteTextLCDFlash		;
	rcall	ClearExtRam			; ClearExtRam ( ) ;
	rcall	LongDelay			; LongDelay ( ) ;
Main_0	:					; LOOP
	rcall	ClrScrLCD			;  ClrScrLCD ( ) ;
	ldz	SelectOption<<1			;  WriteTextLCDFlash ( SelectOption ) ;
	rcall	WriteTextLCDFlash		;
Main_2	:					;  REPEAT
	rcall	KeyPressed			;  UNTIL KeyPressed ( ) ;
	brcc	Main_2				;
	rcall	RunCommand			;  RunCommand ( ) ;
	rjmp	Main_0				; END (* LOOP *) ;
						;END (* Main *) ;
;-----------------------------------------------------------------------------
RAMError:	.db	"Blad RAM: ",0,0
RAMOK:		.db	"Pamiec RAM: OK",0,0
SelectOption:	.db	"Wybierz opcje",0
NoDataMessage:	.db	"Brak danych",0
UnknownOption:	.db	"???",0
DataCntText:	.db	"Liczba elementow w tablicy : ",0
LCDDataCntText:	.db	"L.elemet.=",0,0
SortReadyTxt:	.db	"Nie wymaga sort.",0,0
SortingTxt:	.db	"Sortowanie",0,0
MinText:	.db	"Min ",0,0
MaxText:	.db	"Max ",0,0
SumTxt:		.db	"Srednia ",0,0
;-----------------------------------------------------------------------------
.exit
